#!/opt/perl/bin/perl
use GPS::Orbits;
use GPS::DATES;
use File::Basename;
use Log::Log4perl qw(:easy);
use Getopt::Long;
my $progname = basename $0;
my $cache_age = 7; #days
my $purge=0;
my $help='';
my $type = ''; 
my $date = '';
my $ftp=1;
my $noftp=0;
my $ac = 'jgp';
my $del_sat='';
my $t1=undef;
my $t2=undef;
my $frame='igs08';

Log::Log4perl->easy_init({level => $INFO,
		          category => '',
		          file => "get_orbits_pl.log"},
		         {level => $INFO, 
		          category => 'GPS::Orbits'});
		          
my $args = @ARGV;
my $usage = qq(

 $progname -type [final|rapid] -date date -ac [igs|jgp|code] [-noftp] [-purge]

);
GetOptions ( "type:s" => \$type,
             "date:s" => \$date,
             "ac:s"   => \$ac,
             "del_sat:s" => \$del_sat,
             "noftp"  => \$noftp,
             "purge"  => \$purge,
             't1:s'   => \$t1,
             't2:s'   => \$t2,
             'frame:s' => \$frame,
             "help"   => \$help,
             ) or die "$progname : Error in optionsn";
if ( ! $args || $help ) {
  die $usage;
}
if ( $ac !~ /^(igs|ig1|jgp|code)$/i ) {
  die "Invalid 'ac' option \n$usage";
}
if ( ! $date ) {
  die "No date \n$usage";
}
$ftp = 0 if $noftp;
$ac = lc $ac;

if ( $type =~ /ultra_(00|06|12|18)/ ) {  $hh=$1; }

$type = $ac."_".$type;
$cache_age = 0 if $purge;

my %opt= ( type => $type,
           date => $date,
           cache_age=> $cache_age,
           del_sat => $del_sat,
           frame => $frame
         );
if ( $t1 ) {
  $opt{t1}=$t1;
}                         

if ( $t2 ) {
  $opt{t2}=$t2;
}             


my $ok = 0;
my $o = GPS::Orbits->new(%opt);

  # Do subroutine named "clear_cache", this has an implicit
  # variable named "cache_age" that determines whether to actually clear the cache
  # when not provided, it defaults to the value provided at instantiation
  $o->clear_cache(); 

  # Delete the existing version of the orbit files
  unlink @{$o->{goa_files}};

  INFO("Copying from cache"); 
  my $copied = $o->copy_from_cache();
  $ok = $copied;

  INFO("Tried to copy from cache");
  # If we are not copying from the cache 
  if ( ! $copied ) {
    INFO("Trying to copy from archive");
    # Well then, are we copying from the archive?
    $copied = $o->copy_from_archive();

    # No? Ok, lets FTP them
    if ( ! $copied ) {
      if ( $ftp ) {
        my $ftped = $o->ftp();
	if (! $ftped ) { print "Unable to FTP orbits\n"; $ok = 1;}
        my $saved = $o->save2archive(); 
	if (! $saved ) { print "Unable to save orbits\n"; $ok = 1;}
      } else {
        $ok = 1;
        print "No orbits downloaded or copied\n";
      }
      if ($ok == 1) { exit $ok; }
    }

    # Lets convert the orbit files to a format more conducive to processing
    if ( $o->type =~ /^jgp/ ) {
     $ok = $o->jgp2goa_files();
    }
    elsif ( $o->type =~/^ig[s1]/ ) {
      $ok = $o->igs2goa_files();
    }
    elsif ( $o->type =~/^code/ ) {
      $ok = $o->code2goa_files();
    }
   
    # Copy the orbits to the orbit cache 
    $ok = $o->save2cache();

  }

  # Remove the original orbit products 
 unlink map { basename $_ } $o->products;

 foreach my $f ( @{$o->{goa_files}} ) {
   if (! -e $f) {
      $ok = 1;
      ERROR("$f not found for date $date\n"); 
   } else {
     INFO("Retrieved $f for date $date\n");
   }
 }

my $exit_status = 0;
if (! $ok) {
	$exit_status = 1;
}
exit $exit_status;


