#!/usr/bin/env python

import numpy

def read(filename):
    fp = open(filename,'r')

    tdpentries = {} 
    for line in fp.readlines():
        items = line.split()
        epoch = float(items[0])
        value = float(items[2])
        name = items[4]
        if (name not in tdpentries):
            tdpentries[name] = []
        tdpentries[name].append({'epoch': epoch, 'value':value})

    fp.close()

    return tdpentries

def main(filename,t0,site):
    # Read TDP file
    entries = read(filename)
   
    # Figure out the actual component name, it starts with DRYZTROP and ends with
    # the site name
    componentname = [x for x in entries.keys() if (x == 'DRYZTROP{0}'.format(site.upper()))][0]

    # Extract the time series for this component
    t = [x['epoch'] for x in entries[componentname]]
    y = [x['value'] for x in entries[componentname]]

    # Check boundaries
    if (t0 < min(t) or t0 > max(t)):
        print "FATAL, t0 = {0} is outside of bounds [{1}:{2}]".format(t0,min(t),max(t))
        return
    p = numpy.polyfit(t,y,2)

    y0 = numpy.polyval(p,t0)
    print t0,y0


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description='Interpolate DRYZ from a TDP file')
    parser.add_argument('--input','-i',required=True,help='Input file')
    parser.add_argument('-t',type=float,required=True,help='epoch for interpolation')
    parser.add_argument('--site','-s',required=True,help='site name')

    args = parser.parse_args()

    main(args.input,args.t,args.site)
