#!/opt/perl/bin/perl
use strict;
use warnings;
use File::Basename;
use GOA::Util;
use GPS::DATES;
use GOA::QRegres;
use GPS::Defaults;
use Getopt::Long;

my $site ='';
my $date = '';
my $rate = 300;
#my $tides = 'PolTid,FreqDepLove,OctTid,OcnldCpn';
my $tides = 'WahrK1,PolTid,ocl_n'; #,FreqDepLove,OctTid,OcnldCpn';
my $help ='';
my $args = @ARGV;
my $nml='';
my $min_partials = '';
my $qmfile = '';
my $ocean_loading = 1;
my $ionex_dir = '';
my $ac = '';
my $vmf1=undef;
my $tropo_apriori = 'default';
my $trop_map="NIELL";
my @vmf1_args=();

GetOptions(  'site:s' => \$site,
             'date:s' => \$date,
             'tides:s' => \$tides,
             'ocean_loading:s' => \$ocean_loading,
	     'ionex_dir:s' => \$ionex_dir,
             'rate:i'  => \$rate,
             'ac:s'    => \$ac,
             'mp'       => \$min_partials,
             'vmf1'     => \$vmf1,
             'qm:s'     => \$qmfile,
             'nml:s'    => \$nml,
             'tropo0:s'  => \$tropo_apriori, # Should be 'default' or 'gpt'
             'help'    => \$help,
               
 ) or die "Error in options\n";
if ( !$args || $help ) {
  die usage();
}

if ( -e $qmfile ) {
  $site     =  join ',', 
                map { ( split(' ',$_) )[1] } 
                grep { /start/ } 
               `qm_sum $qmfile 2> /dev/null`;

}

if ( ! $site && ! $nml ) {
   print STDERR "No site supplied\n";
   die usage();
}
if (  ! $date  && ! $nml ) {
   print STDERR "No date !\n";
   die usage();
}

unless ($tropo_apriori ~~ [qw(default gpt vmf1)]) {
	die "Unknown apriori tropospheric model $tropo_apriori\n";
}


my $qr ='';
my %def = get_defaults( env => \%ENV );
my $cal='';

my $ant_cal = -e 'GPS_antcal_ref' ? 'GPS_antcal_ref' : 
               die "Couldn't find GPS_antcal_ref\n";

my ($grn_wwww,$sat_wwww) = get_wwww($ant_cal);
if ( $vmf1 ) {
 $trop_map = 'VMF1GRID';
 @vmf1_args = ( '-vmf1dir' , $def{VMF1GRID_DIR} ); 
}
if ( ! $nml ) {
 $cal = get_date($date,'cal');
 $qr  =  qr_nml( sta          => $site,
                 date         => $cal,
                 sta_info_dir => $def{STA_INFO_DIR},
                 rate         => $rate,
                 tides        => $tides,
                 ocean_loading => $ocean_loading,
                 min_set_partials => $min_partials,
                 trop_map => $trop_map,
                 grn_wwww => $grn_wwww,
                 sat_wwww => $sat_wwww,
		 z_dry => $tropo_apriori,
		 z_wet => $tropo_apriori,
		 ionex_dir => $ionex_dir
               );
}

die "No namelist qregres.nml!\n" if ! -e 'qregres.nml';
my $sc_file = -e 'pos'  ? 'pos'  : 
              -e 'peci' ? 'peci' : 
	      die "No pos or peci file \n";
my $scratch_dir = $def{SCRATCH_DIR};
$scratch_dir = "/tmp" if ! -d $scratch_dir;			  

#qregres( qw( -i qmfile
#               -o rgfile
#               -n qregres.nml -scr ), $scratch_dir,
#         qw(   -shad Shadow
#               -time_dep TDPfile 
#               -sc peci
#               -pr qregres.log)
#         );
         
system( "qregres  -i qmfile \\
                  -o rgfile \\
				  -n qregres.nml \\
				  -scr $scratch_dir/SCRATCH_REGRES$$ \\
				  -shad Shadow \\
				  -time_dep TDPfile \\
				  -sc $sc_file @vmf1_args \\
				  -pr qregres.log 
		");

#my $err = qr_errors("qregres.log");

#if ( $err || $? ) {
#     die qq(
#     Fatal Errors detected in the qregres log, see qregres.log
#     log error   : $err
#     system error: ($?)
#     );
#}
sub get_wwww {
  my $file = shift;
  my ($grnw,$satw,$frame);
  for ( slurp($file) ) {
     if ( /^SAT / ) {
	   ($frame,$satw) = unpack 'a5x1a4', (split(' '))[1];
	 }
     if ( /^GRN / ) {
	   ($frame,$grnw) = unpack 'a5x1a4', (split(' '))[1];
	 }
  }
  $frame = lc $frame;
  # Avoid previously sketchy relative phase centers
  if ( $frame eq 'igs08' && ( $satw < 1629 || $grnw < 1629 ) ) {
    #$satw = 1629;
    #$grnw = 1629;
    $satw = 1758;
    $grnw = 1758;
  }
  return($grnw,$satw)
}
          
sub usage {
 my $progname = basename $0;
 my $usage = qq(
   
   $progname set up a qregres namelist and run qregres for 1 station.

   Usage:
   
   $progname -nml qregres.nml

   or

   $progname -site site -date date 
   
);

}
