#!/bin/sh 

. $GPS_HOME_DIR/bin/goa_functions.sh

usage() {
  progname=$( basename $0 )
  more <<EOT
  
  $progname -proj proj_name -date date -snet snet -orb [rapid|final]  -ac [igs|jpl] [options]

  options:   (not required)
   -debug:


  See also:  goa_functions.sh, ppp.sh

  Author:
  Marcelo Santillan
  GPS Data Analyst 
  Geodesy Lab. and PANGA Data Analysis Facility.
  Central Washington University

  Copyright (c) 2009
  Central Washington University

EOT

exit 1

}
snet=()
sites=()
net_sites=()
date=
cal=
orb_type=
ac=
SITE=
proj=
debug_flag=''
clean=0
elev_mask=15
rnx_local=''
args=$@
here=$( pwd )
frame="itrf05"
del_sat=()
out_dir=''
dev_flag=''
archive_sol=1

if [ $# -eq 0 ] ;then   usage; fi

echo "$progname $@" > amb.log

while [ $# -gt 0 ]
do
     case "$1" in
        -net_sites) net_sites=( $( lc $2 | tr ',' ' ' ) )
                    echo "${net_sites[@]}"
                    shift;;
    -snet|-net) snet=($( uc $2 | sed 's/,/ /g' ))
                shift;;
         -date) date=$(gpsdate -i $2 cal )
                cal=$date 
                shift;;
         -orb) orb_type=$(lc $2)
               shift;;
         -ac)  ac=$2
               shift;;
         -proj) proj=$2
                shift;;
         -mask) elev_mask=$2
                shift;;
         -del_sat) del_sat=($(echo $2|tr ',' ' '))
                shift;;
         -local) rnx_local='-local'
                 ;;
         -frame) frame=$2
                 shift;;
         -debug) DEBUG=1
                 debug_flag='-debug'
                 ;;
         -vmf1) vmf1=('-vmf1')
                echo "Using VMF1"
                ;;
         -clean) clean=1
                 ;;
         ############ Developing flags ############        
         -proc_type) PROC_TYPE=$2
                     shift;;
              -k)    PROC_TYPE='k'
                     ;;
         #########################################            
         -no_archive) archive_sol=0
                      ;;
         -out_dir) out_dir=$2
                   shift;;
	 -dev)  . $GPS_HOME_DIR/bin/goa_functions_dev.sh
                dev_flag='-dev'
                 ;;
     esac
     shift
done

load_defaults sta_info_dir amb_dir ppp_dir work_dir_home scratch_dir

if [[ ! -d $SCRATCH_DIR ]]; then
  SCRATCH_DIR="/tmp"
fi

if [[  $proj == '' || $date == '' ||  $ac == '' || $orb_type == '' ]];then
    echo2 "Insufficient arguments !"
    usage    
fi

if [[ $ac != 'jgp' && $ac != 'igs' && $ac != 'code' && $ac != 'ig1' ]]; then
   echo2 "Invalid agency $ac"
   usage 
fi

if [[ $date != '' ]]; then
  ydoy=( $(gpsdate -i $date doy) )
  doy=${ydoy[1]}
  yyyy=${ydoy[0]}
  yy=${yyyy:2:2}
  gwd=$(gpsdate -i $date gwd)
fi

wd="$WORK_DIR_HOME/amb_work/$proj/$ac/$cal"
od="$AMB_DIR/$ac/$proj/$yyyy/$cal"
if [ $DEBUG -eq 1 ]; then
  wd="$here/$cal.$proj.$ac"
  clean=0
fi

if [ ${#snet[@]} -ne 0 ]; then
  proj_nets=(${snet[@]})
fi

for sn in ${proj_nets[@]} ; do
   echo "Processing: proj $proj ; snet: $sn ; date: $date $doy $gwd"
   sites=( ${net_sites[@]} )
   nsites=${#sites[@]};
   echo "Processing ${sites[@]}"
   result='FAILED'
   if [ $nsites -gt 1 ]; then
     echo "Making directories $wd.$sn and $od/$sn"
     mk_dirs $debug $clean "$wd.$sn" "$od/$sn"
     cd $work_dir
     echo "Checking orbits"
     ok_orb=$(get_orbits.pl -ac $ac -type $orb_type -date $date ) 
     if [ -e pos ]; then
         ln -sf pos peci
     fi
     if [ $ok_orb -eq 0 ]; then
       leave $result $archive_sol $work_dir $out_dir
       show_result "Bias-Fixing" "$result |No orbits, Bye ...|" $sn "$cal , $doy , $gwd"
       exit 1
     fi
     echo "Orbits OK"
 
     frame=$(cat frame)
     frame=$(lc $frame)

     # Copy the point position solution
     copy_pp_sol ${sites[@]}

     # Merge QM files and remove bad satellites
     merge_qm ????/$date.????.qm qmfile_in
     weed_qm qmfile_in peci qmfile

     sites=( $(qm_sta qmfile) )
     #mk_qmfile 1> mk_qmfile.$sn.log 2> mk_qmfile.$sn.err
     nsites=${#sites[@]};
     echo "$nsites ${sites[@]}"
     
     if [ $nsites -gt 1 ]; then
       qm_regres
       mv qr_err.log qr.$sn.log
       pre_estimation  1> estimation.$sn.log 2> estimation.$sn.err
       stacov -i smcov.nio -o estimation.stacov -q qregres.nml -tp tpeo.nml
       #cp pre_postfit.log postfit.log
       
       bias_fixing 1> bias_fixing.$sn.log 2> bias_fixing.$sn.err
       
       if [ $(ok_stacov fix.stacov) -eq 1 ]; then
            mk_qfront_from_pp
            proc_summary -o $cal.$sn.summary.log -name "$(uc $sn)" -date "$cal $doy $gwd"
          
            cp fix.stacov $cal.fix.stacov
            cp estimation.stacov $cal.ppp.stacov
            if [[ $orb_type == 'rapid' ]]; then
              cp $cal.fix.stacov $cal.fix.r.stacov
              cp $cal.ppp.stacov $cal.ppp.r.stacov
            fi
          
            stacov_name fix.stacov $cal $sn $ac $orb_type $frame
            stacov_name estimation.stacov $cal "$sn.ppp" $ac $orb_type $frame
          
            pack_solution $cal  $sn \
                          smap_fixed.log \
                          qregres.nml \
                          qregres.log \
                          postfit.log \
                          TDPTABLE \
                          wash_amb.nml
            result="SUCCESS"
        fi
         leave $result $archive_sol $work_dir $out_dir
         show_result "Bias-Fixing" $result $sn "$cal , $doy , $gwd"
         #echo "RESULT: $result Bias-Fixing: $sn , date: $cal , $doy , $gwd"
    else
         echo "Skipping sub-net $sn. It has less than 2 sites for bias-fixing"
         echo2 "Skipping sub-net $sn. It has less than 2 sites for bias-fixing"
         leave $result $archive_sol $work_dir $out_dir
         show_result "Bias-Fixing" "$result |Skipping sub-net $sn. It has less than 2 sites for bias-fixing|" $sn "$cal , $doy , $gwd"
    fi
  fi
done


