#!/opt/perl/bin/perl
use strict;
use Data::Dumper;
use GPS::Defaults;
use GOA::StaPos;
use GPS::DATES qw(get_date sec2cal);
use GOA::Stacov;
use File::Basename;
use Pod::Text::Termcap;
use Getopt::Long;
my $progname = basename $0;
my ($site,$date,$help,$stacov_file);
my $use_pos_date = 0;
my $args = @ARGV;
my $overwrite = 0;
my $show=1;
my $hide=0;
GetOptions( 'site:s' => \$site,
            'date:s' => \$date,
            'use_pos_date' => \$use_pos_date,
            'i:s'      => \$stacov_file,
            'stacov:s' => \$stacov_file,
            'overwrite' => \$overwrite,
            'show'   => \$show,
            'hide'   => \$hide,
            'help' => \$help,
          ) or die "Error in options\n";

if ( !$args || $help ) {
  usage();
  exit 1;
}
$show = 0 if $hide;
my %par  = get_defaults( );

my $sta_pos  = GOA::StaPos->new( file => "$par{STA_INFO_DIR}/sta_pos",
                                 overwrite => $overwrite );

my $pos    =  GOA::StaPos->new( );
my $stacov =  GOA::Stacov->new( file => $stacov_file );
#print Dumper($stacov);
my $npar = $stacov->npar;

$site = uc $site if $site;

if ( ! $site && $npar == 3 ) {
  #my $h = $stacov->ipar(1); $h->{site};
  $site =  $stacov->site(1);  
}

if ( ! $date ) {
  $date = $stacov->date;
}
my $today = get_date('today','cal,time');
my @sites = $site ? ($site) : $stacov->sites;
my ($ss,$s,$p,%args);
foreach my $sta ( @sites ) {
   if ( $use_pos_date ) {
     $ss = $sta_pos->get($sta); 
     ($s) = sort { $a <=> $b } keys %$ss;
     $date = sprintf ( "%4d-%02d-%02d",sec2cal($s));
   }
   $p=$stacov->xyz($sta);
   if ( ! $p ) { 
      warn "No information for $sta in $stacov_file\n"; 
      next;
   }
   %args= ( sta     => $sta ,
            date    => $date,
            x       => $p->[0],
            y       => $p->[1],
            z       => $p->[2],
            comment => "|Generated from $stacov_file by VMS on $today|",
	  );
  
  if ( $overwrite ) {
    $sta_pos->add(%args);
  }
  $pos->add(%args);
}

if ( $overwrite ) {
  $sta_pos->print( file => "$par{STA_INFO_DIR}/sta_pos" );
}

$pos->print() if $show;

sub usage {
  my $pod=Pod::Text::Termcap->new();
  $pod->parse_from_filehandle(man_page());
}

sub man_page {
my $man =qq(
=head1 NAME

$progname 

=head1 DESCRIPTION
 
$progname updates the sta_pos with information from a stacov file.


$progname -i stacov [OPTIONS]

=head1 OPTIONS

=over 3

=item -site SITE     Station from the stacov file to be updated 

=item -date DATE     Generate or overwrite  sta_pos entry at this date

=item -overwrite     Overwrite sta_pos entry 

=item -use_pos_date  Uses the date of an existing sta_pos entry

=item -show          Print sta_pos entry to stdout



=back

=head1 EXAMPLES

=over 3

=item $progname -i stacov_final -date 20010101 -site albh  


=back

=head1 AUTHOR

Victor Marcelo Santillan <marcelo\@geology.cwu.edu>

=head1 COPYRIGHT

Copyright \(c\) 2007 

Central Washington University

=cut
);
open(my $fh,"<",\$man ) or die "$!\n";
return $fh
}

