#!/usr/bin/env python

class Tide(object):
    """
    A class to hold tide amplitude, phase and name information
    """

    def __init__(self,name=None,amplitude=None,phase=None):
        self.name = name
        self.amplitude = amplitude
        self.phase = phase

    def __str__(self):
        retstr = "{0:s} {1:f} {2:f}".format(self.name,self.amplitude,self.phase)
        return retstr

class Harmonics(object):
    """
    A class to hold ocean tide harmonics for an observation station
    """

    def __init__(self,name=None):
        self.name = name
        self.radialTide = None
        self.ewTide = None
        self.nsTide = None

    def addRadialTide(self,tide):
        self.radialTide = tide

    def addEWTide(self,tide):
        self.ewTide = tide

    def addNSTide(self,tide):
        self.nsTide = tide

    def __str__(self):
        retstr = "{0:s}".format(self.name)
        return retstr

class BLQReader(object):
    """
    A BLQ Reader class
    """

    tides = ('M2','S2','N2','K2','K1','O1','P1','Q1','MF','MM','SSA')

    def __init__(self):
        self.tideData = [] 

    def parse(self,filename):
        fp = open(filename,"r")

        lines = [x.strip() for x in fp.readlines() if (x[0:2] != '$$')]
        i = 0
        while (i < len(lines)):
            # Station name lines have one entry
            # Tidal harmonic lines have eleven entries
            harmonics = Harmonics(name=lines[i].split()[0])
            # Radial amplitude in meters
            radialAmplitude = self.parseDataLine(lines[i+1])
            # EW amplitude in meters
            ewAmplitude = self.parseDataLine(lines[i+2])
            # NS amplitude in meters
            nsAmplitude = self.parseDataLine(lines[i+3])
            # Radial phase in degrees
            radialPhase = self.parseDataLine(lines[i+4])
            # EW phase in degrees
            ewPhase = self.parseDataLine(lines[i+5])
            # NS phase in degrees
            nsPhase = self.parseDataLine(lines[i+6])
            for j,tide in enumerate(BLQReader.tides):
                radialTide = Tide(name=tide,amplitude=radialAmplitude[j],phase=radialPhase[j])
                ewTide = Tide(name=tide,amplitude=ewAmplitude[j],phase=ewPhase[j])
                nsTide = Tide(name=tide,amplitude=nsAmplitude[j],phase=nsPhase[j])
                harmonics.addRadialTide(radialTide)
                harmonics.addEWTide(ewTide)
                harmonics.addNSTide(nsTide)
            self.tideData.append(harmonics)
            i = i + 7

        fp.close()

    def parseDataLine(self,line):
        data = map(float,line.split())
        if (len(data) != 11):
            raise ValueError("Invalid Tidal Coefficient Line: {0:s}".format(line))
        return data

if __name__ == "__main__":
    blq = BLQReader()
    blq.parse('/Users/walter/temp/tides/ocnld_coeffs')
    for station in blq.tideData:
        print station.name
