#!/usr/bin/env python

import argparse
import datetime
import timeutils
import pbo_process

def main(args):
    project = 'panga'
    ac = 'jgp'
    orb = 'final'
    # Final orbits are available with a 14 day latency
    # If we are running the the finals on Friday, we want
    # start on the first Sunday greater than 14 days ago
    # this day should be 20 days prior
    if (args.date):
        finalsDate = datetime.datetime.strptime(args.date,"%Y-%m-%d")
    else:
        finalsDate = datetime.datetime.today() - datetime.timedelta(days=14)
    # The Sunday of the week 
    startDate = timeutils.getSunday(finalsDate)

    args = argparse.Namespace(date=startDate,
            proj=project,
            ac=ac,
            orb=orb,
            snet=None,
            ndays=7,
            local=False,
            upload=True,
            update=True,
            amb=True,
            wlpb=False,
            address=['walter@geology.cwu.edu'])
    pbo = pbo_process.main(args)

if __name__ == "__main__":
    import argparse
    import logging.config

    parser = argparse.ArgumentParser(description="PANGA Weekly processing script")
    parser.add_argument('--date','-d',help='Date in week to process')

    args = parser.parse_args()
    format = "%(asctime)s- %(name)s - %(levelname)s - %(message)s "
    logging.config.dictConfig({
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'standard': {
                'format': format
            },
         },
         'handlers': {
             'file': {
                 'level': 'DEBUG',
                 'class': 'logging.handlers.RotatingFileHandler',
                 'filename': '/var/log/pbo/panga-weekly.log',
                 'formatter': 'standard',
                 'maxBytes': 5242880,
                 'backupCount': 5
             }
         },
         'loggers': {
             '': {
                 'handlers': ['file'],
                 'level': 'DEBUG',
                 'propagate': True
             }
         }
    })
    main(args)
