#!/usr/bin/env python

import datetime
import timeutils
import pbo_supplemental

def main(args):
    project = 'pbo-supplemental'
    ac = 'jgp'
    orb = 'final'

    if (args.date):
        finalsDate = datetime.datetime.strptime(args.date,"%Y-%m-%d")
    else:
        finalsDate = datetime.datetime.today() - datetime.timedelta(weeks=13)

    # The Sunday of the week
    startDate = timeutils.getSunday(finalsDate)

    args = argparse.Namespace(date=startDate,
            proj=project,
            ac=ac,
            orb=orb,
            snet=None,
            ndays=7,
            local=False,
            upload=False,
            update=True,
            amb=False,
            wlpb=True,
            address=['pbo-supplemental@geology.cwu.edu'])
    pbo = pbo_supplemental.main(args)

if __name__ == "__main__":
    import argparse
    import logging.config

    parser = argparse.ArgumentParser(description="PBO 3 month supplemental processing script")
    parser.add_argument('--date','-d',help='Date in week to process')

    args = parser.parse_args()
    format = "%(asctime)s- %(name)s - %(levelname)s - %(message)s "
    logging.config.dictConfig({
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'standard': {
                'format': format
            },
         },
         'handlers': {
             'file': {
                 'level': 'DEBUG',
                 'class': 'logging.handlers.RotatingFileHandler',
                 'filename': '/var/log/pbo/pbo-supplemental3.log',
                 'formatter': 'standard',
                 'maxBytes': 5242880,
                 'backupCount': 5
             }
         },
         'loggers': {
             '': {
                 'handlers': ['file'],
                 'level': 'DEBUG',
                 'propagate': True
             }
         }
    })

    main(args)
