#!/usr/bin/env python

import datetime
import distutils
import pbo_process
import gipsy6

def main(args):
    remake = pbo_process.PBOProcess(args.proj,args.ac,args.orb)

    startdate = datetime.datetime.strptime(args.date,"%Y-%m-%d")

    for i in range(args.ndays): 
        date = startdate + datetime.timedelta(days=i)
        # Copy the archived solution to the working area
        remake.retrieveArchivedSolutions(date,amb=False)
        # Merge the networks
        remake.mergeNetworks(date,amb=False)
        # Archive the merged solution
        remake.archive(date,rinex=False,ppp=False,amb=False)
        # Clean working directory
        remake.clean(date,rinex=False,ppp=True,amb=False)

    remake.shutdown()

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="Remake SINEX files")
    parser.add_argument('--date','-d',help="Date to process",required=True)
    parser.add_argument('--ndays',help='Number of days to process',type=int,default=1)
    parser.add_argument('--orb',help="Orbit type",default='rapid',choices=['rapid','final'])
    parser.add_argument('--ac',help="Acquisition center",default='igs',choices=['igs','jgp'])
    parser.add_argument('--proj',help="Project name")

    args = parser.parse_args()
    main(args)
