#!/usr/bin/env python

import os.path
import urllib
import gzip
import timeutils

class SINEX(object):

    def __init__(self):
        self.solutionEpochs = []
    # Right now, only need to read in the +SOLUTION/EPOCHS (-SOLUTION/EPOCHS) tag

    # Observations codes are
    # 'GNSS' => 'P'
    # 'DORIS' => 'D'
    # 'SLR' => 'L'
    # 'VLBI' => 'R'
    # 'LLR' => 'M' # Lunar laser ranging, M for Moon, I guess
    # 'Combined' => 'C'

    # Entries are Site Code, Point Code, Solution ID, Observation Code, Start Time, End Time, Mid Time
    def parseSolutionEpoch(self,line):
        entries = line.split()
        solutionEpoch = {'Site Code':entries[0],
                'Point Code': entries[1],
                'Solution ID': entries[2],
                'Observation Code':entries[3],
                'Start Time': entries[4],
                'Stop Time': entries[5],
                'Mid Time': entries[6]}

        return solutionEpoch


    def parse(self,filename):
        fp = gzip.open(filename,'rb')

        sectionTag = None
        for line in fp:

            if (line[0] == '+'):
                sectionTag = line[1:].rstrip()
                continue
            elif (line[0] == '-'):
                sectionTag = None
                continue
            # SINEX V 1.0 the tag was called EPOCH, in V 2.0 it's called EPOCHS 
            if (sectionTag in ('SOLUTION/EPOCH','SOLUTION/EPOCHS')):
                self.solutionEpochs.append(self.parseSolutionEpoch(line))

        fp.close()

def downloadSINEX(date,ac,orbit,frame):
    """
    Download a SINEX file
    :param date: The date of the desired SINEX file
    :param ac: The analysis center that produced the SINEX file
    :param orbit: The orbit type used to make the SINEX fil
    :param frame: The reference frame of the SINEX file
    """
    gpswd = timeutils.datetimeToGPSWeek(date)

    url = "ftp://data-out.unavco.org/pub/products/sinex/"
    directory = "{gpsw}".format(gpsw=str(gpswd[0]))
    #filename = "cwu17460.final_loose.snx.gz" 
    filename = "cwu{gpsw}{gpsd}.{orbit}_{frame}.snx.gz".format(gpsw=str(gpswd[0]),gpsd=str(gpswd[1]),
                                                               orbit=orbit,frame=frame)

    print "Retrieving {url}".format(url=os.path.join(url,directory,filename))
    outputFilename = '/tmp/{filename}'.format(filename=filename)
    urllib.urlretrieve(os.path.join(url,directory,filename),outputFilename)

    return outputFilename

if __name__ == "__main__":
    sinex = SINEX()
    sinex.parse('cwu17460.final_loose.snx.gz')
    for site in sinex.solutionEpochs:
        print site['Site Code']
