#!/usr/bin/env python

import os
import os.path
import shutil
import logging

def move(src,dst,overwrite=False):
    """
    Move a whole directory from one location to another
    A wrapper for the shutil.move function to allow overwriting
    """
    status = True
    if (overwrite):
        if (os.path.exists(dst)):
            logging.info("Removing destination directory %s" % (dst))
            try:
                shutil.rmtree(dst)
            except OSError as e:
                logging.error(e)
                status = False
    # Now move the destination to the source
    try:
        shutil.move(src,dst)
    except (IOError, shutil.Error, OSError) as e:
        logging.error(e)
        status = False

    return status

def moveFiles(src,dst,overwrite=False):
    """
    Move the files in one directory to another location
    """
    status = True
    if (not os.path.exists(dst)):
        os.mkdir(dst)
    for root,dirs,files in os.walk(src):
        for file in files:
            shutil.copy(os.path.join(root,file),os.path.join(dst,file))
            try:
                logging.info("Removing file %s" % (os.path.join(root,file)))
                os.remove(os.path.join(root,file))
            except OSError as e:
                logging.error(e)
                status = False

    return status

def copy(src,dst,overwrite=False):
    """
    Move a whole directory from one location to another
    A wrapper for the shutil.copy function to allow overwriting
    """
    status = True

    
    # Now copy the destination to the source
    try:
        shutil.copy(src,dst)
    except (IOError, shutil.Error, OSError) as e:
        logging.error(e)
        status = False

    return status

def copyFiles(src,dst,overwrite=False):
    """
    Copy the files in one directory to another location
    """
    status = True
    if (not os.path.exists(dst)):
        logging.info("Creating directory %s" % (dst))
        os.makedirs(dst)
    for root,dirs,files in os.walk(src):
        for file in files:
            logging.info("Copying %s to %s" % (os.path.join(root,file),os.path.join(dst,file)))
            shutil.copy(os.path.join(root,file),os.path.join(dst,file))

    return status

def remove(filename):
    """
    Unlink a directory
    """
    pass
