package GOA::OceanLoading;
use Exporter;
use GPS::Geodesy qw(distance);

@ISA = ('Exporter');

@EXPORT = qw( ocean_loading );

sub ocean_loading {
   #! ALBH                       lon/lat:     236.51253      48.38978
   my %opt = ( file_coeffs => "$ENV{GOA_VAR}/sta_info/ocnld_coeff_cm_fes04",
               sta => 'ALBH',
               nsta => 1,
               lon => 236.51253,
               lat => 48.38978,
               min_dist => 1, #km
               max_dist => 10, #km
               @_,
   );
   my $filename = $opt{'file_coeffs'}; 
   my $min_dist = $opt{min_dist};
   my $max_dist = $opt{max_dist};
   my $dist = $opt{max_dist};
   my $distance = '';
   # Set the global minimum distance to some large number
   # so that we can iterate to the actual minimum distance
   my $global_min_distance = 1e9; 
   my ($sta,$lon,$lat);
   my $nml='';
   open( my $fh , '<', $filename ) 
     or die "Couldn't open  $filename $!\n";
   
   while ( <$fh> ) {
       last if /END HEADER/;
       if ( $. > 200 ) {
          die "END OF HEADER not founded within first 200 lines\n".
              "possible error in format? \n";
       }
   }
   my @lines=();
   my @tmp_lines=('')x12;
   my $j = 0;
   while ( <$fh> ) {
        if ( /^\$\$/ ) {
           $tmp_lines[$j++]=$_; 
        }
	#if ( /^\$\$\s+(\w{4})\s+lon\/lat:\s+(.*)$/ ) {
	# Pull out the longitude and latitude
	if ( /lon\/lat:/ ) {
	   my @data = split;
	   $sta = $data[1];
	   $lon = $data[-3]*1.0;
	   $lat = $data[-2]*1.0;
	   my $height = $data[-1]*1.0;
           $dist =  distance( lon1 => $opt{lon},
                              lat1 => $opt{lat},
                              lon2 => $lon,
                              lat2 => $lat
                             );
           if ($dist < $global_min_distance) { $global_min_distance = $dist; }
           if ( $dist >= 0 && $dist <= $min_dist ) {
               $min_dist = $dist;
               $distance = $dist;
               @lines=@tmp_lines;
               my $k = @lines;
               for my $i ( 0..5 ) {
                  $lines[$i+$k]= readline($fh); 
               }
           }
           $j=0;
           @tmp_lines=('')x12;
        }
   }
   if ( @lines ) {
      @lines=map { $_ =~ s/^\$\$/\!/; $_ } @lines;
      @a_r = split ' ',$lines[-6];
      @a_w = split ' ',$lines[-5];
      @a_s = split ' ',$lines[-4];
      @p_r = split ' ',$lines[-3];
      @p_w = split ' ',$lines[-2];
      @p_s = split ' ',$lines[-1];
      $ntides = @a_r;
      $rms = 0;
      for my $k ( 0 .. $ntides-1 ){
        $rms += (($a_r[$k])**2 + ($a_w[$k])**2 + ($a_s[$k])**2) ;
      }
      $nml = join '',@lines[0..$#lines-6];
      $nml .= sprintf("! %.2f km between nominal position of".
                      " %s and database station\n", $distance, $opt{sta});
      $nml .= sprintf("! Overall rms displacement ".
                      "due to ocean loading = %.2f cm\n", sqrt($rms)*100);
      $nml .= sprintf("  NMLOceanLoad(%3d) = .true.\n", $opt{nsta});

      for my $k ( 0 .. $ntides-1 ) {
         $nml .= sprintf("  NMLOcnLoad_Amp(1,%3d,%3d) = %14.8f%14.8f%14.8f\n",
                         $k+1, $opt{nsta},
                         -1.0e-3*$a_w[$k], -1.0e-3*$a_s[$k], 1.0e-3*$a_r[$k]);
      }
      $d2r = atan2(1,1)/45;
      for my $k ( 0 .. $ntides-1 ) {
         $nml .= sprintf("  NMLOcnLoad_Phs(1,%3d,%3d) = %14.8f%14.8f%14.8f\n",
                            $k+1, $opt{nsta}, 
                            $d2r*$p_w[$k], $d2r*$p_s[$k], $d2r*$p_r[$k]);
      }
   }
   else {
      $nml .= sprintf("! Closest ocean loading database station is %.2f km\n",$global_min_distance);
   } 
   close $fh;

   return $nml; 
}
