package GOA::PCenter;
use Fortran::F90Format;
use Carp;
use GPS::DATES;
use GOA::Util;
use GPS::Defaults;
use File::Basename;

my @fields = qw( goa_ant
                 pc_type
                 east
                 north
                 up
                 comment
               );

# pcenter format: (a9,1x,a2,3f9.4)
# append field for comments, (1x,a)
my $fmt = 'a9,1x,a2,3f9.4,1x,a'; 


my @pc_fields = qw/L1 L2 LC P1 P2 PC C2 DR R2/;

my @today = get_date('today');


my %ROW  = (  goa_ant   => 'ZERO     ',
              pc_type   => '',
              east      => 0.0,
              north     => 0.0,
              up        => 0.0,
              comment   => "Generated on @today by VMS",
             );
my %par=();

sub new {
  my $class = shift;
    my $self = { };
    my %opt = (
	  file => '',
      @_,
    );

    bless $self,$class;
     
    $self->{fmt} = Fortran::F90Format->new( fmt => $fmt );
    
    $self->{overwrite} = 0;
    
    %par = get_defaults( );
   
    $self->{file} = "$par{STA_INFO_DIR}/pcenter";

    if ( $opt{file} ) { 
	  $self->file($opt{file});
      $self->load(%opt);
	}
    
	return $self

}
sub overwrite {
  my $self = shift;
  my $o    = shift || 0;
  $self->{overwrite}  = $o ? 1 : 0;
  return $self->{overwrite}
}
sub file {
  my $self = shift;
  if ( @_ ) {
    my $f = shift;
	if ( -e $f ) {
	  $self->{file} = $f;
	}
	else {
	  die "pcenter file $f doesn't exists \n";
	}
  }
  return $self->{file}
}

sub fmt {
  my $self = shift;
  return $self->{fmt}
}

sub ant_lines_fh {
   my $self = shift;
   my $ant  = uc shift;
   my $ant_lines = join '', grep { /^$ant/ } slurp( $self->file );
   open( my $sh , '<', \$ant_lines )
            or die "ant_lines_fh: Couldn't open string for reading $!\n";
   return $sh
}
sub  load {
  my $self = shift;
  my %opt  = ( goa_ant => '',
               @_,
              );
              
  my $file = $self->file;
  open( $fh,"<", $file ) or die "Couldn't open $file $!\n";

  if ( $opt{goa_ant} ) {
     $fh = $self->ant_lines_fh( $opt{goa_ant} )
  }
  my ($h,$secs);
  while( <$fh> ) {
    chomp;
    my $h = {};   
    @{$h}{@fields}=$self->fmt->read($_);
    @{$self->{PC}{ $h->{goa_ant}   }
                 { $h->{pc_type}   }
                }{ qw/east
                      north
                      up
                      comment /  } = @{$h}{qw/east  
                                              north     
                                              up      
                                              comment / };
                                                   
                 
                 
  }
  return $self

}


sub add {
    my $self = shift;
    my %e    = ( %ROW,
                 @_,
                ); 

    foreach my $f ( keys %e ) { delete $e{$f} if ! exists $ROW{$f}; } 
    $self->set( @e{@fields} ); 
    
}

sub set {
   my $self = shift;
   my @vals = @_ == @fields ? @_ : croak "Incorrect number of fields\n";
   my $h = {};

   @{$h} {@fields} = @ROW{@fields};
   @{$h}{@fields}=@vals;
    $h->{goa_ant} = uc $h->{goa_ant};
    $h->{pc_type} = uc $h->{pc_type};
    
    my $ok = 1;
    if ( ! $h->{goa_ant} || length($h->{goa_ant}) > 9 ) {
      carp "Invalid goa_ant field $h->{goa_ant} \n";
      $ok=0;
    }
    if ( ! $h->{pc_type} || ! (grep /$h->{pc_type}/,@pc_fields) ) {
      carp "Invalid pc_type field $h->{pc_type}\n";
      $ok = 0;
    }
    
    if ( $ok ) { # add new entry
       if ( exists $self->{PC}{$h->{goa_ant}}{pc_type}   ) {
           carp "$h->{goa_ant} $h->{pc_type} already exists\n";
           return unless $self->{overwrite} 
       }
       @{$self->{PC}{ $h->{goa_ant} }
                    { $h->{pc_type}  } } #slice
                    { qw/east
                         north
                         up
                         comment/      }
                                        = @{$h}{ qw/east
                                                    north
                                                    up
                                                    comment / };
   } #ok  
   else {
    carp "Not enough information to add a new row (goa_ant,pc_type,..)\n";
  } 
}

sub select {
    my $self     = shift;
    my $goa_ant  = uc shift || '';
    my $pc_type  = uc shift || '';
    my $rows = [];
    
    foreach my $pc ( @pc_fields ) {
      my $hh = $self->get($goa_ant,$pc);
      my $h={};
      @{$h}{@fields} = ( $goa_ant, 
                         $pc,
                         @{$hh}{ qw/east
                                    north
                                    up
                                    comment / }
                       );
      push @$rows,$h; 
    }
    
    return $rows
}

sub get {
   my $self      = shift;
   my $goa_ant   = shift || '';
   my $pc        = shift || '' ;
   my $s = $self->{PC};
      $s = $s->{$goa_ant}   if $goa_ant  && exists $s->{$goa_ant}; 
      $s = $s->{$pc}        if $pc       && exists $s->{$pc}; 
   return $s 
}

sub print {
   my $self = shift;
   my %opt = (
        file => '',
        fh   => \*STDOUT,
        @_,
   );
   my $fh = $opt{fh};
   if ( $opt{file} ) {
      $fh = \*FILE;
      open( $fh , "> $opt{file}") 
        or croak "Couldn't open $opt{file} $!\n";
   }
   my $hga = $self->get;
   foreach my $ga ( sort keys %{$hga} ) {
         foreach my $pc ( @pc_fields ) {
             my $hh = $self->get($ga,$pc);
             my @v = ( $ga,
                       $pc,
                       @{$hh}{ qw/east
                                  north
                                  up
                                  comment /   }
                      );
            #print "@v\n";
            print $fh $self->fmt->write(@v);
        }
   }

}

1;









