package GOA::PostfitLog;
use Carp;
use GOA::Util;

sub new {
  my $class = shift;
  my $self = { file => 'postfit.log',
               stats => {},
             };
  bless $self,$class;
  $self->file(@_);             
  $self->load();
  return $self
}
sub file {
  my $self = shift;
  if ( @_  ) {
    $self->{file} = shift;
    croak "$self->{file} doesn't exists!\n" if ! -e $self->{file};
  }
  return $self->{file}
}

sub stats {
  my $self = shift;
  my $type = lc shift || '';
  if ( ! $type ) {
    return $self->{stats}
  }
  if ( exists $self->{stats}{$type} ) {
    return $self->{stats}{$type}
  }
  else {
    croak "stats: type |$type|, doesn't exists\n";
  }
}

sub load {
  my $self = shift;
  my $file = $self->file(@_);
  open(my $fh,'<',$file ) or 
    croak "Couldn't open $file $!\n";
  
  my $stats = $self->stats;
  my %fmap = ( DATA => 'dtype',
               TRANSMITTER => 'sat',
               RECEIVER    => 'site',
               MEAN        => 'mean',
               SIGMA       => 'sigma',
               NUM         => 'num_pts',
               'PRE-FIT'  => 'pre_fit',
               'POST-FIT' => 'post_fit',
               );
  my %dtype = ( 110 => 'PC',
                111 => 'P1',
                112 => 'P2',
                120 => 'LC',
                121 => 'L1',
                122 => 'L2',
                130 => 'KBR',
                160 => 'SCC',
                170 => 'R2',
                181 => 'AZ',
                182 => 'EL',
  );               
  my $stype;               
  while ( <$fh> ) {
     s/(^\s+|\s+$)//g;
     if ( /(P.*?-FIT)\s+N.*:(.*)$/ ) {
       $stats->{$fmap{$1}} = trim($2);
     }
     if ( /^DATA/ ) {
       @fields = @fmap{split(' ',$_)};
       my %f;
       @f{@fields}=@fields;
       $stype =   exists $f{sat} &&   exists $f{site}   ? 'by_sat_site' :
                ! exists $f{sat} &&   exists $f{site}   ? 'by_site'     :
                  exists $f{sat} && ! exists $f{site}   ? 'by_sat'      : 'all';
       #print "$stype\n@fields\n";                   
     }
     #110 111 112 120 121 122 130 160 170 181 182
     #PC  P1  P2  LC  L1  L2  KBR SCC R2  AZ  EL
     #see filter user guide
     next if ! /^(110|111|112|120|121|122|130|160|170|181|182) /;
     my @vals = split(' ',$_);
     my $h={};
     @{$h}{$dtype{$vals[0]},@fields[1..$#fields]} = @vals;
     push @{ $stats->{$stype} },$h;
     #print "$dtype{$vals[0]} @vals[1..$#vals]\n";
  }
}

1;
