package GOA::QReg;
use POSIX qw( strftime );
use File::Copy;
use lib "$ENV{GOA}/perl_pm";
use Scalar::Util qw(looks_like_number);
use Carp;
use File::Basename;
use Exporter;
use GOA::Util;
use GOA::StaPos;
use GOA::StaSvec;
use GOA::StaId;
use GOA::PCenter;
use GPS::DATES;
use GPS::Geodesy qw( distance );
use trans;
@ISA = ( 'Exporter' );

@EXPORT = qw( qr_nml );

sub tides {
   my $tides = shift ;
   my @posible_tides = qw( PolTid 
                           WahrK1
                           FreqDepLove
                           OctTid
                           OcnldC;adfkotwpn
                         );
   my $nml = "\n";
   foreach my $t ( @posible_tides ) {
     next if  $tides !~ /$t/i;
     next if  $t =~ /WahrK1/i && $tides =~/FreqDepLove/i ;
     $nml .= "    NML$t = .TRUE.\n";
  } 
                 
  return $nml or croak "No valid tides in |$tides|\n";
}

sub station_model {
   my %opt = ( sta     => '',
               nsta    => 1,
               goa_ant => '',
               date => '',
               rate => 300,
               ocean_loading => 1,
               tides => 'PolTid,WahrK1' ,
               sta_info_dir => "$ENV{GOA_VAR}/sta_info",
               abs_pc   => '',
               z_dry => 'default',
               z_wet => 'default',
               nml  => {},
               min_set_partials  => '', 
			   grn_wwww => '',
               @_,
   );

   my $sta = uc $opt{sta};
   my $dir = $opt{sta_info_dir};
   my $svec      = GOA::StaSvec->new( file=>"$dir/sta_svec" , abs_pc =>  $opt{abs_pc} );
   my $sv        = $svec->sv($sta,$sta,$opt{date});   
   my $svec_file = $svec->file;
      $opt{goa_ant} =  $sv->{pc_ant} if ! $opt{goa_ant};
   my $pc_ant = pack 'A9',$opt{goa_ant};
   my $pos       = GOA::StaPos->new( file => "$dir/sta_pos" , abs_pc =>  $opt{abs_pc});
   my $xyz_pos = $pos->xyz($sta,$opt{date});
   my $pcenter = GOA::PCenter->new( file => "$dir/pcenter" , abs_pc =>  $opt{abs_pc});
   my $pc = $pcenter->get($pc_ant); 
   my $nmlsite = "\n";
   my $nsta = $opt{nsta};
   my $i = 1;
   my $m2km = 1.0e-3; 
   foreach my $dt ( qw/L1 L2 LC P1 P2 PC C2 DR R2/ ) {
      my @v=($m2km)x3;
      $v[0] *= $pc->{$dt}{east}   + $sv->{x};
      $v[1] *= $pc->{$dt}{north}  + $sv->{y};
      $v[2] *= $pc->{$dt}{up}     + $sv->{z} + $sv->{height};
      $nmlsite .= sprintf ' 'x5 . 
                      "NMLSite(1,%2d,%3d) =  %15.7f%15.7f%15.7f\n",
                      $i,$nsta,@v ;
      $i++;                             
   }
   # make xyz in km to use trans::xyz2llh
   my $xyz = [];
      @$xyz = map { $_*$m2km } @{$xyz_pos}; 
   my ($lat,$lon,$h ) = trans::xyz2llh( @$xyz );
      $h*=1000; # km to meters 
   
   my $dry = $opt{z_dry};
   my $wet = $opt{z_wet};
   $dry = 1.013 * 2.27 * exp(-.000116 * $h) if $dry eq 'default';
   $wet = 0.1 if $wet eq 'default';
   $dry = sprintf "%10.4f", $dry;
   $wet = sprintf "%10.4f", $wet;
  
   my $llh = sprintf "!    %12.7f    %12.7f    %10.4f\n\n",$lat,$lon,$h;
   @$xyz = map { sprintf "%18.6f",$_ } @$xyz;
   my $ocean_loading = '';
      if ( $opt{ocean_loading} ) {
           $ocean_loading =  ocean_loading( sta => $sta,
                                            nsta => $nsta,
                                            lon => $lon,
                                            lat => $lat,
                                    );
      }
   my $pcm_dir = "$dir/phase_center_maps/by_antenna_type.$opt{grn_wwww}";
   my $goa_ant = trim($pc_ant);
   my $pcm_grn = '';
   if ( ! $opt{min_set_partials} ) {
      $pcm_grn = join '',
                 map { s/Ant_no/$nsta/g;$_ }
                 grep { ! /^(\$station_model|\$end)/ }
                 slurp("$pcm_dir/$goa_ant");
   }
#follow gd2p to setup nml for single station data
   my $nml = qq(
     NMLname( $nsta ) = '$sta'
     NMLTropMap( $nsta ) = 'NIELL'
     NMLMonument(1, $nsta) = 
                          $xyz->[0]
                          $xyz->[1]
                          $xyz->[2]
!        lat             long          elev (m)
$llh

!   Antenna type: $pc_ant ( from $svec_file )

      NMLTROP_DRY( $nsta ) = $dry 
      NMLTROP_WET( $nsta ) = $wet

$nmlsite
$ocean_loading
$pcm_grn
);

  return $nml

}
sub ocean_loading {
   #! ALBH                       lon/lat:     236.51253      48.38978
   my %opt = ( file_coeffs => "$ENV{GOA_VAR}/sta_info/ocnld_coeffs",
               sta => 'ALBH',
               nsta => 1,
               lon => 236.51253,
               lat => 48.38978,
               min_dist => 1, #km
               max_dist => 10, #km
               @_,
   ); 
   my $min_dist = $opt{min_dist};
   my $max_dist = $opt{max_dist};
   my $dist = $opt{max_dist};
   my $distance = '';
   my ($sta,$lon,$lat);
   my $nml='';
   open( my $fh , '<', $opt{file_coeffs} ) 
     or die "Couldn't open  $opt{file_coeffs} $!\n";
   
   while ( <$fh> ) {
       last if /END HEADER/;
       if ( $. > 200 ) {
          die "END OF HEADER not founded within first 200 lines\n".
              "possible error in format? \n";
       }
   }
   my @lines=();
   my @tmp_lines=('')x12;
   my $j = 0;
   while ( <$fh> ) {
        if ( /^\$\$/ ) {
           $tmp_lines[$j++]=$_; 
        }
        if ( /^\$\$\s+(\w{4})\s+lon\/lat:\s+(.*)$/ ) {
           $sta = $1 ; 
           ($lon, $lat) = split ' ',$2;
           $dist =  distance( lon1 => $opt{lon},
                              lat1 => $opt{lat},
                              lon2 => $lon,
                              lat2 => $lat
                             );
           if ( $dist >= 0 && $dist <= $min_dist ) {
               $min_dist = $dist;
               $distance = $dist;
               @lines=@tmp_lines;
               my $k = @lines;
               for my $i ( 0..5 ) {
                  $lines[$i+$k]= readline($fh); 
               }
           }
           $j=0;
           @tmp_lines=('')x12;
        }
   }
   if ( @lines ) {
      @lines=map { $_ =~ s/^\$\$/\!/; $_ } @lines;
      @a_r = split ' ',$lines[-6];
      @a_w = split ' ',$lines[-5];
      @a_s = split ' ',$lines[-4];
      @p_r = split ' ',$lines[-3];
      @p_w = split ' ',$lines[-2];
      @p_s = split ' ',$lines[-1];
      $ntides = @a_r;
      $rms = 0;
      for my $k ( 0 .. $ntides-1 ){
        $rms += (($a_r[$k])**2 + ($a_w[$k])**2 + ($a_s[$k])**2) ;
      }
      $nml = join '',@lines[0..$#lines-6];
      $nml .= sprintf("! %.2f km between nominal position of".
                      " %s and database station\n", $distance, $opt{sta});
      $nml .= sprintf("! Overall rms displacement ".
                      "due to ocean loading = %.2f cm\n", sqrt($rms)*100);
      $nml .= sprintf("  NMLOceanLoad(%3d) = .true.\n", $opt{nsta});

      for my $k ( 0 .. $ntides-1 ) {
         $nml .= sprintf("  NMLOcnLoad_Amp(1,%3d,%3d) = %14.8f%14.8f%14.8f\n",
                         $k+1, $opt{nsta},
                         -1.0e-3*$a_w[$k], -1.0e-3*$a_s[$k], 1.0e-3*$a_r[$k]);
      }
      $d2r = atan2(1,1)/45;
      for my $k ( 0 .. $ntides-1 ) {
         $nml .= sprintf("  NMLOcnLoad_Phs(1,%3d,%3d) = %14.8f%14.8f%14.8f\n",
                            $k+1, $opt{nsta}, 
                            $d2r*$p_w[$k], $d2r*$p_s[$k], $d2r*$p_r[$k]);
      }
   } 
   return $nml; 
}

sub qr_nml {
   my %opt = ( sta     => '',
               goa_ant => '',
               date => '',
               rate => 300,
               tides => 'PolTid,WahrK1' ,
               sta_info_dir => "$ENV{GOA_VAR}/sta_info",
               abs_pc   => '',
               z_dry => 'default',
               z_wet => 'default',
               min_set_partials => '',
               tpnml  => 'tpeo.nml',
			   grn_wwww => '',
			   sat_wwww => '',
               nml  => {},
               @_,
   );
   $opt{sta} = uc $opt{sta};
   my @sites     = split /,/,$opt{sta};
   my $nosta     = @sites;
   my $tides     = tides( $opt{tides} );
   my $station_model = '';
   my $nsta = 1;
   foreach my $s ( @sites ) {
      $opt{sta} = $s;
      $opt{nsta} = $nsta++;
      $station_model  .= station_model(%opt);
   }
   my $min_set_partials = "\n";
   if ( $opt{min_set_partials} ) {
      $min_set_partials .= "  NMLMinimal_set_partials = .TRUE.\n";
      $opt{abs_pc} = '';
   }
   else {
      $min_set_partials .= qq(
      NMLStaPrtl(1,  1) = 
                          'STA X',
                          'STA Y',
                          'STA Z',
                          'WETZTROP',
                          'TRPAZCOS',
                          'TRPAZSIN',
                          'STA BIAS',
                          ' '
);
   }
   my $tdppo = 2; #TdpPolyOrd
   if ( $opt{rate} != 300 ) {
      $tdppo = 3; 
   }
   my $epoch = sprintf "%.2f", get_date( $opt{date},'secj2000');
   my @date = get_date( $opt{date} );
#follow gd2p to setup nml for single station data
   my $sn_NoSat = 0;
   my $sn_NoAntenna = 1;
   my $sn_NoAntennaD = 2*$sn_NoSat;
   my $sn_SatNM = '   NMLSatNM(1)  = ';
   my $sn_SatModel = '';
   if ( $opt{abs_pc} ) {
      my $pcm_dir = "$opt{sta_info_dir}/phase_center_maps/by_antenna_type.$opt{sat_wwww}";
      my $pcm = sat_pc_maps($pcm_dir,$opt{date});
      my @sat_names = map { (split(/\./,$_))[0] } @$pcm;
         $sn_NoSat = scalar @{$pcm};
         $sn_NoAntennaD = 2*$sn_NoSat;
         foreach my $i ( 0 .. $#sat_names ) {
            $sn_SatNM .= "'$sat_names[$i]'";
            $sn_SatNM .= ',' if $i != $#sat_names;
            $sn_SatNM .="\n".' 'x18 if $i && ! ( $i % 6 );
         } 
      my $ant_no = 1;
      foreach my $i ( 0 .. $#{$pcm} ) {
         my $an = sprintf '%6d', $ant_no++;
         $sn_SatModel .= join '',
                         map { s/Ant_no/$an/g ? $_ : $_ }
                         grep { ! /^(\$sat|\$end)/ }
                         slurp("$pcm_dir/$pcm->[$i]");
         $sn_SatModel .= "\n",
      }
   }
  
   my $tp = slurp( $opt{tpnml} );
  
   my $nml = qq(
\$run_type

      Sim = .FALSE.
      ExtEOPrtls = .FALSE.
      TdpPolyOrd = $tdppo

\$end

\$StaNums

     nmldtyp = 9
!     NMLNoAntennaD = 2
     nmlnosta = $nosta

\$end

\$station_model
!   Epoch:   $epoch
!   @date
     $min_set_partials 
     $tides 
     $station_model 
\$end


\$SatNums

    NMLNoSat      = $sn_NoSat 
    NMLNoAntenna  = $sn_NoAntenna
    NMLNoAntennaD = $sn_NoAntennaD

\$end
 
\$satellite_names
   
$sn_SatNM

\$end

\$ionosphere_mdl

\$end

\$satellite_prtls

\$end

\$satellite_model

$sn_SatModel

\$end

\$exclude_data_types

\$end

\$data_times
   StartTime = '01-jan-1970 00:00:00.0000'
   EndTime   = '01-jan-2100 00:00:00.0000'
\$end

$tp

);

   open( my $fh, '>', 'qregres.nml') 
      or die "Couldn't open qregres.nml for  writing $!\n";
   print $fh $nml;
   return $nml
}


sub sat_pc_maps {
  my $pcm_dir = shift;
  my $date = shift ;
  $date = get_date($date,'cal,num');
  opendir(my $dh , $pcm_dir) or croak "Couldn't open $pcm_dir $!\n";
  @pc_maps = sort grep { /^GPS\d\d\./ } readdir $dh;
  my ($gps,$sdate,$edate);
  my @out=();
  foreach my $pc ( @pc_maps ) {
    ($gps,$sdate,$edate) = split /\./,$pc;
    next if $sdate > $date;
    next if $edate ne '00000000' && $edate < $date;
    push @out,$pc;
  }
  return \@out
}


#=====
1;
