package GOA::QRegres;
use POSIX qw( strftime );
use File::Copy;
use lib "$ENV{GOA}/perl_pm";
use Scalar::Util qw(looks_like_number);
use Carp;
use File::Basename;
use Exporter;
use GOA::Util;
use GOA::StaPos;
use GOA::StaSvec;
use GOA::StaId;
use GOA::PCenter;
use GOA::OceanLoading qw( ocean_loading );
use GOA::TDP;
use GOA::GPT;
use GPS::DATES;
use GPS::Geodesy qw( distance );
use trans;
@ISA = ( 'Exporter' );

@EXPORT = qw( qr_nml );

sub tides {
   my $tides = shift ;
   my @possible_tides = qw( PolTid 
                           WahrK1
                           FreqDepLove
                           OctTid
                           OcnldCpn
			   ocl_n
                         );
   my $nml = "\n";
   foreach my $t ( @possible_tides ) {
     next if  $tides !~ /$t/i;
     next if  $t =~ /WahrK1/i && $tides =~/FreqDepLove/i ;
     if ($t eq 'ocl_n') { $nml .= "    NML$t = 11\n"; }
     else { $nml .= "    NML$t = .TRUE.\n"; }
  } 
                 
  return $nml or croak "No valid tides in |$tides|\n";
}

sub station_model {
   my %opt = ( sta     => '',
               nsta    => 1,
               goa_ant => '',
               date => '',
               rate => 300,
               tides => 'PolTid,WahrK1' ,
               sta_info_dir => "$ENV{GOA_VAR}/sta_info",
               z_dry => 'default',
               z_wet => 'default',
               nml  => {},
               min_set_partials  => '', 
	       grn_wwww => '',
               @_,
   );

   my $sta = uc $opt{sta};
   my $dir = $opt{sta_info_dir};
   my $svec      = GOA::StaSvec->new( file=>"$dir/sta_svec" );
   my $sv        = $svec->sv($sta,$sta,$opt{date});   
   my $svec_file = $svec->file;
      $opt{goa_ant} =  $sv->{pc_ant} if ! $opt{goa_ant};
   my $pc_ant = pack 'A9',$opt{goa_ant};
   my $pos       = GOA::StaPos->new( file => "$dir/sta_pos" );
   my $xyz_pos = $pos->xyz($sta,$opt{date});
   #my $pcenter = GOA::PCenter->new( file => "$dir/pcenter" );
   my ($pcfile) = grep { /$opt{grn_wwww}/ } glob("$dir/phase_center_maps/pcenter.igs*");
   my $pcenter = GOA::PCenter->new( file => $pcfile );
      $pcenter->load("$dir/phase_center_maps/pcenter.ant_info.abs.atx");
   my $pc = $pcenter->get($pc_ant); 
   my $nmlsite = "\n";
   my $nsta = $opt{nsta};
   my $i = 1;
   my $m2km = 1.0e-3; 
   foreach my $dt ( qw/L1 L2 LC P1 P2 PC C2 DR R2/ ) {
      my @v=($m2km)x3;
      $v[0] *= $pc->{$dt}{east}   + $sv->{x};
      $v[1] *= $pc->{$dt}{north}  + $sv->{y};
      $v[2] *= $pc->{$dt}{up}     + $sv->{z} + $sv->{height};
      $nmlsite .= sprintf ' 'x5 . 
                      "NMLSite(1,%2d,%3d) =  %15.7f%15.7f%15.7f\n",
                      $i,$nsta,@v ;
      $i++;                             
   }
   # make xyz in km to use trans::xyz2llh
   my $xyz = [];
      @$xyz = map { $_*$m2km } @{$xyz_pos}; 
   my ($lat,$lon,$h ) = trans::xyz2llh( @$xyz );
      $h*=1000; # km to meters 
   
   my $dry = $opt{z_dry};
   my $wet = $opt{z_wet};
   if ($dry eq 'gpt') {
	   ($dry,$wet) = GOA::GPT::gpt($opt{date},$lat,$lon,$h);
   } elsif ($dry eq 'vmf1') {
           ($dry,$wet) = tropnominal('VMF1GRID',$sta,$opt{date},$lat,$lon,$h);
   } elsif ($dry eq 'default') {
           $dry = 1.013 * 2.27 * exp(-.000116 * $h) if $dry eq 'default';
           $wet = 0.1 if $wet eq 'default';
   }

   $dry = sprintf "%10.4f", $dry;
   $wet = sprintf "%10.4f", $wet;
  
   my $llh = sprintf "!    %12.7f    %12.7f    %10.4f\n\n",$lat,$lon,$h;
   @$xyz = map { sprintf "%18.6f",$_ } @$xyz;
   $ocean_loading = ocean_loading( file_coeffs => "$dir/ocnld_coeffs_cm",
	                           sta => $sta,
                                   nsta => $nsta,
                                   lon => $lon,
                                   lat => $lat,
                                 );
   my $pcm_dir = "$dir/phase_center_maps/by_antenna_type.$opt{grn_wwww}";
   my $goa_ant = trim($pc_ant);
   my $pcm_grn = '';
   if ( ! $opt{min_set_partials} ) {
      if ( ! -e "$pcm_dir/$goa_ant" ) {
         $pcm_dir="$dir/phase_center_maps/by_antenna_type";
      }
      $pcm_grn = join '',
                 map { s/Ant_no/$nsta/g;$_ }
                 grep { ! /^(\$station_model|\$end)/ }
                 slurp("$pcm_dir/$goa_ant");
   }
#follow gd2p to setup nml for single station data
   $TropMap=uc $opt{trop_map};
   
   my $nml = qq(
     NMLname( $nsta ) = '$sta'
     NMLTropMap( $nsta ) = '$TropMap'
     NMLMonument(1, $nsta) = 
                          $xyz->[0]
                          $xyz->[1]
                          $xyz->[2]
!        lat             long          elev (m)
$llh

!   Antenna type: $pc_ant ( from $svec_file )

      NMLTROP_DRY( $nsta ) = $dry 
      NMLTROP_WET( $nsta ) = $wet

$nmlsite
$ocean_loading
$pcm_grn
);

  return $nml

}

sub qr_nml {
   my %opt = ( sta     => '',
               goa_ant => '',
               date => '',
               rate => 300,
               tides => 'PolTid,WahrK1' ,
               sta_info_dir => "$ENV{GOA_VAR}/sta_info",
               z_dry => 'default',
               z_wet => 'default',
               min_set_partials => '',
               tpnml  => 'tpeo.nml',
               grn_wwww => '',
               sat_wwww => '',
               nml  => {},
	       ionex_dir => '',
               @_,
   );
   $opt{sta} = uc $opt{sta};
   my @sites     = split /,/,$opt{sta};
   my $nosta     = @sites;
   my $tides     = tides( $opt{tides} );
   my $station_model = '';
   my $nsta = 1;
   foreach my $s ( @sites ) {
      $opt{sta} = $s;
      $opt{nsta} = $nsta++;
      $station_model  .= station_model(%opt);
   }
   my $min_set_partials = "\n";
   if ( $opt{min_set_partials} ) {
      $min_set_partials .= "  NMLMinimal_set_partials = .TRUE.\n";
   }
   else {
      $min_set_partials .= qq(
      NMLStaPrtl(1,  1) = 
                          'STA X',
                          'STA Y',
                          'STA Z',
                          'WETZTROP',
                          'TRPAZCOS',
                          'TRPAZSIN',
                          'STA BIAS',
                          ' '
);
   }
   my $tdppo = 2; #TdpPolyOrd
   if ( $opt{rate} != 300 ) {
      $tdppo = 3; 
   }
   my $epoch = sprintf "%.2f", get_date( $opt{date},'secj2000');
   my @date = get_date( $opt{date} );
#follow gd2p to setup nml for single station data
   my $sn_NoSat = 0;
   my $sn_NoAntenna = 1;
   my $sn_NoAntennaD = 2*$sn_NoSat;
   my $sn_SatNM = '   NMLSatNM(1)  = ';
   my $sn_SatModel = '';
   my $pcm_dir = "$opt{sta_info_dir}/phase_center_maps/by_antenna_type.$opt{sat_wwww}";
   my $pcm = sat_pc_maps($pcm_dir,$opt{date});
   my @sat_names = map { (split(/\./,$_))[0] } @$pcm;
   $sn_NoSat = scalar @{$pcm};
   $sn_NoAntennaD = 2*$sn_NoSat;
   foreach my $i ( 0 .. $#sat_names ) {
     $sn_SatNM .= "'$sat_names[$i]'";
     $sn_SatNM .= ',' if $i != $#sat_names;
     $sn_SatNM .="\n".' 'x18 if $i && ! ( $i % 6 );
   } 
   my $ant_no = 1;
   foreach my $i ( 0 .. $#{$pcm} ) {
     my $an = sprintf '%6d', $ant_no++;
     $sn_SatModel .= join '',
                         map { s/Ant_no/$an/g ? $_ : $_ }
                         grep { ! /^(\$sat|\$end)/ }
                         slurp("$pcm_dir/$pcm->[$i]");
     $sn_SatModel .= "\n",
   }

   # Ionospheric model
   my ($yyyy,$doy) = split(' ',get_date($opt{date},'doy'));
   my $yy = unpack('x2a2',$yyyy);
   # Set up the ionosphere namelist block to be blank, then if 
   # we don't end up wanting to model the ionosphere, it won't 
   # be included!
   my $ionosphereMDL = '';
   my $ionex_dir = $opt{ionex_dir};
   #my $ionex_file = "$ionex_dir/$yyyy/igsg${doy}0.${yy}i";
   my $ionex_file = "$ionex_dir/$yyyy/jplg${doy}0.${yy}i";
   # Check Ionosphere, does this file exist? Do we want to estimate ionosphere?
   # If we do not want to estimate 2nd-order ionosphere, we need to leave
   # the $ionex_dir variable empty
   if ( -e $ionex_file ) {
       $ionosphereMDL =    "   Ion_2nd = .true\n";
       $ionosphereMDL .=   "   mag_mdl = 'IGRF'\n";
       $ionosphereMDL .=   "   tec_mdl = 'IONEX'\n";
       $ionosphereMDL .=   "   ionex_file = '$ionex_file'\n";
       $ionosphereMDL .=   "   Ion_2nd_shell_hgt = 600\n";
   } else {
	   warn "IONEX file $ionex_file does not exist\n";
   }

   my $tp = slurp( $opt{tpnml} );
  
   my $nml = qq(
\$run_type

      Sim = .FALSE.
      ExtEOPrtls = .FALSE.
      TdpPolyOrd = $tdppo

\$end

\$StaNums

     nmldtyp = 9
!     NMLNoAntennaD = 2
     nmlnosta = $nosta

\$end

\$station_model
!   Epoch:   $epoch
!   @date
     $min_set_partials 
     $tides 
     $station_model 
\$end


\$SatNums

    NMLNoSat      = $sn_NoSat 
    NMLNoAntenna  = $sn_NoAntenna
    NMLNoAntennaD = $sn_NoAntennaD

\$end
 
\$satellite_names
   
$sn_SatNM

\$end

\$ionosphere_mdl

$ionosphereMDL

\$end

\$satellite_prtls

\$end

\$satellite_model

$sn_SatModel

\$end

\$exclude_data_types

\$end

\$data_times
   StartTime = '01-jan-1970 00:00:00.0000'
   EndTime   = '01-jan-2100 00:00:00.0000'
\$end

$tp

);

   open( my $fh, '>', 'qregres.nml') 
      or die "Couldn't open qregres.nml for  writing $!\n";
   print $fh $nml;
   return $nml
}


sub sat_pc_maps {
  my $pcm_dir = shift;
  my $date = shift ;
  $date = get_date($date,'cal,num');
  opendir(my $dh , $pcm_dir) or croak "Couldn't open $pcm_dir $!\n";
  @pc_maps = sort grep { /^GPS\d\d\./ } readdir $dh;
  my ($gps,$sdate,$edate);
  my @out=();
  foreach my $pc ( @pc_maps ) {
    ($gps,$sdate,$edate) = split /\./,$pc;
    next if $sdate > $date;
    next if $edate ne '00000000' && $edate < $date;
    push @out,$pc;
  }
  return \@out
}

sub tropnominal {
  my($model,$name,$date,$lat,$lon,$h) = @_;

  my $start = get_date( $date,'secj2000');
  my $end = $start + 86400;
  # Run tropnominal to create a TDPfile for the wet and dry component 
  my $arguments = "$model $name $lat $lon $h $start $end 21600";
  print "Running tropnominal.e $arguments\n";
  `tropnominal.e $arguments`;

  # Now parse the created TDPfile and extract the first values as aprioris
  my $filename = "$name.TDPdry";
  my $tdp = GOA::TDP->new($filename);
  my $data = $tdp->tdp();
  my $dry = $data->{"DRYZTROP$name"}->[0]->{'val'};

  $filename = "$name.TDPwet";
  $tdp = GOA::TDP->new($filename);
  $data = $tdp->tdp();
  my $wet = $data->{"WETZTROP$name"}->[0]->{'val'};

  # Check to see if the DRYZ component is already in the TDPfile
  my $return = system('grep -q "DRYZ" TDPfile');
  $return = $return >> 8;
  # grep returns non-zero if there is no match
  # so, if there is no match, add in the DRY and WET values 
  # from tropnominal
  if ($return != 0) {
	  # Concatenate the estimate of the dry component to the TDPfile
	  `cat TDPfile $name.TDPdry $name.TDPwet > foo1`;
	  `cp foo1 TDPfile`;
  }

  print "Dry: $dry\nWet: $wet\n";
  return $dry,$wet
}


#=====
1;
