package GOA::Shadow;
use Carp;
use GPS::DATES;
use GPS::PRN_GPS;
my $pg = GPS::PRN_GPS->new();

sub new { 
    my $class = shift;
    my $self = { file => 'Shadow',
                 sats => {},
                 nsats => 0,
               };
    $self->{file} = shift if @_;
    bless $self,$class;       
    $self->load();
    return $self
}

sub file {
  my $self = shift;
  if ( @_ ) {
    $self->{file} = shift;
  }
  return $self->{file}
}

sub gps {
  my $self = shift;
  my @gps = sort keys %{$self->{sats}};
  return wantarray ? @gps : \@gps 
}

sub list {
  my $self = shift;
  my $gps =  shift || '';
  if ( $gps !~ /^\d+$/ && $gps !~ /^GPS\d+$/ ) {
     croak "Invalid GPS sat name:$gps\n";
  }
  if ( $gps =~ /^\d+$/ ) {
    $gps = "GPS$gps";
  }
  my (@sats,$h);
  foreach my $g ( $self->gps ) {
     next if $gps && $gps ne $g;
     foreach my $h ( @{$self->{sats}{$g}} ) {
       push @sats, $h;
     }
  }
  return wantarray ? @sats : \@sats

}
sub load {
   my $self = shift;
   my $file = $self->file(@_);
   my ($prn,$gps,$date,$event,$hms,$secs);
   open(my $fh,'<',$file) or croak "Couldn't open $file $!\n";
   while ( <$fh> ) {
     if ( /^NUMBER / ) {
       $self->{nsats} = (split(/:/,$_))[-1];
     }
     if ( /^GPS/ ) {
       ($gps,$event,$date,$hms)=split(' ',$_);
        $secs = get_date("$date $hms",'secj2000');
        $prn = $pg->prn( gps => $gps, date => $date );
       push @{$self->{sats}{$gps}},{ event => $event,
                                     date => "$date $hms",
                                     secs => $secs,
                                     gps => $gps,
                                     prn => $prn,
                                   };
     }
   } 
   return $self
}

1;
