package GOA::StaId;
use Fortran::F90Format;
use Carp;
use GPS::DATES;
use GPS::StrUtil;
use GPS::Defaults;
use File::Basename;

my %par = get_defaults();
my @fields = qw( sta 
                 id 
                 name 
                 iers_domes 
                 location 
                 net_info
                 comment
               );
# sta_id format:  (1x,a4,i6,1x,a60)
# modified format to allow alias to contain extra info
# alias ( a60 ) is now:
# alias = name,iers_domes,city_town,country,state_province
# alias = a16,1x,a9,1x,a20,1x,a7,1x,a4
# new fmt = '1x,a4,i6,1x,a16,1x,a9,1x,a20,1x,a7,1x,a4,1x,a',
my $fmt = '1x,a4,'.
          'i6,'.
          '1x,a16,'.
          '1x,a9,'.
          '1x,a20,'.
          '1x,a7,'.
          '1x,a';

my @today = get_date('today');

my %ROW  = (   sta        => '',
               id         => 0 ,
               name       => '',
               iers_domes => '',
               location   => '',
               net_info   => '',
               comment    => "Generated on @today by VMS",
             );

sub new {
  my $class = shift;
    my $self = { };
    my %opt = (
      @_,
    );

    bless $self,$class;
     
    $self->{fmt} = Fortran::F90Format->new( fmt => $fmt );
    
    $self->{overwrite} = 0;
    
    $self->{file} = "$par{STA_INFO_DIR}/sta_id";
    
	if ( $opt{file} ) {
	  $self->file($opt{file});
      $self->load(%opt);
	}
    return $self

}
sub overwrite {
  my $self = shift;
  my $o    = shift || 0;
  $self->{overwrite}  = $o ? 1 : 0;
  return $self->{overwrite}
}

sub file {
  my $self = shift;
  if ( @_ ) {
    my $f = shift;
	if ( -e $f ) {
	  $self->{file} = $f;
	}
	else {
	  die "File $f doesn't exists \n";
	}
  }
  return $self->{file}
}

sub fmt {
  my $self = shift;
  return $self->{fmt}
}

sub load {
  my $self = shift;
  my %opt  = ( @_,
              );
              
  my $file = -e $opt{file} ? $opt{file} : $self->file;
  open( $fh,"<", $file ) or die "Couldn't open $file $!\n";
  my ($h,$secs);
  while( <$fh> ) {
    chomp;
    my $h = {};   
    @{$h}{@fields}=$self->fmt->read($_);
    
    @{$self->{Id}{ uc $h->{sta} }
                 { $h->{id}     }
                }{ qw/ name
                       iers_domes
                       location
                       net_info
                       comment /  } = @{$h}{qw/name  
                                               iers_domes
                                               location
                                               net_info
                                               comment / };
                                                   
                 
                 
  }
  return $self
}


sub add {
    my $self = shift;
    my %e    = ( %ROW,
                 @_,
                ); 

    foreach my $f ( keys %e ) { delete $e{$f} if ! exists $ROW{$f}; } 
    $self->set( @e{@fields} ); 
    
}

sub set {
   my $self = shift;
   my @vals = @_ == @fields ? @_ : croak "Incorrect number of fields\n";
   my $h = {};

   @{$h} {@fields} = @ROW{@fields};
   @{$h}{@fields}=@vals;
 
    $h->{sta}   = uc $h->{sta};
    $h->{name}  = $h->{sta} if ! trim($h->{name})  ;
    
    my $ok = $h->{sta}      !~ /^\w{4}$/   ?  0 : 
                                              1 ;
    if ( $ok ) { # add new entry
       if ( exists $self->{Id}{$h->{sta}}{id}   ) {
           carp "$h->{sta} $h->{id} already exists\n";
           return unless $self->{overwrite} 
       }
       #print map { "$_ |$h->{$_}|\n" } sort keys %$h;
       $h->{id} = sprintf '%6d',$h->{id};
       @{$self->{Id}{ $h->{sta} }
                    { $h->{id}  } } #slice
                    { qw/name 
                         iers_domes 
                         location 
                         net_info 
                         comment/      }
                                        = @{$h}{ qw/name 
                                                    iers_domes 
                                                    location
                                                    net_info 
                                                    comment / };
   } #ok  
   else {
       carp "Not enough information to add a new row (sta)\n";
   } 
}

sub select {
    my $self = shift;
    my $sta  = uc shift || '';
    my $hid = $self->get($sta);
    my $rows = [];
    
    foreach my $id ( sort { $b <=> $a } keys %{$hid} ) {
      my $hh = $self->get($sta,$id);
      my $h={};
      @{$h}{@fields} = ( $sta, 
                         $id,
                         @{$hh}{ qw/name 
                                    iers_domes 
                                    location
                                    net_info 
                                    comment / }
                       );
      push @$rows,$h; 
    }
    
    return $rows
}

sub get {
   my $self  = shift;
   my $sta   = shift || '';
   my $id    = sprintf "%6d",shift ;
   $id = '' if $id !~ /\d+/;
   my $s = $self->{Id};
      $s = $s->{$sta}   if $sta  && exists $s->{$sta}; 
      $s = $s->{$id}    if $id ne '' && exists $s->{$id}; 
   return $s 
}

sub print {
   my $self = shift;
   my %opt = (
        file => '',
        @_,
   );
   my $fh; 
   if ( $opt{file} ) {
      open( $fh , "> $opt{file}") 
        or croak "Couldn't open $opt{file} $!\n";
   }
   else {
     $fh=\*STDOUT;
   }
   my $hsta = $self->get;
   foreach my $st ( sort keys %{$hsta} ) {
         foreach my $id ( sort { $b <=> $a } keys %{$hsta->{$st}} ) {
             my $hh = $self->get($st,$id);
             my @v = ( uc $st,
                       $id,
                       @{$hh}{ qw/name 
                                  iers_domes 
                                  location 
                                  net_info
                                  comment /   }
                      );
            #print "|",join '|',@v,"\n";
            print $fh $self->fmt->write(@v);
        }
   }
   close $fh if $opt{file};
}

1;









