package GOA::StaLoc;
use Fortran::F90Format;
use Carp;
use GPS::DATES;
use GPS::StrUtil;
use GPS::Defaults;
use File::Basename;

my %par;
my @fields = qw( sta 
                 lon lat 
                 region
                 type 
                 id16char 
                 tectonic_group
                 status 
                 orig_network 
                 location);
                 
my $fmt = 'A4,'.
          '2(2X,F9.4),'.
          '2X,A9,'.
          '1X,A10,'.
          '1X,A16,'.
          '1X,A11,'.
          '1X,A16,'.
          '1X,A10,'.
          '1X,A'; 

my @today = get_date('today');

my %ROW  = (   sta        => '',
               lon        => '',
               lat        => '',
               region     => '',
               type       => sprintf("%-10s",'CGPS'),
               id16char   => '',
               tectonic_group => '',
               status      => '',
               orig_network => '',
               location   => '',
             );

sub new {
  my $class = shift;
    my $self = { };
    my %opt = (
	  file => '',
      @_,
    );

    bless $self,$class;
   
    %par = get_defaults( );
     
    $self->{fmt} = Fortran::F90Format->new( fmt => $fmt );
    
    $self->{overwrite} = 1;
    $self->{dir}  = $par{STA_INFO_DIR};
    $self->{file} = "$par{STA_INFO_DIR}/sta_loc";
    
	if ( $opt{file} ) {
	  $self->file($opt{file});
      $self->load(%opt);
	}
    
    return $self

}
sub overwrite {
  my $self = shift;
  my $o    = shift || 0;
  $self->{overwrite}  = $o ? 1 : 0;
  return $self->{overwrite}
}

sub file {
  my $self = shift;
  if ( @_ ) {
    my $f = shift;
	if ( -e $f ) {
	  $self->{file} = $f;
	}
	else {
	  die "File $f doesn't exists \n";
	}
  }
  return $self->{file}
}

sub fmt {
  my $self = shift;
  return $self->{fmt}
}

sub load {
  my $self = shift;
  my %opt  = (  @_,
              );
              
  my $file = -e $opt{file} ? $opt{file} : $self->file;
  open( $fh,"<", $file ) or die "couldn't open $file $!\n";
  my ($h,$secs);
  while( <$fh> ) {
    chomp;
    my $h = {};   
    @{$h}{@fields}=$self->fmt->read($_);
    
    #next if exists $self->{Loc}{$h->{sta}};
    
    @{$self->{Loc}{ $h->{sta} }
                 }{ qw/lon 
                       lat 
                       region
                       type 
                       id16char 
                       tectonic_group
                       status 
                       orig_network
                       location/  } = @{$h}{qw/lon 
                                               lat 
                                               region
                                               type 
                                               id16char
                                               tectonic_group
                                               status
                                               orig_network
                                               location/ };
                                                   
                 
                 
  }
  return $self

}


sub add {
    my $self = shift;
    my %e    = ( %ROW,
                 @_,
                ); 

    foreach my $f ( keys %e ) { delete $e{$f} if ! exists $ROW{$f}; } 
    $self->set( @e{@fields} ); 
    
}

sub set {
   my $self = shift;
   my @vals = @_ == @fields ? @_ : croak "Incorrect number of fields\n";
   my $h = {};

   @{$h} {@fields} = @ROW{@fields};
   @{$h}{@fields}=@vals;
 
    $h->{sta}   = uc $h->{sta};
    my $ok = $h->{sta} !~ /^\w{4}$/   ?  0 : 
                                         1 ;
    if ( $ok ) { # add new entry
       if ( exists $self->{Loc}{$h->{sta}} && ! $self->{overwrite}  ) {
           carp "$h->{sta} entry already exists\n";
           return 0;
           #return unless $self->{overwrite} 
       }
       
       @{$self->{Loc}{ $h->{sta} }
                    }{qw/lon 
                       lat 
                       region
                       type 
                       id16char 
                       tectonic_group
                       status 
                       orig_network
                       location/  } = @{$h}{qw/lon 
                                               lat 
                                               region
                                               type 
                                               id16char
                                               tectonic_group
                                               status
                                               orig_network
                                               location/ };

   } #ok  
   else {
       carp "Not enough information to add a new row ($h->{sta})\n";
   } 
}

sub select {
    my $self = shift;
    my $sta  = uc shift || '';
    my $hsta = $self->loc($sta);
    my $rows = [];
    
    foreach my $s ( sort keys %{$hsta} ) {
      my $hh = $self->loc($sta);
      my $h={};
      @{$h}{@fields} = ( $sta, 
                         @{$hh}{  qw/lon 
                                     lat 
                                     region
                                     type 
                                     id16char
                                     tectonic_group
                                     status
                                     orig_network
                                     location/ },
                       );
      push @$rows,$h; 
    }
    
    return $rows
}
sub loc {
   my $self  = shift;
   my $sta   = shift || '';
   my $s = $self->{Loc};
      $s = $s->{$sta}   if $sta  && exists $s->{$sta}; 
   return $s 
}

sub print {
   my $self = shift;
   my %opt = (
        file => '',
        @_,
   );
   my $fh;
   if ( $opt{file} ) {
      open( $fh , "> $opt{file}") 
        or croak "Couldn't open $opt{file} $!\n";
   }
   else {
     $fh = \*STDOUT;
   }
   my $hsta = $self->loc;
   foreach my $st ( sort keys %{$hsta} ) {
         my $hh = $self->loc($st);
         my @v = ( uc $st,
                   @{$hh}{qw/lon
                             lat
                             region
                             type
                             id16char
                             tectonic_group
                             status
                             orig_network
                             location
                             /} 
                      );
            #print "|",join '|',@v,"\n";
            print $fh $self->fmt->write(@v);
   }

   close $fh if $opt{file};
}

1;









