package GOA::StaRcvr;
use Fortran::F90Format;
use Carp;
use GPS::DATES qw( get_date cal2sec sec2cal );
use GPS::StrUtil;
use GPS::Defaults;
use File::Basename;

my @fields = qw( sta 
                 year  month day hours  min sec
                 duration
                 receiver  
                 sn  
                 firmware  
                 comment
               );

my $fmt  = '1x,a4,'.
           '1x,i4,4(1x,i2),1x,f5.2,'.
           '1x,f12.2,'.
           '1x,a20,'.
           '1x,a5,'.
           '1x,a11,'.
           '1x,a';
my $fmt2 = 'a4,'.
           '1x,i4,4(1x,i2),1x,f5.2,'.
           '1x,i4,4(1x,i2),1x,f5.2,'.
           '1x,a20,'.
           '1x,a5,'.
           '1x,a11,'.
           '1x,a';
my @fields2 = qw( sta 
                 year  month day hours  min sec
                 eyear emonth eday ehours  emin esec
                 receiver  
                 sn  
                 firmware  
                 comment
               );
        
my @today = get_date('today');

my %ROW  = (   sta   => '',
               year     => 0,
               month    => 0,
               day      => 0,
               hours    => 0,
               min      => 0,
               sec      => 0,
               duration => 946080000.00, # 30 years
               receiver  => '', 
               sn        => '',
               firmware  => '',
               comment   => "Generated on @today by VMS",
             );
my %par;

sub new {
  my $class = shift;
    my $self = { };
    my %opt = (
      @_,
    );

    bless $self,$class;
   
    %par = get_defaults( );
     
    $self->{fmt} = Fortran::F90Format->new( fmt => $fmt );
    #$self->{fmt2} = Fortran::F90Format->new( fmt => $fmt2 );
    
    $self->{overwrite} = 0;
    
    $self->{file} = "$par{STA_INFO_DIR}/sta_rcvr";
    
	if ( $opt{file} ) {
	  $self->file($opt{file});
      $self->load(%opt);
	}
    
    return $self

}

sub overwrite {
  my $self = shift;
  my $o    = shift || 0;
  $self->{overwrite}  = $o ? 1 : 0;
  return $self->{overwrite}
}

sub file {
  my $self = shift;
  if ( @_ ) {
    my $f = shift;
	if ( -e $f ) {
	  $self->{file} = $f;
	}
	else {
	  die "File $f doesn't exists \n";
	}
  }
  return $self->{file}
}

sub fmt {
  my $self = shift;
  return $self->{fmt}
}

sub load {
  my $self = shift;
  my %opt  = ( @_,
              );
  my $file = -e $opt{file} ? $opt{file} : $self->file;
  open( $fh,"<", $file ) or die "Couldn't open $file $!\n";
  my ($h,$secs,$psecs,$psta,$duration);
  while( <$fh> ) {
    chomp;
    my $h = {};   
    @{$h}{@fields}= $self->fmt->read($_);
    #@{$h}{@fields2}= $self->{fmt2}->read($_);
    $secs = cal2sec( @{$h}{qw/year month day hours min sec/} );
 #   $duration = $psecs - $secs; 
 #   if ( ! $psta || $psta ne $h->{sta} ) { 
 #       $psta = $h->{sta}; 
 #       $psecs = 0.0;
 #       $duration = 946080000.00; # 30 years
 #   }
    #my $receiver = pack 'A20',trim($h->{receiver}); 
    #$h->{receiver}=$receiver;
    @{$self->{Rcvr}{ $h->{sta}   }
                   { $secs       }
                   }{ qw/duration
                         receiver  
                         sn  
                         firmware
                         comment /  } = @{$h}{ qw/ duration
                                                   receiver  
                                                   sn  
                                                   firmware
                                                   comment / };
                                        
                                         
    #my @v = (  $h->{sta},
    #           @{$h}{qw/year month day hours min sec/} ,
    #           $duration,
    #           @{$h}{ qw/receiver  
    #                     sn  
    #                     firmware
    #                     comment / }
    #         );                                     
    #$psecs = $secs;                              
    #print $self->fmt->write( @{$h}{@fields} );
    #print "@{$h}{@fields}\n";
    #print '|',join '|',@v,"\n";
  }
  #$self->print(); 
  return $self

}


sub add {
    my $self = shift;
    my %e    = ( %ROW,
                 @_,
                ); 

    if ( $e{date} ) {
      @e{ qw/year month day hours min sec/} = get_date($e{date});
    }

    foreach my $f ( keys %e ) { delete $e{$f} if ! exists $ROW{$f}; } 

    $self->set( @e{@fields} ); 
    
}

sub set {
   my $self = shift;
   my @vals = @_ == @fields ? @_ : croak "Incorrect number of fields\n";
   my $h = {};

   @{$h} {@fields} = @ROW{@fields};
   @{$h}{@fields}=@vals;
 
    $h->{sta}   = uc $h->{sta};
    my $ok = $h->{sta}      !~ /^\w{4}$/   ?  0 : 
             $h->{year}     !~ /^\d{4}$/   ?  0 :
             $h->{month}    !~ /^\d{1,2}$/ ?  0 :
             $h->{day}      !~ /^\d{1,2}$/ ?  0 :
                                              1 ;
    if ( $ok ) { # add new entry
       my $secs = cal2sec( @{$h}{qw/year month day hours min sec/} );
       if ( exists $self->{Rcvr}{$h->{sta}}
                          { $secs         }     ) {
           #carp "$h->{sta} $secs already exists\n";
           return unless $self->{overwrite} 
       }
       $h->{receiver}= pack 'A20',trim($h->{receiver}); 
       @{$self->{Rcvr}{ $h->{sta}   }
                      { $secs          } } #slice
                      { qw/duration
                           receiver
                           sn
                           firmware
                           comment /
                       } 
                        = @{$h}{ qw/duration 
                                    receiver
                                    sn
                                    firmware
                                    comment /
                                };
   } #ok  
   else {
     carp "Not enough information to add a new row (sta,date,x,y,z)\n";
   } 
}

sub select {
    my $self = shift;
    my $sta  = uc shift || '';
    my $hsecs = $self->get($sta);
    my $rows = [];
    my ($y,$m,$d,$hours,$min,$sec);
    
    foreach my $ss ( sort { $b <=> $a } keys %{$hsecs} ) {
      ($y,$m,$d,$hours,$min,$sec)=sec2cal($ss);
      my $hh = $self->get($sta,$ss);
      my $h={};
      @{$h}{@fields} = ( $sta, 
                         $y,$m,$d,$hours,$min,$sec,
                         @{$hh}{ qw/duration
                                    receiver  
                                    sn  
                                    firmware 
                                    comment / }
                       );
      push @$rows,$h; 
    }
    
    return $rows
}

sub get {
   my $self  = shift;
   my $sta   = shift || '';
   my $secs  = shift || '';
   my $s = $self->{Rcvr};
      $s = $s->{$sta}   if $sta  && exists $s->{$sta}; 
      $s = $s->{$secs}  if $secs && exists $s->{$secs}; 
   return $s 
}

sub print {
   my $self = shift;
   my %opt = (
        file => '',
        @_,
   );
   my $fh;
   if ( $opt{file} ) {
      open( $fh , "> $opt{file}") 
        or croak "Couldn't open $opt{file} $!\n";
   }
   else {
     $fh = \*STDOUT;
   }
   my $hsta = $self->get;
   foreach my $st ( sort keys %{$hsta} ) {
         my $hsecs = $self->get($st);
         $ssp = 'begin';
         foreach my $ss ( sort { $b <=> $a } keys %{$hsecs} ) {
           my $hh = $self->get($st,$ss);
           $hh->{duration} = $ssp eq 'begin' ? 946080000.00 : $ssp - $ss;
           my ($y,$m,$d,$hours,$min,$sec)=sec2cal($ss);
                   
           my @v = ( $st,  
                     $y,$m,$d,$hours,$min,$sec,
                     @{$hh}{ qw/duration
                                receiver  
                                sn  
                                firmware 
                                comment /   }
                    );
                #print "$ssp $ss\n"; 
                #print "@v\n";
               print $fh $self->fmt->write(@v);
               $ssp = $ss;
        }
   }
 
   close $fh if $opt{file};

}

1;









