package GOA::TDP;
use Carp;
use Fortran::F90Format;
use GPS::DATES;

my $fmt_str = '1X,F14.4,1X,F17.9,1X,F16.9,2X,E13.3,2X,A16';
sub new {
 my $class = shift;
 my $self = { tdp => {},
              file => 'TDPfile',
              };
    $self->{file} = shift if @_ ; 
    $self->{fmt} = Fortran::F90Format->new( fmt => $fmt_str);
 bless $self,$class;
 $self->load();
 return $self
}
sub fmt {
  $self=shift;
  if ( @_ ) {
    $self->{fmt} = Fortran::F90Format->new( fmt => shift);
  }
  return $self->{fmt};
}
sub tdp {
  my $self = shift;
  return $self->{tdp}
}
sub file {
  my $self = shift;
  if ( @_  ) {
     $self->{file} = shift;
  }
  return $self->{file}
}
sub load {
  my $self = shift;
  my $file = $self->file( @_ ) ;
  open(my $fh,'<',$file) or croak "Couldn't open $file $!\n";
  my ($secs,$nval,$val,$sig_flag,$name);
  my @f;
  my $tdp = $self->tdp;
  while ( <$fh> ) {
     chomp;
     s/#.*$//;
     #($secs,$nval,$val,$sig,$name)=$self->fmt->read($_);
     @f=split(' ',$_);
     ($secs,$nval,$val,$sig)=@f[0..3];
     $name = "@f[4..$#f]";
     push @{$tdp->{$name}},{ secs => $secs, 
                             nom_val => $nval,
                             val => $val,
                             sig => $sig,
                           };
  }
  return $tdp
}

sub sats {
  my $self = shift;
  my @list = map { unpack('x8a5',$_)  } 
             grep { /SAT BIAS/ } 
             $self->names;
  return wantarray ? @list : \@list
}

sub sat_bias {
  my $self = shift;
  my $n = shift || ''; # GPS23
  $n =~ s/^GPS//i;
  my $gps ='';
  if ( $n =~ /^\d\d$/ ) {
     $gps = "SAT BIASGPS$n";
  }
  if ( exists $self->tdp->{$gps} ) {
    my $table = $self->tdp->{$gps};
    return wantarray ? @{$table} : $table 
  }
  else {
    return undef
  }
}

sub names {
  my $self = shift;
  my @names = sort keys %{$self->tdp};
  return wantarray ? @names : \@names
}

sub print {
   my $self = shift;
   my @fields = qw( secs nom_val val sig);
   foreach my $name ( $self->names ) {
     foreach my $h ( @{$self->tdp->{$name}} ) {
       print ' ',join ' ',@{$h}{@fields},' ',$name,"\n";
     }
   }
}

1;
