package GOA::TdpSol;
use GOA::Util;
use GPS::DATES;
use GPS::Geodesy;

sub new {
  my $class = shift;
  my $self = {  smsol_file => 'xyz_smsol.txt',
                tdp_file   => 'tdp_final',
				corr_file  => 'tdp_corr_final',
                xyz => {},
				tdp   => {},
				cc  => {},
				nominal => {},
                site => '',
				epoch => '',
				sec_epoch => 0.0,
			  };
  bless $self,$class;
  
}
sub site {
  my $self = shift;
  return $self->{site}
}
sub nxyz {
  my $self = shift;
  my @xyz = @{$self->{nominal}}{qw(X Y Z)};
  return wantarray ? @xyz : \@xyz
}
sub smsol {
   my $self = shift;
   return $self->{smsol}
}
sub xyz {
  my $self = shift;
  return $self->{xyz}
}
sub cc {
  my $self = shift;
  return $self->{cc}
}
sub read_smsol {
  my $self = shift;
  my $f = shift;
  open(my $fh,'<',$f) or die "Couldn't open $f $!\n";
  my($comp,$site,$epoch,$nominal,$secs,$val,$sig,$cal_time);
  $self->{xyz}={};
  while( <$fh> ) {
    chomp;
    s/^\s+//;
	s/\s+$//;
	next unless length;
    next if  /SECONDS PAST/;
	next if /^---+/;

	#if ( /NOMINAL: (.*?)$/ ) {
	#   $nominal=$1;
	#   $self->{nominal}{$comp}=$nominal*1e3;
	#  # print "$sec_epoch $comp $site $epoch $nominal\n";
	#  next;
	#}
	if ( /EPOCH: (.*?)$/ ) {
	   $epoch=$1;
	   $cal_time= [ get_date($epoch) ];
	   $sec_epoch=`cal2sec @$cal_time`; chomp $sec_epoch;
	   $self->{epoch}=$epoch;
	   $self->{sec_epoch}=$sec_epoch;
	   next;
	}
	if ( /STA (X|Y|Z)   (....)\s+NOMINAL: (.*?)$/ ) {
	   $nominal=0;
	   $comp=$1;
	   $site=$2;
	   $nominal=$3;
	   $self->{nominal}{$comp}=$nominal*1e3;
	   $self->{site}=$site;
	   next;
	}

    ($secs,$val,$sig)=split(' ');
	($val,$sig) = map { $_*1e3 } ($val,$sig);
	$secs = sprintf ("%.1f",$secs+$sec_epoch);
    
    $self->{xyz}{$secs}{$comp}={ val => $val+$self->{nominal}{$comp} , sig => $sig};
  }

}

sub read_tdp {
  my $f = shift;
  open(my $fh,'<',$f) or die "Couldn't open $f $!\n";
  my $xyz={};
  my ($secs,$apr,$val,$sig,$comp,$site,$epoch);
  while( <$fh> ) {
     chomp;
     if ( /STA [X|Y|Z]/ ) {
	   ($secs,$apr,$val,$sig,undef,$comp,$site)=split(' ');
	   next if $comp ne 'Y';
	   my $epoch=`sec2cal $secs`; chomp $epoch;
	   $epoch=[split(' ',$epoch)];
       print join ' ',($secs,$apr,$val,$sig,$comp,$site,@$epoch),"\n";

	 }
	 else {
	   next;
	 }
  }
  close $fh;
}

sub read_corr {
  my $self = shift;
  my $f=shift;
  open(my $fh,'<',$f) or die "Couldn't open $f $!\n";
  my ($ttag,$secs,$epoch,$c1,$c2,$val,);
  $self->{cc} ={};
  while( <$fh> ) {
    chomp;
    if ( /^CORRELATION TIMETAG: (.*?)$/ ) {
	   $ttag=trim($1); 
	}
    elsif ( /^STA/ ) {
	  ($c1,$c2,$val,$secs)=split(' ');
	   $c1=unpack('x3a1',$c1);
	   $c2=unpack('x3a1',$c2);
       #@cal = GPS::DATES::sec2cal($secs);
       #$cal[5]=int($cal[5]);
       next if $c1 eq $c2;
	   #print "$secs $c1 $c2 $val\n";
       $self->{cc}{$secs}{$c1}{$c2}=$val;
	}

  }

}
1;
