package GPS::APC;
use Carp;
use GPS::RcvrAnt;
use GPS::Antex;
use GPS::PCV;
#use GPS::PRN_GPS;
use GPS::DATES;

#my $Time_Stamp = `date`;chomp $Time_Stamp;
my $Time_Stamp = get_date('today','cal,time');
my $cr_yyyy = unpack('a4',$Time_Stamp);  # original year 2007
my  $ra = GPS::RcvrAnt->new();
our ($atx,$pcv);

sub new {
  my $class = shift;
  my $self =  {};
  bless $self,$class;
  return $self->init(@_) 
}

sub init {
  my $self = shift;
  my %opt  = (
     file => '',
     @_,
  );
  if (  $opt{file} =~  /ant_info\.abs$/ ) {
    $pcv = GPS::PCV->new(%opt);
    return $self->load_pcv( %opt )
  }
  elsif ( $opt{file} =~ /\.atx$/ ) {
    $atx = GPS::Antex->new( $opt{file},$opt{skip_freq_values} );
    $atx->read_atx("$ENV{GPS_HOME_DIR}/igs_info/ant_info.abs.atx");
    return $self->load_atx(%opt)
  }
  else {
    croak "You must supply an antex or pcv file\n".
          "Ex: igs05_1402.atx /ant_info.abs \n"; 
  }

}
	     
sub get_vendor {
    my $self = shift;
    my %opt  = (
       igs_code => '',
       vendor   => '',
       @_,
    );

    my $c  = $opt{igs_code};
    my $ac = pack("a3",$c);
    my %vc = %other_vendor_codes; 
    my $v_ra  = '';
    if ( $ra->{antennae}{trim($c)}{vendor}  !~ /(OTHER|SATELLITE)/ ) { 
       $v_ra = $ra->{antennae}{trim($c)}{vendor};
    }
    # print map{ "$_ $vc{$_}\n"} keys %vc;
    #exit; 
    
    my $v  = trim($opt{vendor}) || $self->{$c}{vendor};
    
    
    if ( $ac !~ /\d\d\d/ ) {
       $v  = $vc{$ac} if  $v ne $vc{$ac} ;
    } 	    
    
    $v ||= $v_ra;  #assign from rcvr_ant.tab if not set yet
    
    if ( exists $vendor_alias{trim($v)} ) {
      $v = $vendor_alias{trim($v)};
    }
   
    #print "$c | $v |$ac $vc{$ac}| $v_ra | ",$vendor_alias{trim($v)},"$opt{file}\n";
    

    return $v || $c #return same code if there isn't a match
}
sub load_pcv {
  my $self =  shift;
  my $list =  $pcv->list();
  my ($vendor,$goa_name);
  foreach my $igs_code ( @$list ) {
  
     next if exists $self->{$igs_code};
	 
     $vendor   = $ra->get_vendor( $igs_code );
     
	 $pcv->set( igs_code => $igs_code,
                 field    => 'vendor',
                 value    => $vendor
               );
     $goa_name = $ra->goa_name(  $igs_code, $self );
     
	 $pcv->set( igs_code => $igs_code,
                 field    => 'goa_name',
                 value    => $goa_name,
               );
         
     $self->{$igs_code} = $pcv->{$igs_code};

  }
  return $self
}

sub load_atx {
    my $self = shift;
     
    my $desc=''; 
    foreach my $igs_code ( $atx->list ) {
     
     $atx->{antennae}{$igs_code}{comment}     = pack 'A16',$atx->file;
     $desc = $igs_code =~ /(BLOCK I|GLONASS)/ ? '|Satellite|':'|Ground|';
     $desc .= $atx->{antennae}{$igs_code}{atx_comment}||'';
     
     $atx->{antennae}{$igs_code}{description} = $desc ;

     $vendor   = $ra->get_vendor( $igs_code );
     $atx->{antennae}{$igs_code}{vendor}=$vendor;

     $goa_name = $ra->goa_name( $igs_code, $atx->{antennae} );

     $atx->{antennae}{$igs_code}{goa_name}=$goa_name;
     
     @{$atx->{antennae}
             {$igs_code}
             {l1_offset}}= @{$atx->{antennae}
                                   {$igs_code}
                                   {frequency}{G01}}
                                   {qw/north east up/};
     @{$atx->{antennae}
             {$igs_code}
             {l2_offset}}= @{$atx->{antennae}
                                   {$igs_code}
                                   {frequency}{G02}}
                                   {qw/north east up/};

     $self->{$igs_code} = $atx->{antennae}{$igs_code};
     
     
  }

  return $self
     

}

sub list {
   my $self = shift;
   my $list = lc shift || 'igs';
   my @names=();
   #my $none = 'NONE'.' 'x12 .'NONE';
   my $none = ' 'x16 .'NONE';
   if ( $list eq 'igs' ) {
      foreach my $a ( sort keys %$self ) {
        $a = $a eq  'NONE'.' 'x12 .'    ' ? $none : #' 'x16 .'NONE' :
             $a eq   ' 'x16 .'NONE'       ? $none :
                                            $a;
        push @names, $a;
      }
   }
   if( $list eq 'goa' ) {
      $goa_names =  $self->goa_names();
      @names = sort map { $goa_names->{$_} } keys %$self;
   }
   
   return wantarray ? @names : \@names ;
  
}
sub goa_names {
   my $self = shift;
   my $goa_names={};
   my $list = $self->list();
   foreach my $c ( @$list ) {
       $goa_names->{$c} = $self->{$c}{goa_name};
       if ( $self->{$c}{old_name} ) {
          $goa_names->{$self->{$c}{old_name}} = $self->{$c}{goa_name};
       }
   }
   return wantarray ? %{$goa_names} : $goa_names;
}

sub goa_offsets {
   my $self = shift;
   my %opt = (
	     igs_code => '',
	     to_meters => 1e-3,
	     @_,
             );
   
   croak "No igs_code !\n" if ! $opt{igs_code};
   
   my ($L1,$L2) = ( $self->{$opt{igs_code}}{l1_offset},
	            $self->{$opt{igs_code}}{l2_offset}
	           );	
   # scale 
   @$L1 = map { $_ * $opt{to_meters}  } @$L1;
   @$L2 = map { $_ * $opt{to_meters}  } @$L2;
   # calculate LC
   my $LC=[];
   my $L0=[];
   my $i = 0;
   while ( $i < 3 ) {
	#LC = 2.5457*L1-1.5457*L2
	$$LC[$i]=2.5457 * $$L1[$i]-1.5457 * $$L2[$i];
	$$L0[$i]=0.0;
	$i++;
   };
   my $out = {};
   $out->{L1} = { east => $$L1[1] , north => $$L1[0], up => $$L1[2]  } ;
   $out->{L2} = { east => $$L2[1] , north => $$L2[0], up => $$L2[2]  } ;
   $out->{LC} = { east => $$LC[1] , north => $$LC[0], up => $$LC[2]  } ;
   $out->{P1} = { east => $$L1[1] , north => $$L1[0], up => $$L1[2]  } ;
   $out->{P2} = { east => $$L2[1] , north => $$L2[0], up => $$L2[2]  } ;
   $out->{PC} = { east => $$LC[1] , north => $$LC[0], up => $$LC[2]  } ;
   $out->{C2} = { east => $$L2[1] , north => $$L2[0], up => $$L2[2]  } ;
   $out->{DR} = { east => $$L0[1] , north => $$L0[0], up => $$L0[2]  } ;
   $out->{R2} = { east => $$L0[1] , north => $$L0[0], up => $$L0[2]  } ;
  
   return $out
}
sub vals {
  my $self = shift;
  my $vals;
  if ( $pcv ) { 
     $vals = $pcv->vals(@_);
  }
  elsif ( $atx ) {
     $vals = $atx->vals(@_) ;
     #$vals = $vals;
    # print "vals: @{$vals->[0]}\n";
  }
  else {
    croak "No source (pcv|atx) defined\n"; 
  }
  return $vals
}

sub nml_sat_num_names {
   my $self = shift;
   my %opt = (
       sat_names => '',
       to_meters => 1.0e-3,
       to_km => 1.0e-6,
       comment => '',
       ant_no  => 1,
       out_dir => '',
       date    => 'today',
       file    => '',
       fh      => '',
       @_,
    );
   my $sat_names = [];
   my $sat_names = $opt{sat_names} ; 
   my $n_sat = scalar @{$sat_names};
   my $nml = ''; 
   #my $gdate = get_date($opt{date},'cal,time');
   
   $nml .= qq/
 ! NML generated by $ENV{USER} $Time_Stamp
 ! Geodesy Lab. and PANGA.
 ! Copyright (c) $cr_yyyy Central Washington University
 ! \n/;
   #set SatNums group
   $nml .="\$SatNums\n\n"; 
   $nml .= $self->write_var("NMLNoSat"      , $n_sat   );
   $nml .= $self->write_var('NMLNoAntenna'  , 1        );
   $nml .= $self->write_var( 'NMLNoAntennaD', $n_sat*2 );
   $nml .= "\n\$end\n\n"; 
   #set satellite_names group 
   $nml .="\$satellite_names\n\n"; 
   my $sn=[]; 
   #satellite names ordered by igs code name
   @$sn = sort @{$sat_names} ;
   $nml .= $self->write_var( 'NMLSatNm(1)', $sn);    
   $nml .= "\n\$end\n\n"; 
   return $nml
}

sub nml_sat {
   my $self = shift;
   my %opt = (
       igs_code     => '',
       to_meters    => 1.0e-3,  # millimeters to meters
       to_km        => 1.0e-6,  # millimeters to km
       ant_no       => 'Ant_no',
       out_dir      => '',
       date         => 'today',
       file         => '',
       fh           => '',
       @_,
    );
   my $igs_code = uc $opt{igs_code};
   if ( ! $igs_code ) {
     carp "No igs_code !\n";
     return ''
   }
   elsif ( ! exists $self->{$igs_code} ) {
      carp "IGS code |$igs_code| doesn't exists\n"; 
      return ''
   }
   my $nml = ''; 
   my $ant_no=$opt{ant_no};
   my %Dtype = ( L1C => 1, 
                 L2C => 2,
                 LCC => 3,
                 L1P => 4, 
                 L2P => 5,
                 LCP => 6,
                 KBR => 7,
               );
   my $a = $self->{$igs_code};
   my $incAz = $a->{dazi} == 0.0 ? 45.0 : $a->{dazi};
   my $dszEl = ($a->{zen2}-$a->{zen1})/$a->{dzen};
   if ( $dszEl < 0 ) { croak "Wrong format: ZEN1 > ZEN2 in $igs_code\n"}
   my $szEl = 90/$a->{dzen} + 1;
   my $s_El = int($a->{zen1}/$a->{dzen}) ;
   my $e_El = int($a->{zen2}/$a->{dzen}) ;
   my @v=(sprintf('%10.5e',0.0))x$szEl; 
   my $szAz = 360/$incAz + 1;
   my $i=1;
   my $dt ='';
   my $freq;
   my $ecc = $a->{frequency};
   my $xyz = '';
   #set satellite model
   $nml .= "\$satellite_model\n\n";
   foreach my $dt ( qw/LCC LCP/ ) {
      $freq = $dt eq 'LCC' ? 'G01' : 'G02';
      $xyz = sprintf '%s %s %s', map { $_ * $opt{to_km} } 
                                 @{ $ecc->{$freq} } {qw/north east up/};
      $nml .= qq/
   
   ! Phase Center Map NML generated by $ENV{USER} $Time_Stamp
   ! Geodesy Lab. and PANGA.
   ! Copyright (c) $cr_yyyy Central Washington University
   ! NML Generated from : $self->{$igs_code}{comment}
   ! NML Data type      : $dt == $Dtype{$dt} 
   ! Description        : $self->{$igs_code}{description}
   ! IGS Code PRN       : $igs_code
   ! GPSNN              : $self->{$igs_code}{sat_code_nnn}
   ! ANTENNA #          : $ant_no
   ! ECCENTRICITY XYZ   : $xyz (m)

/;
      $nml .= $self->write_var( "NMLOSzAzD($ant_no)"  , $szAz               );
      $nml .= $self->write_var( "NMLOIncrAD($ant_no)" , $incAz              );
      $nml .= $self->write_var( "NMLOSzElD($ant_no)"  , $szEl               );
      $nml .= $self->write_var( "NMLOIncrED($ant_no)" , $a->{dzen}          );
      $nml .= $self->write_var( "NMLOPtCenD($ant_no,1,$Dtype{$dt})", $ant_no);
      $i=1;
      for(my $azi=0.0; $azi <= 360.0 ; $azi+=$incAz ) {
           my $v1 = $self->vals(
                 antenna      => $igs_code,
                 freq_code    => "L1",
                 azi_angle    => $a->{dazi} == 0.0 ? 'NOAZI': $azi,
               );
           my $v2 = $self->vals(
                 antenna      => $igs_code,
                 freq_code    => "L2",
                 azi_angle    => $a->{dazi} == 0.0 ? 'NOAZI': $azi,
               );

           my $k=0;
           for( my $j=$s_El; $j<=$e_El; $j++ ) {
                $k=$szEl - 1 - $j;
                #$k=$j;
	        #LC   = 2.5457*L1-1.5457*L2
                $v[$k]= 2.5457*$v1->[$j]-1.5457*$v2->[$j];
                $v[$k]= sprintf "% 10.5e",$v[$k]*$opt{to_km};
           }         
           $nml .= $self->write_var( "NMLOCen(1,$i,$ant_no)" , @v  );
           $i++;
       } #end for
 
    } #end dt
   $nml .= "\n\$end\n\n"; 
   
   return $nml
    
}

sub write_var {
  my $self = shift;
  my $lhs  = shift;
  #my $rhs = shift;
  my $rhs = ref $_[0] eq 'ARRAY' ? $_[0]  : 
            ref $_[0] eq ''      ? [ @_ ] :
                                   croak "Invalid reference type for rhs\n" ;
  my $nl  = 80;
  my @l;
  my $eq  = trim($lhs) ? '=' : ' ' ;
  $l[0]  = "   $lhs $eq ";
  my $line = '';
  my $lpad = ' 'x(length $l[0]);
  my $sep = @$rhs > 1 ? ', ': '' ;
  #print "input: $lhs @$rhs\n";
  my @r = @$rhs;
  while ( @r ) {
    #my $v = trim(shift @r);
    my $v = shift @r;
    #print "output: $lhs |$v|\n";
    my $lvs=length( "$l[-1]$sep$v$sep");
    if ( @r ) {
       if ( $lvs < $nl ) {
         $l[-1] .= "$v$sep";
       }
       else {
         $l[-1] .= "\n";
         push @l, "$lpad$v$sep";
       }
    }
    else  {
      if ( $lvs < $nl ) {
        $l[-1].= $v;
      }
      else {
        $l[-1] .= "\n";
        push @l, "$lpad$v";
      }
    }
   }
  $line = join '',@l;
  return "$line\n"
}

sub sat_names {
   my $self = shift;
   my $date = shift || '';
   my $sat_sys = shift || 'G'; #G=GPS ; R=GLONNAS ; E=Galileo; 
   if ( $date ) { 
     $date = get_date($date,'jd');
   }
   my $list =  $self->list();
   my %sat_name = ();
   my ($edate,$delta)=('',0.0);
   foreach my $igs_code ( @$list ) {
        my $a = $self->{$igs_code};
        next if ! exists $a->{sat_code_nn};
        next if  $a->{sat_code_nn} !~ /^$sat_sys/;  
        ($edate,$delta)=('',0.0);
        if ( $date && exists $a->{eyyyy} ) {
           $edate = sprintf"%4d-%02d-%02d %02d:%02d:%10.7f",
                    @{$a}{qw/eyyyy emm edd ehh emin esec/};
           $edate = get_date($edate,'jd');
           $delta = $date - $edate;
           #print "found valid until field: $edate |$delta\n";
           next if $delta > 0.0 ;
        }
        my $gps = $a->{sat_code_nnn};
        $gps =~ s/^$sat_sys//; $gps =~ s/^0+//;
        $gps = sprintf 'GPS%02d',$gps;
        $sat_name{$igs_code} = $gps;
   }
   return \%sat_name
}
sub nml_ground {
  my $self = shift;
  my %opt = (
       igs_code => '',
       nml      => '',
       to_meters => 1.0e-3,
       to_km => 1.0e-6,
       comment => '',
       ant_no  => 'Ant_no', #1,
       out_dir => '',
       @_,
    );
  my $nml = ''; 
  my $igs_code = uc $opt{igs_code};
  croak "No igs_code !\n" if ! $igs_code;
  croak "IGS code |$igs_code| doesn't exists\n" if ! exists $self->{$igs_code};
  my $a = $self->{$igs_code};  
  my $ecc = $a->{frequency};
  my $neu = '';
  my $freq;
  my $ant_no = $opt{ant_no};
  my $incAz = $a->{dazi} == 0.0 ? 45.0 : $a->{dazi};
  my $dszEl = ($a->{zen2}-$a->{zen1})/$a->{dzen};
  if ( $dszEl < 0 ) { croak "Wrong format: ZEN1 > ZEN2 in $igs_code\n"}
  my $szEl = 90/$a->{dzen} + 1;
  my $s_El = int($a->{zen1}/$a->{dzen}) ;
  my $e_El = int($a->{zen2}/$a->{dzen}) ;
  my $szAz = 360/$incAz + 1;
  my %Dtype = ( L1C => 1, 
                L2C => 2,
                LCC => 3,
                L1P => 4, 
                L2P => 5,
                LCP => 6,
                KBR => 7,
               );

  $nml .= "\$station_model\n\n";
  $nml .= qq/\n
!
! Phase Center Map NML generated by $ENV{USER} $Time_Stamp
! Geodesy Lab. and PANGA.
! Copyright (c) $cr_yyyy Central Washington University
! NML Data type     : $dt
! Data source       : $self->{$opt{igs_code}}{comment}
! Vendor            : $self->{$opt{igs_code}}{vendor}
! Description       : $self->{$opt{igs_code}}{description} 
! PCENTER Name      : $self->{$opt{igs_code}}{goa_name}
! ECCENTRICITY NEU  : $neu (m)
! ANTENNA #         : $ant_no
!
! |IGS Code        Dome|   
! |$igs_code|
! 
  \n/;
  $nml .= $self->write_var( "NMLGPtCen($ant_no)" , $ant_no      ); 
  $nml .= $self->write_var( "NMLGSzEl($ant_no)"  , $szEl        ); 
  $nml .= $self->write_var( "NMLGIncrE($ant_no)" , $a->{dzen}   ); 
  $nml .= $self->write_var( "NMLGSzAz($ant_no)"  , $szAz        ); 
  $nml .= $self->write_var( "NMLGIncrA($ant_no)" , $incAz       ); 


  my @v=(sprintf('%10.5e',0.0))x$szEl; 
  foreach my $dt ( qw/L1 L2/ ) {
     my $i=1;
     $freq = $dt eq 'L1' ? 'G01' : 'G02';
     $neu  = sprintf '%s %s %s', map { $_ * $opt{to_meters} } 
                                  @{ $ecc->{$freq} } {qw/north east up/};
     $nml .= qq/
! ECCENTRICITY NEU  : $neu (m)
! ANTENNA #         : $ant_no
\n/;
     for(my $azi=0.0; $azi <= 360.0 ; $azi+=$incAz ) {
        my $vals = $self->vals(
                 antenna      => $igs_code,
                 freq_code    => $dt,
                 azi_angle    => $a->{dazi} == 0.0 ? 'NOAZI': $azi,
               );
        for( my $j=$s_El; $j<=$e_El; $j++ ) {
                $k=$szEl - 1 - $j;
                $v[$k]= sprintf "% 10.5e",$vals->[$j]*$opt{to_km};
                #print "$szEl $j $k $v[$k]\n";
        } 
        #@v = map { sprintf "%10.2e", $_ * $opt{to_km} } @{$vals};
        #@v[$s_El..$e_El] = map { sprintf "%10.2e", $_ * $opt{to_km} } reverse @{$vals};
        #print "@{$vals1}\n"; 
        $nml .= $self->write_var( "NMLGCen$dt(1,$i,$ant_no)" , @v  ); 
        #$nml .= "    NMLGCen$fc(1,$i,$ant_no) = @v\n"; 
        $i++;
     }
     $nml.="! \n";
  }
  $nml .= "\n\$end\n\n";
  return $nml
}

sub goa_name {
   my $self = shift;
   my %opt = (
      igs_code => '',
      vendor   => '',
      @_,
   );
  croak "No igs_code !\n" if ! $opt{igs_code};
   
  my $c  = $opt{igs_code};
  my ($a,$d) = unpack("a16a4",$c);
  my $v  = $opt{vendor};
  my $gv = $goa_vendor{ $v }; 
  my $gd = $goa_dome{ $d } || 'X'; # X : unknown dome. 
  my $m =  pack("x2a6",fix_str($a)); 
    
  my $gc;

  return pack("A9",'ZERO') if  trim($a) eq 'NONE' || $a eq ' 'x16;
	  
  if ( $gv ) {
    $m = $a;
    my $vc  = unpack ("a3",trim($a)) ;
    $m =~ s/$vc// if ( $vc =~ /[A-Z]+/ && $v   ) ;
    $m  = unpack ("a6",$m); #trim($a)) if length(trim($a)) >= 9; 
    $m = fix_str($m);
    #substr($m,-length($ct),$ct);
    
    
  } else {	  
     ($gv,$m) =  unpack("a2a6",fix_str($a)); 
  }
 
  $gc = $self-> get_goa_code(  goa_vendor => $gv,
                                  middle     => $m,
                                  goa_dome   => $gd );
  
  return $gc
  
  
}
sub get_goa_code {
   my $self = shift;
   my %opt = (
      goa_vendor => '',
      middle     => '',
      goa_dome   => '',
      @_,
   );

  my $ct = 'A';
  my @gcodes = map { $self->{$_}{goa_name} } keys %$self;
  my $gv= $opt{goa_vendor};
  my $m = $opt{middle};
  my $gd = $opt{goa_dome};
  my $gc= "$gv$m$gd";
  
  while( grep /$gc/, @gcodes ) {
    #print "$gv$m$gd";
    $m =~ s/\w$/$ct/;
    $ct++;
    $gc="$gv$m$gd";
    #substr($m,-length($ct),$ct);
  }
  
  return pack("A9",uc $gc) 

}

sub fix_str {
  my $s = shift;
  $s =~ s/\s+//g;
  $s =~ s/\//_/g;
  $s =~ s/(\.|\+|\-)//g;
  return $s
}
sub trim {
 my $s = shift;
    $s =~ s/^\s+//;
    $s =~ s/\s+$//;
 return $s
}

1;

