package GPS::AntennaPhaseCenter;
use Carp;
use GPS::RcvrAnt;


our %fmt = ( igs => "a15a20a25",
             ngs => "a20a45"
           );
our %fields = ( igs => [ 'vendor','igs_code','description' ],
                ngs => [ 'igs_code','description'] 
             );

# add missing vendors here
our %other_vendor_codes = ( MAG  => 'MAGELLAN',
	                    SEN  => 'SENSOR SYSTEMS',
	                    THA  => 'THALES NAVIGATION'
			  );
our %vendor_alias = ( MPL => 'MICROPULSE',
	              NOV => 'NOVATEL',
		      
		    'THREE-S' => '3SNAVIGATION'
		     
                    );     

our %goa_vendor = (
	           '3SNAVIGATION'    => '3S',
		   'AEROANTENNA'     => 'AA',
		   'ASHTECH'         => 'AS',
		   'GEOTRACER'       => 'GT',
		   'JAVAD'           => 'JD',
		   'LEICA'           => 'LC',
		   'MAGELLAN'        => 'MG',
		   'MICROPULSE'      => 'MP',
		   'MINIMAC'         => 'MM',
		   'NAVCOM'          => 'NC',
		   'NOVATEL'         => 'NV',
		   'ROGUE'           => 'AO',
		   'SENSOR SYSTEMS'  => 'SS',
		   'SOKKIA'          => 'SK',     
		   'SPECTRA PRECISION' => 'SP',
		   'TEXAS INSTRUMENTS' => 'TI',
		   'THALES NAVIGATION' => 'TN',
		   'TOPCON'          => 'TP', 
		   'TRIMBLE'         => 'TR'
	          );
		    
our %goa_dome = (
	           AUST  => 'A',
		   CAFG  => 'C',
		   CONE  => 'N',
		   DOME  => 'D',
		   DUTD  => 'T',
		   EMRA  => 'E',
		   ENCL  => 'L',
		   GRAZ  => 'G',
		   JPLA  => 'J',
		   LEIC  => 'L',
		   LEIS  => 'I',
		   MMAC  => 'M',
		   NONE  => '_',
		   OSOD  => 'O',
		   RPTR  => 'R',
		   SCIS  => 'S',
		   SCIT  => 'T',
		   SCPL  => 'P',
		   SNOW  => 'W',
		   SPKE  => 'K',
		   TCWD  => 'V',
		   TZGD  => 'Z',
		   UNAV  => 'U',
		   UNKN  => 'X',
		  '    ' => '_'
		 );

		  
sub new {
  my $class = shift;
  my $self =  {};

  bless $self,$class;
  
  return $self->init(@_) 
}

sub init {
  my $self = shift;
  my %opt  = (
     rcvr_ant_dir => '',
     rcvr_ant => '',
     @_,
  );
  our $ra = GPS::RcvrAnt->new( %opt );
  our $vc_ra = $ra->vendor_codes( device => 'antennae');
  # add vendor codes from rcvr_ant.tab
  @other_vendor_codes{ keys %$vc_ra } =  values %$vc_ra;

  return $self->load( %opt )

}
	     
sub get_vendor {
    my $self = shift;
    my %opt  = (
       igs_code => '',
       vendor   => '',
       @_,
    );

    my $c  = $opt{igs_code};
    my $ac = pack("a3",$c);
    my %vc = %other_vendor_codes; 
    my $v_ra  = '';
    if ( $ra->{antennae}{trim($c)}{vendor}  !~ /(OTHER|SATELLITE)/ ) { 
       $v_ra = $ra->{antennae}{trim($c)}{vendor};
    }
    # print map{ "$_ $vc{$_}\n"} keys %vc;
    #exit; 
    
    my $v  = trim($opt{vendor}) || $self->{$c}{vendor};
    
    
    if ( $ac !~ /\d\d\d/ ) {
       $v  = $vc{$ac} if  $v ne $vc{$ac} ;
    } 	    
    
    $v ||= $v_ra;  #assign from rcvr_ant.tab if not set yet
    
    if ( exists $vendor_alias{trim($v)} ) {
      $v = $vendor_alias{trim($v)};
    }
   
    #print "$c | $v |$ac $vc{$ac}| $v_ra | ",$vendor_alias{trim($v)},"$opt{file}\n";
    

    return $v || $c #return same code if there isn't a match
}

sub load {
  my $self = shift;
  my %opt = (
      pcv_files => 'igs_01.pcv,ant_info.003',
      pcv_dir   => "$ENV{GPS_HOME_DIR}/igs_info",
      @_,       
  );
  my $dir  = $opt{pcv_dir};
  croak "Can't find phase center directory $dir\n" if ! -d $dir;
  my @pcvs = split(/\,/,$opt{pcv_files});
  my $type;
		  
 foreach my $file ( @pcvs ) {	 
    $type = "ngs" if $file =~ /ant_info/;
    $type = "igs" if $file =~ /igs_/;
    open(FILE,"< $dir/$file") or croak "couldn't open file: $file\n";
    #skip header
    my $i=0;
    while ( <FILE> ) {
      $i++ if /\| Elevation/;	 
      last if $i > 1;  
    }
    my $old_name='';
    while ( <FILE> ) {
       s/^\s+//;   
       s/\s+$//;   
       next unless length;
       my $p={ vendor => '' };
       
       @$p{ @{$fields{$type}} }=unpack($fmt{$type},$_);
       my $a = $p->{igs_code};
       $p->{igs_code} = $a eq  'NONE'.' 'x12 .'    ' ? ' 'x16 .'NONE' :
                        $a eq  'NONE'.' 'x12 .'NONE' ? ' 'x16 .'NONE' : 
                                                       $a;
       if ( exists $self->{$p->{igs_code}} ) { 
	   for ( 1..6 ) { readline(FILE) };
	   next;
       }
       if ( $p->{description} =~ /OLD NAME/ ) {
           $old_name = $p->{igs_code};
	   for ( 1..6 ) { readline(FILE) };
	   next;
       }
       
       $p->{vendor} = $self->get_vendor( igs_code => $p->{igs_code},
                                         vendor   => $p->{vendor},
				         file     => $file );
       
       $p->{old_code}= $old_name if $old_name;
       $old_name='';
        
       $p->{l1_offset}= [ split(' ',readline(FILE)) ];
       my $l = readline(FILE);
       $l.=readline(FILE);
       $p->{l1_deg}= [ split(' ',$l) ]; 
       $p->{l2_offset}= [ split(' ',readline(FILE)) ];
       my $l = readline(FILE);
       $l.=readline(FILE);
       $p->{l2_deg}=[ split(' ',$l) ]; 
       $p->{comment} = pack("A16",$file);
       $p->{goa_name} = $self->goa_name( igs_code => $p->{igs_code},
                                         vendor   => $p->{vendor} );
       $self->{ $p->{igs_code} } = $p ;
       #print "$p->{igs_code} | $p->{old_code}\n";       
    }
 }
   return $self 

}
sub list {
   my $self = shift;
   my %opt  = (
      igs_names => 1,
      goa_names => 0,
      @_,
   ); 
   my @list;
   $opt{igs_names} = 0 if $opt{goa_names};
   if ( $opt{igs_names} ) {
      foreach my $a ( sort keys %$self ) {
        $a = $a eq  'NONE'.' 'x12 .'    ' ? ' 'x16 .'NONE' :
             $a eq  'NONE'.' 'x12 .'NONE' ? ' 'x16 .'NONE' : 
                                            $a;
        push @list, $a;
      }
   }
   if ( $opt{goa_names} ) {
      push @list, sort keys %{ $self->get_goa_names() };
   }
   
   return \@list;
  
}
sub get_goa_names {
   my $self = shift;
   my $goa_names={};
   my $list = $self->list();
   foreach my $c ( @$list ) {
       $goa_names->{$c} = $self->{$c}{goa_name};
       if ( $self->{$c}{old_name} ) {
          $goa_names->{$self->{$c}{old_name}} = $self->{$c}{goa_name};
       }
   }
   return %{$goa_names}
}
sub get_val  {
   my $self = shift;
   my %opt = (
      igs_code => '',
           val => '',
      @_,      
   );
   
   croak "No antenna code !" if ! $opt{igs_code};
   my $igs_code = uc $opt{igs_code};
   croak "igs_code $igs_code not valid" if ! exists $self->{$igs_code};
   
   croak "No values to get !\n" if !  $opt{val} ;
   
   return $self->{$opt{igs_code}}{$opt{val}};
   
}

sub goa_offsets {
   my $self = shift;
   my %opt = (
	     igs_code => '',
	     to_meters => 1e-3,
	     @_,
             );
   
   croak "No igs_code !\n" if ! $opt{igs_code};
   
   my ($L1,$L2) = ( $self->{$opt{igs_code}}{l1_offset},
	            $self->{$opt{igs_code}}{l2_offset}
	           );	
   # scale 
   @$L1 = map { $_ * $opt{to_meters}  } @$L1;
   @$L2 = map { $_ * $opt{to_meters}  } @$L2;
   # calculate LC
   my $LC=[];
   my $L0=[];
   my $i = 0;
   while ( $i < 3 ) {
	#LC = 2.5457*L1-1.5457*L2
	$$LC[$i]=2.5457 * $$L1[$i]-1.5457 * $$L2[$i];
	$$L0[$i]=0.0;
	$i++;
   };
   my $out = {};
   $out->{L1} = { east => $$L1[1] , north => $$L1[0], up => $$L1[2]  } ;
   $out->{L2} = { east => $$L2[1] , north => $$L2[0], up => $$L2[2]  } ;
   $out->{LC} = { east => $$LC[1] , north => $$LC[0], up => $$LC[2]  } ;
   $out->{P1} = { east => $$L1[1] , north => $$L1[0], up => $$L1[2]  } ;
   $out->{P2} = { east => $$L2[1] , north => $$L2[0], up => $$L2[2]  } ;
   $out->{PC} = { east => $$LC[1] , north => $$LC[0], up => $$LC[2]  } ;
   $out->{C2} = { east => $$L2[1] , north => $$L2[0], up => $$L2[2]  } ;
   $out->{DR} = { east => $$L0[1] , north => $$L0[0], up => $$L0[2]  } ;
   $out->{R2} = { east => $$L0[1] , north => $$L0[0], up => $$L0[2]  } ;
  
   return $out
}

sub nml {
    my $self = shift;
    my %opt = (
       igs_code => '',
       to_meters => 1.0e-3,
       to_km => 1.0e-6,
       comment => '',
       ant_no => 1,
       @_,       
    );

   my $ant_no = $opt{ant_no};

   croak "No igs_code !\n" if ! $opt{igs_code};
   
   my ($L1,$L2) = ( $self->{$opt{igs_code}}{l1_offset},
	            $self->{$opt{igs_code}}{l2_offset}
	           );	
   my ($L1_DEG,$L2_DEG) = ( $self->{$opt{igs_code}}{l1_deg},
	                    $self->{$opt{igs_code}}{l2_deg}
	                  );	
   # scale and acomodate
   # scale and acomodate
   @$L1 = map { $_*$opt{to_meters}  } @$L1;
   @$L2 = map { $_ * $opt{to_meters}  } @$L2;
   #@$L1_DEG = reverse map { sprintf("%3E",$_ * $opt{to_km} ) } @$L1_DEG;
   #@$L2_DEG = reverse map { sprintf("%3E",$_ * $opt{to_km} ) } @$L2_DEG;
   @$L1_DEG = reverse map { sprintf("%s",$_."E-6" ) } @$L1_DEG;
   @$L2_DEG = reverse map { sprintf("%s",$_."E-6" ) } @$L2_DEG;

   my $nml = [];
   push @$nml, qq/
\$station_model
! NML generated from $self->{$opt{igs_code}}{comment}
! Vendor: $self->{$opt{igs_code}}{vendor}
! Description: $self->{$opt{igs_code}}{description}
! IGS Code-15Cols Dome
! $opt{igs_code} 
    NMLGPtCen($ant_no) = $ant_no 
    NMLGSzEl($ant_no) = 19
    NMLGIncrE($ant_no) = 5.0
    NMLGSzAz($ant_no) = 9
    NMLGIncrA($ant_no) = 45.0 
     
/; 
     $i=1;
     while ( $i <= 9 ) {
       push(@$nml, "    NMLGCenL1(1, $i, $ant_no) = ",join(', ',@$L1_DEG),"\n" );
       $i++;
     }
     $i=1;
     while ( $i <= 9 ) {
       push(@$nml, "    NMLGCenL2(1, $i, $ant_no) = ",join(', ',@$L2_DEG),"\n");
       $i++;
     }
	
  push @$nml ,"\$end\n";		
   return $nml

}
sub goa_name {
   my $self = shift;
   my %opt = (
      igs_code => '',
      vendor   => '',
      @_,
   );
  croak "No igs_code !\n" if ! $opt{igs_code};
   
  my $c  = $opt{igs_code};
  my ($a,$d) = unpack("a16a4",$c);
  my $v  = $opt{vendor};
  my $gv = $goa_vendor{ $v }; 
  my $gd = $goa_dome{ $d } || 'X'; # X : unknown dome. 
  my $m =  pack("x2a6",fix_str($a)); 
    
  my $gc;

  return pack("A9",'ZERO') if  trim($a) eq 'NONE' || $a eq ' 'x16;
	  
  if ( $gv ) {
    $m = $a;
    my $vc  = unpack ("a3",trim($a)) ;
    $m =~ s/$vc// if ( $vc =~ /[A-Z]+/ && $v   ) ;
    $m  = unpack ("a6",$m); #trim($a)) if length(trim($a)) >= 9; 
    $m = fix_str($m);
    #substr($m,-length($ct),$ct);
    
    
  } else {	  
     ($gv,$m) =  unpack("a2a6",fix_str($a)); 
  }
 
  $gc = $self-> get_goa_code(  goa_vendor => $gv,
                                  middle     => $m,
                                  goa_dome   => $gd );
  
  return $gc
  
  
}
sub get_goa_code {
   my $self = shift;
   my %opt = (
      goa_vendor => '',
      middle     => '',
      goa_dome   => '',
      @_,
   );

  my $ct = 'A';
  my @gcodes = map { $self->{$_}{goa_name} } keys %$self;
  my $gv= $opt{goa_vendor};
  my $m = $opt{middle};
  my $gd = $opt{goa_dome};
  my $gc= "$gv$m$gd";
  
  while( grep /$gc/, @gcodes ) {
    #print "$gv$m$gd";
    $m =~ s/\w$/$ct/;
    $ct++;
    $gc="$gv$m$gd";
    #substr($m,-length($ct),$ct);
  }
  
  return pack("A9",uc $gc) 

}

sub fix_str {
  my $s = shift;
  $s =~ s/\s+//g;
  $s =~ s/\//_/g;
  $s =~ s/(\.|\+|\-)//g;
  return $s
}
sub trim {
 my $s = shift;
    $s =~ s/^\s+//;
    $s =~ s/\s+$//;
 return $s
}



