package GPS::AntennaTruth;
use Fortran::F90Format;
use GPS::DATES;

my @fields = qw/site epoch height/;
my $fmt  = Fortran::F90Format->new( fmt =>"1X,A4,2X,A7,2X,F7.4" );

sub new {
  my $class = shift;
  my $self  = { };
  bless $self, $class;
  return $self
}
sub add {

  my $self = shift;
  my %opt  = ( site => '',
               epoch => '',
               height => '',
               @_,
              );
  my $site = uc $opt{site};
  if (  $site !~ /\w{4}/ ) {
    die "Invalid site name $site \n";
  }
  if ( ! $opt{epoch} ) {
    die "No epoch/date supplied \n";
  }
  my $cal = get_date( $opt{epoch},'cal,num');
  
  if ( $opt{height} eq '' ) {
    die "No height supplied\n";
  }
  
  my $height = $opt{height};

  $self->{$site}{$cal}=$height;  
  
}

sub load  {
  my $self = shift;
  my %opt = ( file => '',
              fh   => '',
              @_,
             );
  my $fh = $opt{fh};
  if ( $opt{file} ) {
     open( my $gh , "< $opt{file}" ) or die "Couldn't open $opt{file} $!\n";
     $fh=$gh;
  }
  if ( ! $fh ) {
     die "Invalid file handle or No file supplied\n";
  }
  
  while ( <$fh> ) {
     my ($site,$epoch,$height) = $fmt->read($_);
     my $cal = get_date($epoch,'cal,num');
     $self->{$site}{$cal}=$height;
  }
  
}

sub print {
  my $self = shift;
  my %opt = ( file => '',
              fh   => \*STDOUT,
              @_,
             );
  my $fh = $opt{fh};
  if ( $opt{file} ) {
     open( my $gh , "> $opt{file}" ) or die "Couldn't open $opt{file} $!\n";
     $fh=$gh;
  }
  if ( ! $fh ) {
     die "Invalid file handle or No file supplied\n";
  }
  foreach my $sta ( sort keys %$self ) {
     my $hc = $self->{$sta};
     foreach my $cal ( sort keys %$hc ) {
        my $epoch = get_date($cal,'epoch');
        my $height = $self->{$sta}{$cal};
        print $fh $fmt->write( $sta,$epoch,$height );
     }
  }
  
}


1;







