package GPS::CheckFiles;
use Carp;
use Exporter;
use GPS::Defaults;
use GPS::DATES;
use File::Basename;

our %par = get_defaults();

@ISA=('Exporter');
@EXPORT =( check_pp_amb_files , check_path ); 



sub check_pp_amb_files {
   my %opt = ( date		=> '',
               ac               => '',
	       site		=> '',
	       net		=> '',
	       orb		=> 'final',
	       proj		=> '',
	       @_,
            );
my $date = $opt{date};
my $ac  = $opt{ac};
my $site = $opt{site};
my $orb  = $opt{orb};
my $proj = $opt{proj};

my $cal = get_date( $date ,"cal" );
my ($year) = split(/-/,$cal);
my $dir;

if ( ! $ac ) {
   die "No orbit type or agency (igs|jpl)\n";
}

if ( $net ) {
  $net = uc $net;
  $dir= "$par{AMB_DIR}/$ac/$proj/$year/$cal/$net";
}
if ( $site ) {
  $site = lc $site;
  $dir= "$par{PPP_DIR}/$ac/$year/$cal/$site";
}

if ( $orb ) {
  $orb = lc $orb;

  croak "orbit type must be final/rapid\n" if $orb !~ /(final|rapid)/;

}

my $ok = 1;

opendir(DIR,$dir) or $ok =0; 


my @files =map { "$dir/$_" } grep !/^\.+/, readdir(DIR);

my %ok_files = ();

if ( $site  ) {
  %ok_files=ok_site( $site,$orb,$year,$cal);
}

if ( $net ) {
  %ok_files = ( "$cal.fix.stacov" => 1 );
}

#print ( map { "$_\n" } @files);
foreach my $f ( keys %ok_files ) {
   $ok = 0 if ! grep /$f/,@files;
   last if ! $ok;
}


return $ok

}

sub ok_site {
   my $site = shift;
   my $orb  = shift;
   my $year = shift;
   my $cal  = shift;

   my  @f = qw/ raw.stacov
                point.log  
	        qm
	        smcov.nio  
          /;
   $f[0] = 'itrf00.r.stacov' if $orb eq "rapid";
   
   my %ok;

   foreach my $f ( @f ) {
       my  $file = "$cal.$site.$f";
       $ok{$file}++
   } 
   return %ok
}

sub check_path {
  my $dir = shift;
  my @d=() ;
  my $ok=0;
  my $d= $dir;
  while ( $d ne '/' ) {
    push @d,$d;
    $d = dirname $d;
  }
  foreach $d ( reverse @d  ) {
   $ok=0;
   if ( ! -e $d ) {
     $ok = `ls $d 2>  /dev/null` ;
     if ( ! $ok ) { $ok=0; warn "$d not found\n"; last; }
   } else { $ok = 1  };
  }
  return $ok
}



1;
