# Dates And Time Extremely Simple (DATES)
# perl gps dates and time package
# handle several date conversions for GPS processing
# specially with GIPSY
# Marcelo Santillan CWU 
# started April 2004
# latest: Feb 13 2007 
package GPS::DATES;
use Carp;		
use POSIX qw(strftime floor );
use Exporter;
@ISA= ('Exporter');
@EXPORT = qw(get_date);
@EXPORT_OK = qw( get_date 
	      cal cal2epoch cal2doy cal2jd cal2dy
          current_time
	      doy2cal 
          dow2gdow dow2days 
	      dy2jd dy2epoch dy2cal
          epoch2dy epoch2cal 
	      frac dd2dhms
	      gwd2jd
	      isleap  
	      jd2mjd jd2cal jd2dow jd2dy jd2gpsw jd2sec 
          mjd2sec
          m2month month2m 
          sec2cal cal2sec gps2utc
	      yy2y y2yy
          CHECK_DATES
          GUESS_DATES
	      @months @month_days %Months 
	      @days $JD2000 $GPSSE $MJD0 
          $LEAPSECS @LEAP_SECONDS
              );
#%EXPORT_TAGS = (
#	Functions => [ qw( dow2days m2months months2m cal2epoch 
#			  cal2doy caltodoy  cal2doy cal2jd jd2mjd 
#			  jd2cal dow jd2dy dy2jd gpsw frac doy2cal isleap ) ],
#	Variables => [ qw( @months @months_days @days $JD2000 $GPSSE )]
#);

# constant
our @LEAP_SECONDS    = ();
our $LEAPSECS        = qq(
01-JAN-1979 00:00:00.0000    UTC-TAI= -18
01-JAN-1980 00:00:00.0000    UTC-TAI= -19
01-JAN-1981 00:00:00.0000    UTC-TAI= -20
01-JAN-1982 00:00:00.0000    UTC-TAI= -21
01-JUL-1983 00:00:00.0000    UTC-TAI= -22
01-JUL-1985 00:00:00.0000    UTC-TAI= -23
01-JAN-1988 00:00:00.0000    UTC-TAI= -24
01-JAN-1990 00:00:00.0000    UTC-TAI= -25
01-JAN-1991 00:00:00.0000    UTC-TAI= -26
01-JUL-1992 00:00:00.0000    UTC-TAI= -27
01-JUL-1993 00:00:00.0000    UTC-TAI= -28
01-JUL-1994 00:00:00.0000    UTC-TAI= -29
01-JAN-1996 00:00:00.0000    UTC-TAI= -30
01-JUL-1997 00:00:00.0000    UTC-TAI= -31
01-JAN-1999 00:00:00.0000    UTC-TAI= -32
01-JAN-2006 00:00:00.0000    UTC-TAI= -33
01-JAN-2009 00:00:00.0000    UTC-TAI= -34
); 
our $CHECK_DATES = 1;
our $GUESS_DATES = 1;
our @month_days = qw( 31 28 31 30 31 30 31 31 30 31 30 31 );
our @months= qw( JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC );
our %Months =  (     JAN => 1 ,
                     FEB => 2 ,
                     MAR => 3 ,
                     APR => 4 ,
                     MAY => 5 ,
                     JUN => 6 ,
                     JUL => 7 ,
                     AUG => 8 ,
                     SEP => 9 ,
                     OCT => 10,
                     NOV => 11,
                     DEC => 12
                  );
#our @doy_months = qw(0  31  59  90  120 151 181 212 243 273 304 334)      
our %Month_doy = ( 
                     JAN =>0,
                     FEB =>31,
                     MAR =>59,
                     APR =>90,
                     MAY =>120,
                     JUN =>151,
	                 JUL =>181,
                     AUG =>212,
                     SEP =>243,
                     OCT =>273,
                     NOV =>304,
                     DEC =>334
                   );
our $JD2000=2451545.0; #Current standard epoch (J2000.0)
our $GPSSE=2444244.5;  #GPS standard epoch 1980 jan 6d.0 
our $MJD0=2400000.5;
our @days=qw( monday tuesday wednesday thursday friday saturday sunday ); 
our $hh_min_sec = qr/([Tt:\s])
                     (\d{1,2}:.*?)
                     ([Zz]*)
                     $ 
                    /x;
sub WANT_DATE_ARRAY {
   $WANT_DATE_ARRAY = shift || 0;
}
sub CHECK_DATES {
   $CHECK_DATES = shift || 1;
}
sub GUESS_DATES {
   $GUESS_DATES = shift || 1;
}
sub current_time {
  my $now =strftime "%Y %m %d %H %M %S", localtime;
  return split(' ',$now)
}
sub get_date {
  my $date  = trim(shift) ||''; 
  my $out   = shift || ''; #|| 'cal,time,sp';
  my $delta = shift || 0;
  my ($y,$m,$d,$h,$min,$sec)=(0)x6;
  my ($y2,$yy,$doy,$dy,$jd,$mjd,$num,$hh,$secj2000)=(0)x8;
  my ($cal,$mmm,$epoch,$time,$sign,$ampm,$sfmt)=('')x7;
  my ($gpsw,$gdow,$dow,$wday,$gwsec,
      $delta_days,$delta_hours,
      $delta_mins,$delta_secs ) = (0)x9;

  my $input_date = $date;

  if ( ! $date ) { die "get_date: no input date\n"; }
  # init vars
  
  $time="00:00:00"; 
  $out = "cal,time,sp,short" if $out eq "sta";
  
  if ( $date =~/(today|now)/){  
      $date=strftime "%Y %m %d %H %M %S", localtime;
  } 
  if ( $date =~/(utc|gmt|zulu)/){  
      $date=strftime "%Y %m %d %H %M %S", gmtime;
  } 
  
  #check for sinex format YY:DOY:DAYSECS
  if ( $date =~ /^(\d{2}):(\d{3}):(\d{5})$/ ){ #snx format
     $y = yy2y($1);
     $doy = $2;
     $date = doy2cal($y,$doy);
     ($h,$min,$sec) = sec2hms($3);
     $date = "$date $h:$min:$sec";
  }
  elsif ( $date =~ /^(\d{1,4}):(\d{1,6}[|\.\d]*)$/ ) { 
     $gpsw = $1;
     $gwsec = $2;
     while( $gwsec > 604800 ) {
        $gpsw++;
        $gwsec -= 604800;
     }
     $secj2000 = sprintf("%-22.15f", ( $gpsw*7 + 2444244.5 - 2451545.0 )*86400 + $gwsec );
     ($y,$m,$d,$h,$min,$sec)=sec2cal($secj2000);
     $date = "$y-$m-$d $h:$min:$sec";
  }
  #get time information
  # hh:mm:ss = $2 
  if ( $date =~ s/$hh_min_sec//) {
   $time = setstrtime($2,$out);
   #print "$time|$1|$2|$3|$4|$5|\n";
   ($h,$min,$sec) = split /:/,$time;
   #print "|$h|$min|$sec|\n";
  }
  # julian date since $MJD0 and forward 1858-11-18                     
  #if ( $date =~ /(24\d\d\d\d\d)/ && $date >= $MJD0 ) {
  if ( $date =~ /^(24\d{5}(|\.\d*))$/ ) {
      $jd = $1;	
      ($y,$m,$d,$h,$min,$sec)=jd2cal($jd);
  }
  # modified julian date from 1900
  elsif ( $date =~ /^(\d{5}\.\d*)$/ ) {
     $mjd = $1;	
     $jd  = $mjd+$MJD0;
     ($y,$m,$d,$h,$min,$sec)=jd2cal($jd);
  }
  #year doy 
  elsif ( $date =~ /^([12]\d{3})([\s,_-])(\d{1,3})$/ ){ 
    ($y,$doy)=($1,$3); 
    ($y,$m,$d) = doy2cal($y,$doy);
  }
  # decimal year
  elsif ( $date =~ /^([12]\d{3}(|\.\d+))$/ ) { #decimal year
      $dy = $1;	
      $jd = dy2jd($dy);
      ($y,$m,$d,$h,$min,$sec)=jd2cal($jd);
  }
  # secj2000 
  #elsif ( $date =~ /^([\+-]|)(\d+\.\d+)$/ ) { #decimal year
  #    $sign= $1 eq '-' ? -1:
  #           $1 eq '+' ? 1 :
  #           $1 eq ''  ? 1 :
  #                       1 ;
  #    $secj2000 = $sign*$2;	
  #    ($y,$m,$d,$h,$min,$sec)=sec2cal($secj2000);
  #}
  # standard iso date: 
  # yyyy-mm-dd hh min sec 
  #  $1  $3 $5 $8 $10 $12
  elsif ( $date =~ /^               # match beginning
                 (\d{4})         # yyyy
                 (\D+)           # non-digit
                 (\d{1,2})       # mm
                 (\D+)           # non-digit
                 (\d{1,2})       # dd
                 (|
                   (\D+|)           # non-digit
                   (\d{1,2}|)       # hh
                   (\D+|)           # non-digit
                   (\d{1,2}|)       # min
                   (\D+|)           # non-digit
                   (\d+|\d+\.\d+|)       # sec
                   (\D+|)           # non-digit
                 )
                 $               # match end
                 /x
      ) { 
     ($y,$m,$d)=( $1 , $3, $5 );
     $h ||=$8||0;
     $min ||=$10||0;
     $sec ||=$12||0;
     #($h,$min,$sec) = ($8||0,$10||0,$12||0);
     #$cal = $date;  
  }       
  # gipsy date YYMMMDD
  elsif ( $date =~ /^(\d{2,4})
                     ([-\.\s\/\\!\|,;]|)
                     ([a-zA-Z]{3,9})
                     ([-\.\s\/\\!\|,;]|)
                     (\d{2,4})
                    $
                   /x                  ) {
    my($l1,$l5)=(length($1),length($5));
    ($y,$d) = $l1 == 4             ?  ( $1       , $5       ) :
              $l5 == 4             ?  ( $5       , $1       ) :
              $l1 == 2 && $1 > 31  ?  ( yy2y($1) , $5       ) :
              $l5 == 2 && $5 > 31  ?  ( yy2y($5) , $1       ) :
              $l1 == 2 && $l5 == 2 ?  ( yy2y($1) , $5       ) :
              $2 eq '' && $4 eq '' ?  ( yy2y($1) , $5       ) :
                                      ( 0        , 0        ) ;
    $mmm = unpack('a3',$3);
    $m = month2m($mmm);
    if ( $y && $d && $m ) { 
      $y2 = unpack('x2a2',$y);
      $epoch = sprintf"%02d%3s%02d",$y2,$mmm,$d;
    }
  }
  # calendar number 
  elsif ( $date =~ /^\d{8}$/ ) { 
    ($y,$m,$d)=unpack('a4a2a2',$date);
  }
  # gps week day WWWWD
  elsif ( $date =~ /^(\d{1,4}?)([0-6])$/) { 
     $gpsw = $1;
     $gdow = $2;
     $jd = gwd2jd("$gpsw$gdow");
     $jd += hms2dd($h,$min,$sec); 
     ($y,$m,$d,$h,$min,$sec)=jd2cal($jd);
  } 
    #guess if it is required
  if ( ( ! $y || ! $m || ! $d ) && $GUESS_DATES ) {
    ($y,$m,$d) = guess($date) ;
  }
  # check what we got so far
  if ( ! $y || ! $m || ! $d  ) {
       carp "get_date: Unrecognized date format: $date\n";
       return ''
  }
  # validate 
  if ( $CHECK_DATES && ! ok_date($date,$y,$m,$d,$h,$min,$sec) ) {
     carp "Invalid date $date ;  input=|$input_date| \n";
     return ''
  }
  

  if ( $delta ) {
    my @d = delta_date($delta,$y,$m,$d,$h,$min,$sec);
    if ( scalar @d == 1 ) {
      return $d[0]
    }  
    else {
      ($y,$m,$d,$h,$min,$sec) = @d;
    }
  } 

  #at this point we should have a valid date
  if ( ! $out  ) {
    return wantarray ? ($y,$m,$d,$h,$min,$sec): "$y $m $d $h $min $sec";
  }
  
  $time = setstrtime("$h:$min:$sec",$out); 
  $hh=$h+$min/60+$sec/3600;
  # get the date in the requested format 
  if ( $out =~/(today|now)/){  
      $date=strftime "%Y-%m-%d ; %H:%M:%S ; %A ; %j", localtime;
  }
  # strftime in $2
  elsif ( $out =~/(,*)
                  (
                    .*
                      (%[aAbBcCDeEFGghHIjklMnOpPrRsStTuUVwWxXyYzZ%])+
                    .*?
                  )
                  (,*)
                  /x ) {
    $doy =  $doy || cal2doy($y,$m,$d);
    $date =  strftime $2, date2localtime($y,$m,$d,$h,$min,$sec,$doy); 
  } 
  elsif ( $out =~ /(eop|char)/ ) {
     $date = sprintf("%02d-%s-%4d %02d:%02d:%7.4f",
                     $d,uc(m2month($m)),$y,$h,$min,$sec
                    );
  }
  elsif ( $out =~ /cal/ )   {  
       $date=sprintf("%d-%02d-%02d",$y,$m,$d); 
  }  
  elsif ( $out =~ /doy/ )   {  
      $doy = cal2doy($y,$m,$d);
      $date=sprintf ("%d %03d",$y,$doy); 
  }  
  elsif ( $out =~ /snx/ ) {
       $yy  = unpack("x2a2",$y);
       $sec = $h*3600+$min*60+int($sec);
       $doy = cal2doy($y,$m,$d);
       $date = sprintf ("%2s:%03d:%05d",$yy,$doy,$sec);
  }  
  elsif ( $out =~ /(dd|dh|dm|ds)/ ) {  
       $jd=cal2jd($y,$m,$d,$h,$min,$sec) if ! $jd;
       $jdnow = cal2jd(current_time());
       $delta_days  =   $jdnow - $jd;
       $delta_hours =   $delta_days*24;
       $delta_mins  =   $delta_hours*60;
       $delta_secs  =   $delta_mins*60;
       $date = $1 eq 'dd' ? $delta_days  :
               $1 eq 'dh' ? $delta_hours :
               $1 eq 'dm' ? $delta_mins  :
               $1 eq 'ds' ? $delta_secs  :
                            $delta_days  ;
  } 
  elsif ( $out =~ /dy/ )    {  
       $jd=cal2jd($y,$m,$d,$hh);
       $date=sprintf ("%.18f", jd2dy($jd));
  } 
  elsif ( $out =~ /(epoch|date)/ ) {  
       (undef,$y2) = unpack("a2a2", $y); 
       $mmm=uc m2month($m);
       $date=sprintf("%02d%3s%02d",$y2,$mmm,$d);
  } 
  elsif ( $out =~ /(gpsw|gdow|gwd|gwsec)/ ) {
       $jd=cal2jd($y,$m,$d,$h,$min,$sec) if ! $jd;
       $gpsw= jd2gpsw($jd);
       $dow=  jd2dow($jd);
       $gdow=  dow2gdow($dow);
       $gwsec=$gdow*86400+ $h*3600+$min*60+$sec;
       $gwd  = "$gpsw$gdow";
       $date = $1 eq 'gwd'    ? $gwd         :
               $1 eq 'gpsw'   ? $gpsw        :
               $1 eq 'gdow'   ? $gdow        :
               $1 eq 'gwsec'  ? $gwsec       :
                                $gwd         ;                         
  } 
  elsif ( $out =~ /^jd/ )   { 
      $date= cal2jd($y,$m,$d,$h,$min,$sec);
  }
  elsif ( $out =~ /mjd/ )   { 
      $jd= cal2jd($y,$m,$d,$h,$min,$sec) if ! $jd;
      $date= jd2mjd( $jd );
  }
  elsif ( $out =~ /(sj2000|secj2000)/)  { 
       $jd= cal2jd($y,$m,$d,$h,$min,$sec) if ! $jd;
       $date = fix_secj2000(sprintf("%.18g",($jd - $JD2000)*86400));
  } 
  elsif ( $out =~ /wday/ )  {  
            $jd=cal2jd($y,$m,$d,$hh) if ! $jd;
            $dow=  jd2dow($jd);
            $wday= ucfirst(dow2days($dow));
	    $date=$wday;
  }
  elsif ( $out =~ /zulu/ ) {
     $date=sprintf("%d-%02d-%02d",$y,$m,$d); 
	 $date.="T${time}Z";   
  }
  
  if ( $out =~ /time/ && $out !~ /(today|now|snx|zulu)/ ) {
	 $date.=" $time";   
  }
  if ( $out =~ /sp/ ){  
       $date=~s/(-|:|;)/ /g;
       $date=~s/\s+/ /g; 
  }
  if ( $out =~ /num/  && $out !~ /(today|now)/ ) {
        #$date=~s/(:| )00/0/g;
        $date=~s/(\s+|-|:|\..*$)//g; 
  }
  return $date
}
sub print_cal {
  print "($y,$m,$d,$h,$min,$sec)\n";
}
sub delta_date {
  my $delta = shift;
  my ($y,$m,$d,$h,$min,$sec) = @_;
  my ($dd,$dh,$dm,$ds) =(0)x4;
  $jd = cal2jd($y,$m,$d,$h,$min,$sec) ;
  if ( $delta =~ /^([\+-]|)        # sign 
                    (\d+)
                    (:\d+.*|) 
                   $               # end of line
                 /x                                 ){
        $sign = $1 eq '-' ? -1:
                $1 eq '+' ?  1:
                $1 eq ''  ?  1:
                             1;
        $dd = $2;
        $dh = $3 || 0; 
        $dh  =~ s/:/ /g;
        ($dh,$dm,$ds) = split ' ',$dh;
        $dh ||= 0; $dm ||= 0; $ds||=0;
        #$dd = $sign*$dd;
        #print "delta: |$delta|$dd|$dh|$dm|$ds|\n";
        $dd += ($dh/24.00 + $dm/1440.00 + $ds/86400.00); 
        #print "# jd: $jd "; 
        $jd += ($dd*$sign);
        #print "jd+dd: $jd dd:$dd sign:$sign\n";
  }
  elsif ( $delta =~ /^([\+\-]*)(\d+|\d+\.\d+)$/ ) {
        $sign = $1 ||'';
        $jd += $delta; 
  }
  elsif (my $jd2 = get_date($delta,'jd') ) {
    $delta = $jd - $jd2;
    return $delta 
  }

  
   ($y,$m,$d,$h,$min,$sec) = jd2cal($jd); 
     #(undef,$h,$min,$sec) = dd2dhms(frac($jd));
     #$time = join ':',($h,$min,$sec);
     #$hh=$h+$min/60+$sec/3600;
   
    return ($y,$m,$d,$h,$min,$sec)
    
}
sub load_leap_seconds {
  foreach my $l ( grep /UTC/, $LEAPSECS =~ /(.*)/gm ) {
     if ( $l =~/(.*?)(UTC-TAI= )(-\d+)/ ) {
       push @LEAP_SECONDS, [get_date($1,'secj2000'), $3];
     }
  }
  return scalar @LEAP_SECONDS
}
sub gps2utc {
  my $gps=0;
  my $gpsse=-630763200.00000000; #$GPSSE in secj2000
  my ($y,$m,$d,$h,$min,$sec) = (0)x6;
  if ( @_ == 1 ) {
    $gps= shift ;
  }
  else {
    $gps = cal2sec(@_);
  }
 #load array if it is not loaded yet 
 if ( ! @LEAP_SECONDS ) { 
    load_leap_seconds() or croak "\@LEAP_SECONDS is empty!\n";
 }

 my $utc_tai = 0;
 my $epoch = '';
 my $utc = $gps;
 foreach my $l ( @LEAP_SECONDS ) {
   ($epoch,$utc_tai) = @$l;  
   next if $epoch < $gpsse;
   if ( $epoch <= $gps ) { 
     $utc = $gps + ( $utc_tai + 19 );
   }
   else {
    last;
   }
 }
 return wantarray ? sec2cal($utc) : $utc 
}

sub date2localtime {
   my ($y,$m,$d,$h,$min,$sec,$doy) = @_;
   return ($sec,$min,$h,$d,$m-1,$y-1900,$doy,0)
}
sub ok_date {
  my $date = shift;
  ($y,$m,$d,$h,$min,$sec) = @_;
  $m =~ s/^0//;
  $d =~ s/^0//;
  my $invalid = "|$date|=>|$y|$m|$d|$h|$min|$sec|=> invalid";
  my $ok = 1;

  if ( $y !~ /^\d{2,4}$/ ) {
    carp "$invalid year   : $y\n";
    $ok=0;
  }
  if (  $m < 1 || $m > 12 ) {
    croak "$invalid month  : $m\n" ;
    $ok=0;
  }
  if (  $d < 1 ||  $d > $month_days[$m-1]+ isleap($y) ) {
    carp "$invalid day    : $d\n";
    $ok=0;
  }
  if (  $h < 0 || $h > 23 ) {
    carp "$invalid hours  : $h\n";
    $ok=0;
  }
  if (  $min < 0 || $min > 59 ) {
    carp "$invalid minutes: $min\n";
    $ok=0;
  }
  if (  $sec < 0 || $sec >= 60 ) {
    carp "$invalid seconds: $sec\n";
    $ok=0;
  }
  return $ok
}


sub trim {
  my $s = shift;
  $s =~ s/^\s+//;
  $s =~ s/\s+$//;
  return $s
}
sub hms2dd {
  my ($h,$min,$sec)=(0,0,0);
  ($h,$min,$sec)=@_;
  return $h/24.000+$min/1440.000+$sec/86400.00
}
sub dd2dhms {
   my $dd = shift;
   my $d = floor($dd);
   my $q = $dd - $d;
   my $h = floor ( $q*24 );
   my $m = floor ( $q*1440 - $h*60 );
   my $s = floor( $q*86400 - $h*3600 - $m*60);
   my $fsec = $q*86400-$h*3600-$m*60-$s;
   my $epsilon=1.0e-4;
   if ( (1.0 - $fsec) < $epsilon ){
     $s++;
     #if ( ( $s - 60 ) < $epsilon ) {
     if ($s > 60) {
        $m++;
        $s=0;
     }
     #if ( ($m-60) < $epsilon ) {
     if ($m > 60) {
        $h++;
        $m=0;
     }
     $fsec=0;
   }
   elsif ( $fsec < $epsilon ) {
    $fsec=0;
   }
  
   return ( $d,$h,$m,$s+$fsec )
}
sub setstrtime {
   my $time = shift;
   my $out  = shift ||'';
   $time =~ s/(AM|PM)//i;
   $ampm = $1 || '';
   ($h,$min,$sec)=split(/:/,$time);
    $h||=0;$min||=0;$sec||=0;
    $h+=12 if uc($ampm) eq 'PM';
    $sfmt="%-10.7lf";
    $sfmt = "%02d" if $out =~ /(int|zulu)/;
    $sfmt = "%05.2f" if $out =~ /short/;
    $time = sprintf("%02d:%02d:$sfmt",$h,$min,$sec);
    return $time
}
sub guess {
  my  $date = uc shift;
  my (%d,$v,@u);
  my ($y,$m,$d)=(0)x3;
  while ( $date =~ /([A-Z]+|[0-9]+)/g ){
    $v = trim($1);
    if ( exists $Months{$v} ) { 
          $m = $Months{$v} ;
    } elsif ( $v =~ /^\d{4}$/ && ($v > 1900 && $v < 2100) ) { 
         $y  = $v ;
    } elsif  ( $v =~ /^\d{1,2}$/ && $v > 12 && $v <= 31) {
         $d  = $v ; 
    } else {
         push @u,$v;
    }
  }
  if ( @u == 1 && $y &&  $d ) {
       $v = $u[0];
       $m = $v if $v =~ /^\d+$/ && $v > 0 && $v <= 12;
  } elsif ( @u == 1 && $y && $m ) { 
       $v=$u[0];
       $d=$v if $v !~ /[A-Z]/ && $v > 0 && $v <=31 ; 
  }
  
  if ( ! $m || ! $d || ! $y ) { 
       carp "Warning: guessing date ".
            "by assigning numbers ordered by month/day/year\n";
      # USA convention
      #print @u,"\n"; 
      while ( @u ) { 
        $v = shift @u;
        next if $v =~ /[A-Z]/;
        if ( ! $m && ($v > 0 && $v <= 12 ) ) {
           $m = $v; 
        }elsif ( ! $d  && $m && ( $v > 0 && $v <= 31 ) ) {
           $d = $v ;
        } elsif ( ! $y &&  $d && $m && $v > 0 && $v <= 99 ) {
           $y = yy2y($v);    
        }
      }
  }  
  #print "$m / $d / $y\n";
  if ( $m && $d && $y ) {
       #return sprintf("%4d-%02d-%02d",$y,$m,$d);
       return ($y,$m,$d)
  } 
  else {  
    return  0 
  }
}

sub dow2days { 	# number of day of week to civil name 
  my $d=shift;
  return  $days[$d]
}
sub m2month { #month number to month name
  my $m=shift;
  return $months[$m-1]
}
sub month2m { #month name to month number
  my $month = unpack 'a3', uc shift;
  return sprintf("%02d", $Months{$month});
  #return $Months{$month};
}

sub epoch2dy {
	my $epoch = shift;
	my $dy = jd2dy(cal2jd(epoch2cal($epoch)));
	return $dy
}

sub cal {
   my ($y,$m,$d,$h,$min,$sec) = @_;
   if ( $sec >= 60 )
   {
     $min++;
     $sec-=60;
   }
   if ( $min >= 60 )
   {
     $h++;
     $min-=60;
   }
   if ( $h >= 24 )
   {
     $d++;
     $h-=24;
   }
   if ( $d > ($month_days[$m-1] + isleap($y)) ) { 
     $m++; 
     $d-=$month_days[$m-1];
   }
   
   if ( $m > 12 ) 
   { 
     $y++; 
     $m-=12;
   }
   
   if ( wantarray ) {
     return ($y,$m,$d,$h,$min,$sec)
   }
   else {
     return  sprintf ('%4d-%02d-%02d',$y,$m,$d)
   }
}


sub epoch2cal {
  my ($y,$mmm,$d) = unpack("a2a3a2",uc shift);
   $m = $Months{$mmm};
   $y = yy2y($y);
   return cal($y,$m,$d) 
}

sub y2yy {
  my $y = shift;
  return unpack ('x2a2',$y)
}

sub yy2y {
   my $y = shift;
   $y = $y < 80 ? $y+2000 : $y+1900;
   return $y
}

sub cal2epoch {
  my ($y,$m,$d) = @_ == 1 ? split /-/, $_[0] : @_;
  my $yy=unpack('x2a2',$y) ;  #,substr($y,2);
  return sprintf ('%02d%3s%02d',$yy,$months[$m-1],$d) 
}

sub cal2doy {  # year, month, day to doy 
  my ($y,$m,$d) = @_ == 1 ? split /-/, $_[0] : @_;
  foreach my $numdays (@month_days[0..$m-2]) {
    $d += $numdays;
  }
  $d++ if $m > 2 && isleap($y);
  return wantarray ? ($y,$d) : sprintf("%03d",$d)
}



sub cal2jd { # year,month,day, ut to Julian Date   
  my ($y,$m,$d) = @_ == 1 ? split /-/, shift : (shift,shift,shift);
  my $h = shift;
  my $min = shift;
  my $sec = shift;
  my $ut= 0.0;
  $h   ||=  0;
  $min ||=  0;
  $sec ||=  0.0;
  $ut = $h + $min/60.0 + $sec/3600.00;
  #$ut-= 0.5;
  if ( $m <= 2 ) { 
  	$y=$y-1;
	$m= $m+12;
  }
  my $jd = int(365.25*$y) + int(30.6001*($m+1))+ $d+ $ut/24.0 + 1720981.5;

  return $jd ;
}

sub cal2dy {
  return jd2dy(cal2jd(@_))
}
sub fix_secj2000 {
   my $sec = shift;
   my $isec = int($sec);
   my $fsec = $sec - int($sec);
   if ( 1 - $fsec < 1e-4 ) {
     $sec = $isec+1;   
   }
   elsif ( $fsec < 1e-4 ) {
     $sec = $isec;
   }
   return $sec;
}
sub cal2sec {
   my $jd= cal2jd(@_);
   return fix_secj2000( sprintf("%14.4f",($jd - $JD2000)*86400.0000));
}
sub sec2jd { 
   my $sec = shift;
   return $sec/86400.0000 + $JD2000
}
sub sec2cal {
   my $sec = shift;
   return jd2cal($sec/86400.0000 + $JD2000)
}

sub sec2hms {
  my $sec = shift;
  my $h = int($sec/3600);
  my $min = int($sec/60 - $h*60);
  $sec -= ($min*60+$h*3600);
  return ($h,$min,$sec)
}

sub jd2mjd { #Julian date to modified julian date
  my $jd=shift;
  return sprintf"%.11f", $jd-$MJD0;
}

sub mjd2sec {
   my $mjd = shift;
   return cal2sec(jd2cal( $mjd + $MJD0 ) )
}

sub jd2cal {       	#Julian date to year,month, day
  my $jd=shift;
  my($a,$b,$c,$d,$e,$D,$M,$Y,$q,$h,$min,$sec);
  $jd += 0.5;
  $q = $jd - floor($jd);
  #$q = frac($jd);
  $a= int($jd);
  $b= $a + 1537;
  $c= int(($b-122.1)/365.25);
  $d= int(365.25 * $c );
  $e= int ( ($b-$d)/30.6001);
  #$D = $b - $d - int (30.6001 * $e) + frac($jd + $hd);
  $D = $b - $d - int (30.6001 * $e) + $q;
  $M = $e -1 - 12* int($e/14);
  $Y = $c - 4715 - int((7+$M)/10);
  (undef,$h,$min,$sec) = dd2dhms($q);
  #($Y,$M,$D,$h,$min)= map{ int($_) } ( $Y,$M,$D,$h,$min);
  ($Y,$M,$D,$h,$min,$sec)= split ' ',sprintf( "%4d %02d %02d %02d %02d %f",$Y,$M,$D,$h,$min,$sec);
  return  cal($Y,$M,$D,$h,$min,$sec)
  
}

sub jd2sec {
   my $jd = shift;
   return cal2sec(jd2cal($jd))
}


sub jd2dow { # day of week for julian date jd :0=monday, 6=sunday;
  my $jd=shift;
  return (int($jd+0.5) % 7)
}

# dow2gdow: JD day of week to GPS day of week: 0 = sunday 6=saturday
sub dow2gdow { 
   my $d = shift;
   my $gdow = ( $d + 1 ) % 7 ;
   return $gdow	
}

sub jd2dy { #julian date to decimal year
  my $jd=shift;
  my $jd2k=$JD2000 - 0.500000 ; 
  return(($jd-$jd2k)/365.25 + 2000.000000);
}
sub dy2jd { #decimal year to julian date
  my $dy=shift;
  my $jd2k=$JD2000 - 0.5 ; 
  return (($dy-2000)*365.25+$jd2k)
}
sub dy2epoch {
  my $dy = shift;
  return cal2epoch(jd2cal(dy2jd($dy)))
}
sub dy2cal {
   my $dy = shift;
   return jd2cal(dy2jd($dy))
}
sub jd2gpsw { # julian date to gps week 
 my $jd=shift;
 return int(($jd-$GPSSE)/7) 
}
sub gwd2jd {
   my $gwd = shift;
   $gwd =~ /(\d+?)(\d)$/;
   my ($gpsw,$gdow) = ($1,$2);
   my $jd = $gpsw * 7 + $GPSSE + $gdow;
   return $jd
}
sub frac { #auxiliary function to obtain the fractional part of a number 
  my $num = shift;
  my $l = length $num;
  $num=sprintf("%.*f",$l,$num);
  $frac = (split(/\./, $num))[-1];
  return "0.$frac"
}

sub doy2cal { # year, doy to month, day
  my $y = shift;
  my $d = shift;
     $d =~ s/^0+//;
  my $m = 1;
  foreach (@month_days) {
    my $numdays = $_;  # make a copy of $_ since may modify it
    $numdays++ if $m == 2 && isleap($y);
    last if $d <= $numdays;
    $d -= $numdays;
    $m++;
  }
  return cal($y,$m,$d,0,0,0)
}

sub isleap { # return 1 if it is a leap year 0 otherwise
  my $year = shift;
  my $leap = 0;
  if ($year % 4 == 0) {
    $leap = 1;
    if ($year % 100 == 0) {
      $leap = 0;
      if ($year % 400 == 0) {
        $leap = 1;
      }
    }
  }
  
  return $leap;
}

1;
__DATA__
=head1 NAME

GPS::DATES 

=head2 GPS Dates And Time Extremely Simple 

=head1 SYNOPSYS

use GPS::DATES;

my $date_time_in       = 'today';

my $format = 'doy,time';   #day of year

my $delta         = -10;     #ten days ago

my $date_time_out = get_date( $date_time_in,$format,$delta);

print $date_out,"\n";

my @date_time = get_date($date_time_in,'',$delta);


=head1 DESCRIPTION 

B<GPS::DATES> Handle conversions between different date and time formats.

=head2 Normal Usage

The main function is get_date(), which takes 3 strings as arguments 
I<date_time_in>, I<format> and I<delta>, and returns by default a string 
with the date and time in the specified I<format> applying a time shift given
by B<$delta>. The default for delta is 0. If the format string is the empty
string an array with  (YYYY,MM,DD,hh,mm,ss.ss) is returned.

my $date_time_out = get_date( $date_time_in,$format,$delta);

In the second form, B<delta> can be replaced by a B<date>, and the difference
in days and fractions of a day (date_time_1 - date_time_2)  will be returned.

my $delta_days_1_2 = get_date( $date_time_1,$format,$date_time_2);

Another way of using get_date is to give a empty string as format, in which
case it will return an array with (YYYY,MM,DD,hh,mm,ss.ss)

my ($yyyy,$mm,$dd,$hh,$min,$sec) = get_date($date_time_in,'',$delta);

or

my ($yyyy,$mm,$dd,$hh,$min,$sec) = get_date($date_time_in);


=head1 OPTIONS

=over 1

=item B<date_time_in>

Is the string that get_date will try to parse. The format is:

B<date_time_in> = date[sep][time]

=over 1  

=item B<date> format 

Note that some formats are ambiguous, like seconds past J2000.
the format is a  real number that is not compatible with other dates
which are also real numbers, like mjd,jd and dy, they overlap at 
some point in time. You can use cal2sec() and sec2cal() to handle 
dates in seconds past J2000.

yyyy=4 digit year, mm=2 digit month , dd=2digit day of month, 

mmm= 3 char month abreviation.

dy= decimal year

jd= Julian date ( from 1858-11-18 and forward )

doy= (day of year) 

sssss= day seconds 1 to 86400 

snx= SINEX date yy:doy:sssss

epoch= date format used in GIPSY.

=item B<NAME  FORMAT>  

=item   cal   B<yyyy-mm-dd>   

=item   cal   B<yyyymmdd>       

=item   doy   B<yyyy,doy>    

=item   epoch B<yymmmdd>    

=item   dy    B<yyyy.nnnn> 

=item   snx   B<yy:doy:sssss>

=item   eop   B<dd-mmm-yyyy hh:min:sec>

=item   char  B<dd-mmm-yyyy hh:min:sec>

=item   zulu  B<yyyy-mm-ddThh:min:secZ>

=item   jd    B<nnnnnnn.nnnnnn>

=item   mjd   B<nnnnn.nnnnn>

=back

=over 1

=head2 B<Time format>

The time information is appended at the end of the B<date> string
using a separator [sep] that could be: T,t,:,' '.

Example using B<hh:min:secs>

my $date = '2006-11-03 12:20:34.5';

=back

=head2 Misc. Variables.

=item B<CHECK_DATES> = 1 (default)

use CHECK_DATES(0) to set it to 0 and get_date will not check for a valid date.

=item B<GUESS_DATES> = 1 (default)

Use GUESS_DATES(0) to set it to 0 and get_date will try to 
further parse the date and guess a valid date in the case of umbiguous data.











