package GPS::DataTable;
use Scalar::Util qw(looks_like_number);
use Fortran::F90Format;
use GPS::StrUtil;
use Carp;

sub new {
  my $class = shift;
  my $self = { @_ };

  bless $self,$class;

  return $self->init(@_);

}

sub init {
  my $self = shift;
  my %opt  = ( fmt        => '',
               pack_fmt   => '',
               cfmt       => '',
               fields_in  => '',
               fields_out => '',
	           @_,
              );
  my @keys = split(/\,/,$opt{fields_in});
  croak "No fields_in !\n fields_in => 'name1,name2,..'\n" if ! @keys;
  my @reserved_fields =qw/data negate compare/ ;
  my %r;
  @r{@reserved_fields} =();
  foreach my $f ( @keys  ) {
    if ( exists $r{$f} ) {
  croak "field_in: $f conflicts with internal function, pick another name :)\n";
    }
  }
  
  $self->{fields} = \@keys;
  $self->{format}='';
  if ( $opt{fmt} ) {
    $self->{format} = Fortran::F90Format->new( fmt => $opt{fmt} );  	  
  }
  $self->{pack_fmt} = $opt{pack_fmt};
  $self->{cfmt}     = $opt{cfmt};
  if ( exists $opt{data} ) {
    $self->{data}   = $opt{data};
  } else {
    $self->load(%opt);  
  } 
  return $self
}
             
sub load {
  my $self = shift;
  my %opt  = (
       fields_in => join(',',@{$self->{fields}}),
       file => '',
       skip => 0,
       head => 0,
       tail => 0,
       field_sep  => ' ',
       trim       => 1,
       start_comment => '#',
       fmt        => '', # will assume fields separated by spaces
       @_,
       
  );  
  if ( ! $opt{fields_in} ) {
    croak "You need to supply fields_in => 'key1,key2,...,keyN'\n";
  }
  my @keys = split(/\,/,$opt{fields_in});
  my @k_out = @keys;
  @k_out = split (/\,/, $opt{fields_out}) if $opt{fields_out};
  my %keys_out;
  @keys_out{@k_out} = @k_out;
  
  $self->{fields} = \@keys;
  $opt{field_sep} = escape( $opt{field_sep} );
  my (@fields,$nrows);
  my $fh = $opt{fh};
  if ( $opt{file} ) {
     #croak "can't find file: $opt{file}\n" if ! -e $opt{file};
     open( $fh,"< $opt{file} ") || croak "Couldn't open  $opt{file} $!\n";
  }
  while ( <$fh> ) {
      next if $. < $opt{skip};
      chomp;
      #s/#.*//;
      #s/^\s+//;
      #s/\s+$//;
      next unless length;
      if ( $self->{format} ) {
	     @fields = $self->{format}->read($_);      
      }
      elsif( $self->{pack_fmt} ) {
	     @fields = unpack( $self->{pack_fmt}, $_ );      
      }
      else  {
        s/$opt{start_comment}.*//;
        s/$opt{field_sep}+/$opt{field_sep}/g;
        $_ = trim($_) if $opt{trim};
        next unless length;
        @fields = split(/$opt{field_sep}/,$_);
      }
      my $h = {};
      @$h{@keys}=@fields;
      if ( @k_out < @keys ) {
         foreach my $k ( @keys ) {
	       delete $$h{$k} if ! exists $keys_out{$k};
	     }
      }
      push @{$self->{data}} ,$h ;   	      
   };
  if ( $opt{tail} ) { 
    @{$self->{data}}=@{$self->{data}}[ -$opt{tail} .. -1];
  }
  if ( $opt{head} ) { 
    @{$self->{data}}=@{$self->{data}}[ 0 .. $opt{head} -1 ];
  }
  close($fh);
  return $self
 
}

sub print {
  my $self = shift;
  my %opt = (
     data       => '',
     fields     => '',
     fmt        => $self->{format},
     cfmt       => $self->{cfmt},
     pack_fmt   => $self->{pack_fmt},
     new_line   => "\n",
     trim       => 0,
     field_sep  => ' ',
     fh         => \*STDOUT,
     file       => '',
     @_, 
  );
  my $fh = $opt{fh};
  if ( $opt{file} ) {
     $fh = \*FILE;
     open( $fh , "> $opt{file}") or croak "couldn't open file: $opt{file}\n"; 
  }
  my $data = $opt{data} || $self->{data};
  my @keys = @{$self->{fields}};
  #my @keys = sort keys %{ $$data[0] };
  @keys = split (/\,/,$opt{fields}) if $opt{fields};
  my $l;
  foreach my $h ( @$data ) {
    $l='';
    @$h{@keys} = map { trim($_) } @$h{@keys} if $opt{trim};
    if ( $opt{fmt} ) {
      $l = $opt{fmt}->write( @$h{@keys} );
      #$l = sprintf($self->{format}{pf_fmt},@$h{@keys});
      #print join(" ",@$h{@keys}),"\n";
    } elsif( $opt{cfmt} ) {
      $l = sprintf($opt{cfmt},@$h{@keys});
      $l .= $opt{new_line};
    } elsif( $opt{pack_fmt} ) {
      $l = spack( $opt{pack_fmt},@$h{@keys});
      $l .= $opt{new_line};
    }else{ 
      $l = join($opt{field_sep},@$h{@keys}); 
      $l = trim($l) if $opt{trim};
      next unless length($l);
      $l .= $opt{new_line};
    }
    print $fh $l;    
  };
  if ( $opt{file} ) {close $fh;} 
}

sub read_array2d {
   my $self = shift;
   my %opt = ( array2d => '',
               @_,
             );
   my @fields = @{$self->{fields}}; 
   my $n = @fields;
   my $array = $opt{array2d};
   my $data = [];
   foreach my $r ( @{$array} ) {
      my $h={};
      @{$h}{@fields} =  @{$r}[0..$n-1];
      push @$data, $h;
   }
   $self->{data} = $data;
}
sub write_array2d {
   my $self = shift;
   my %opt = ( array2d => [],
               data    => $self->{data},
               @_,
             );
   my @fields = @{$self->{fields}}; 
   my $n = @fields;
   my ( $i,$j)=(0)x2;
   foreach my $r ( @{$opt{data}} ) {
      $j=0;
      foreach my $v ( @{$r}{@fields} ) {
        $opt{array2d}[$i][$j++] = $v;
      }
      $i++;
   }
   return $opt{array2d};
}

sub read_lines {
   my $self = shift;
   my %opt = ( ln_array => '',
               @_,
             );
   my $input = join("\n",@{$opt{ln_array}});
   open(my $fh,"<",\$input);
   $self->load( fh => $fh );
   return $self->{data}
}

sub write_lines {
    my $self = shift;
    my %opt  = (
       @_,
    );
    my $output;
    my $array=[];
    open(my $fh, ">",  \$output);
    $opt{fh} = $fh;
    $self->print( %opt ); 
    @$array = split(/\n/,$output);
    return $array
}
sub row_list {
  my $self = shift;
  my %opt  = (
       data => $self->{data},
       @_,
    );
   return @{$opt{data}}
}
sub sort { # my sort
   my $self = shift;
   my %opt = (
      data      => $self->{data},
      field     => '',
      compute   => sub { my ($h,$f)=@_; $h->{$f}; },
      rev       => '',
      unique    => '',
      @_,   
   );
   my $data  = $opt{data}; 	
   my $field = $opt{field};
   my %fname;
   @fname{ @{$self->{fields}} }=(); 
   croak "field: $field is not a valid name\n" if ! exists $fname{$field};
   my $compute = $opt{compute};
   my $rev = $opt{rev} ? '_reverse': '' ;
   my %fcomp = ( num           =>  sub { $a->[0] <=> $b->[0] },
                 num_reverse   =>  sub { $b->[0] <=> $a->[0] }, 
                 alpha         =>  sub { $a->[0] cmp $b->[0] },
                 alpha_reverse =>  sub { $b->[0] cmp $a->[0] },  
                ); 
   my $unique_func = $opt{unique} ? \&unique_field : \&non_unique ; 
   my (@sorted,$ftype,$sort_func);
   # see what type of field is 
   $ftype = $self->is_numeric( field => $field ) ? 'num' : 'alpha';
   $sort_func = $fcomp{"$ftype$rev"};
   @sorted =  sort $sort_func  
             (map  { [ $compute->($_,$field) , $_ ] } @$data) ;  
   @$data  = $opt{unique} ? unique(@sorted) : non_unique(@sorted);
   
   $self->{data} = $data;
   return $self->{data}
}
sub non_unique {
  return map { $_->[1] } @_
}
sub unique {
  my %seen;
  return map { $_->[1] } grep { ! $seen{$_->[0] }++ } @_
}
sub is_numeric {
  my $self = shift;
  my %opt  = ( 
     data  => $self->{data},
     field => '',
     @_);
  my $is_num = 1;
  foreach my $c (  $self->get_col( col => $opt{field}, data => $opt{data} ) ) {
    if ( ! looks_like_number($c) ) {
       $is_num = 0;
       last;
    }
  }
  return $is_num 
}
sub minmax {
  my $self = shift;
  my %opt = (
     data  => $self->{data},
     field => '',
     @_,
  );
  my $data = $self->get_data ( data => $opt{data} );
  croak "No field selected\n" if ! $opt{field};
  my ($field) = split /\,/, $opt{field};
  croak "Not a valid field\n" if ! exists $opt{data}[0]{$field};
  my $mm = $self->sort( data => $data, field => $field );
  #my ($min,$max) = ($mm->{data}[0]{$field},$mm->{data}[-1]{$field});
  my ($min,$max) = ($mm->[0]{$field},$mm->[-1]{$field});
  return ($min,$max)
}
sub interval {
  my $self = shift;
  my %opt = (
     data  => $self->{data},
     field => '',
     min   => 0,
     max   => 0,
     @_,
  );
  croak "No field selected\n" if ! $opt{field};
  my ($field) = split /\,/, $opt{field};
  croak "$field is Not a numeric field\n" if ! $self->is_numeric(%opt);
  croak "Not a valid field\n" if ! exists $opt{data}[0]{$field};
  my $i = $opt{max} - $opt{min};
  croak "Not a valid interval $opt{min} -> $opt{max} \n" if $i <= 0 ;
  my $data = $self->get_data ( data => $opt{data} );
  my $int = $self->sort( data => $data, field => $field );
 
  my $out =[];
  foreach my $h ( @$int ) {
    next if $h->{$field} < $opt{min};
    last if $h->{$field} > $opt{max};
    push @$out,$h;
  }
  return $out
}
sub baricenter {
  my $self = shift;
  my %opt = (
     data  => $self->{data},
     field => '',
     @_,
  );
  croak "No field selected\n" if ! $opt{field};
  my @fields = split /\,/, $opt{field};
  foreach my $f ( @fields ) {
    croak "Not a valid field\n" if ! exists $opt{data}[0]{$f};
    croak "$f is Not a numeric field\n" if ! $self->is_numeric( field => $f );
  }
  my $data = $self->get_data ( data => $opt{data} );
 
  my $b=$self->new_row();
  my $n = @$data;
  foreach my $h ( @$data ) {
    foreach my $f ( @fields ) {
      $b->{$f} += $h->{$f};
    }
  }
  foreach my $f ( @fields ) {
      $b->{$f} /= $n ;
  }
   
  return [$b]
}

sub get_row {
    my $self = shift;
    my %opt = (
       data   => $self->{data},
       @_,
    );
    my $data = $opt{data};
    delete $opt{data};
    my $row = [];
    foreach my $h ( @$data ){
      if ( $self->equal( \%opt,$h ) ) {
        return [$h]
        #push @$row ,$h;
        #last;
      }      
    } 
    return $row
}
sub select {
   my $self = shift;	
   my %opt = (
       data      => $self->{data},
       negate    => 0 ,
       compare   => 0,
       @_,
   );
   my @data = @{$opt{data}};
   my %fields = ();
   @fields{@{$self->{fields}}}=();
   my @not_fields = grep { ! exists $fields{$_} } keys %opt;
   my $not  = $opt{negate} ;
   #my $comp    =  sub { my ($l,$r) = @_; $l =~ m/ $r /ixms };
   #vms 20080122 modify comp to supply regexp in the arguments
   my $comp    =  sub { my ($l,$r) = @_; $l =~ /$r/ };
   ############
   my @keys = grep { ! /^(negate|compare|data)$/ } keys %opt;
   my %fcomp = ();
   if ( $opt{compare} ) {
      %fcomp = %{$opt{compare}};
      @keys = keys %fcomp;
   } 
   elsif (@keys) {
       @fcomp{@keys} = ($comp)x@keys; 
   }
   else{
        croak "No keys to filter !\n";
   }
   my ($comp_key,$comp);
   my $ffilter =  sub { my ($key,$hl,$hr,$not)=@_;
                     my ( $l,$r ) = ($hl->{$key},$hr->{$key});
                     return 
                     $not ? ! $fcomp{$key}->($l,$r) : $fcomp{$key}->($l,$r) 
                   };
                 
   
   #get the field and a regular expression to filter it 
   my @out;
   foreach my $k ( @keys ) {
      @sel=();
      foreach my $h ( @data ) {
         push @sel, $h if  $ffilter->($k,$h,\%opt,$not); 
      } 
      @data = @sel;
   }
   return \@data
}
sub equal {
  my $self = shift;
  my $e1 = shift;
  my $e2 = shift;
  my %opt = ( @_ ); 
  my $keys = [ keys %{$e1} ]  ;
     $keys = [ keys %opt   ] if scalar keys %opt;
  my $eq = 1;
  my %comp = (num   => sub { my ($k,$h1,$h2) = @_; $h1->{$k} == $h2->{$k} },
              alpha => sub { my ($k,$h1,$h2) = @_; $h1->{$k} eq $h2->{$k} }
     ); 
  my ($n1,$n2);
  foreach my $k ( @$keys ) {
     #$n1=0; $n2=0;
     if ( ! exists $e1->{$k} || ! exists $e2->{$k} ) {
        #print "don't exist e1 || e1 \n";
        $eq = 0;
        last;
     }
     $n1 = looks_like_number( $e1->{$k} ) ? 1 : 0;
     $n2 = looks_like_number( $e2->{$k} ) ? 1 : 0;
     $eq = $n1 && $n2 ?  $comp{num}->($k,$e1,$e2) : $comp{alpha}->($k,$e1,$e2); 
     #print "$k $n1 $n2 $e1->{$k} $e2->{$k} equal $eq\n" ;#if ! $eq;
     last if $eq == 0;
  }
  #print "key:name $n1 $n2 $e1->{name} $e2->{name} equal $eq\n" ;#if ! $eq;
  return $eq 
}

sub equal2 {
  my $self = shift;
  my $e1 = shift;
  my $e2 = shift;
  my %opt = @_; 
  my $keys = [ keys %opt ] || [ keys %$e1 ];
  
  my $eq = 1;
  
  foreach my $k ( @$keys ) {
     $eq = 0 if ! exists $e2->{$k};
     $eq = 0 if exists $e2->{$k} && $e2->{$k} ne $e1->{$k};
     #print "$k\n" if ! $eq;
     last if ! $eq;
  }
  #print $e1->{sta_to},"\n" if $eq;
  return $eq 

}
sub remove {
   my $self = shift;
   my %opt = (
      entries => '',
      @_,
   );
   my $data = $self->{data};
   my $entries = $opt{entries};
   delete $opt{entries};
   #carp "Warning: No keys to compare !" if ! %opt;
   
   #my $key = $opt{key} || ${$self->{fields}}[0];
   
   my $i=0; 
   my %rm;
   my $data_out=[];
   
   #find common elements
   foreach my $d ( @$data ) {
      foreach my $d2 ( @$entries ) {
   # print "$d->{name} $d2->{name} equal = ",$self->equal ($d,$d2, %opt )," \n"; 
         $rm{$i}++ if $self->equal ($d,$d2, %opt ); 
	# push @$data_out, $d if ! equal ($d,$d2, %opt ) ;
      }
      $i++;
   }
   #erase uncommon
   $i=0;
   foreach my $d ( @$data ) {
     next if  exists $rm{$i++};
     push @$data_out, $d ;  
   }
   
   $self->{data} = $data_out; 
   return $data_out   
}

sub get_col {
  my $self = shift;
  my %opt = ( data   => $self->{data},
              unique => '', 
              col    => '',
              @_,
            );
  my @array = ();
  my %f = ();
  my @fields = @{$self->{fields}};
  @f{@fields}=(1)x@fields;
  if ( exists $f{ $opt{col} } ) {
    @array = map { $_->{$opt{col}} } @{$opt{data}};
    if ( $opt{unique} ) {
      my %seen;
      @array = grep { ! $seen{$_}++ } @array;
    }
  }
  return @array
}
sub get_data {
   my $self = shift;
   my %opt = (
       data => $self->{data},
       @_,
   );
   my $data = [];
   my $h;
   my @fields = @{$self->{fields}};
   foreach my $d ( @{$opt{data}} ) {
      $h={};
      @{$h}{@fields} = @{$d}{@fields};
      push @$data , $h;
   }
   return $data
}

sub del_col {
   my $self = shift;
   my %opt = (
       data  => $self->{data},
       field => '',
       @_,
   );
   my $f = $opt{field};
   my $data = $opt{data};
   foreach my $d ( @$data ) {
       delete $d->{$f} if exists $d->{$f};
   }
   return $data
}

sub add_col {
   my $self =  shift;
   my %opt  = (
      data  => $self->{data},
      field => '',
      value => '',
      force => 0,
      @_,
   );
   my @fields = @{$self->{fields}};
   my %f ;
   @f{@fields}=();
   push @{$self->{fields}},$opt{field} if ! exists $f{$opt{field}} ;
   my $data = $opt{data};
   my $h;
   #print "number of rows ".scalar @$data."\n";
   #print "@fields\n";
   my $key = $opt{field};
   foreach my $d ( @$data ) {
     #print "$key exists\n" if exists $d->{$key};
     $d->{$key} = $opt{value} if ! exists $d->{$key}
                                               ||
                                          $opt{force}
                                               ||
                                          $opt{value};
     #print "$key | $d->{$key} | $d->{name}\n";
   }
   return $data
}
sub update {
  my $self = shift;
  my %opt  = (
     data => $self->{data},
     fields => $self->{fields},
     row  => '',
     prim_keys => '',
     @_,
  );
  my @pkeys = split /\,/,$opt{prim_keys};
  croak "I can't update without prim_keys => 'k1,k2,..'\n" if ! @pkeys;
  my $data = $opt{data};
  my $new_row  = $opt{row};
  foreach my $k ( keys %$new_row ) {
     my $e = trim($new_row->{$k});
     #print STDERR "$k $e\n";
     delete $new_row->{$k} if ! $e;
  }
  my %row =();
  @row{@pkeys} = @{$new_row}{@pkeys};
  my $current_row  =  $self->get_row(data => $data, %row );
  if ( ! @$current_row ) {
     my $fields = join ',',@{$opt{fields}};
     $current_row = [ $self->add_row( fields => $fields ) ] ;
  }
  my @k = keys %$new_row;
  @{$current_row->[0]}{@k} = @{$new_row}{@k};
}
sub add_row {
   my $self =  shift;
   my %opt  = (
      data  => '',
      @_,
   );
   my $data = $opt{data} || $self->{data};
   #delete $opt{data};
   my $row = $self->new_row( %opt );
   
   push @$data,$row if $row ;
   return $row 
}

sub new_row {
   my $self =  shift;
   my %opt  = (
      fields => '',
      @_,
   );
   my $row = {};
   my $fields = [ split(/\,/,$opt{fields}) ] || $self->{fields};
   delete $opt{fields};
   
   @$row{ @$fields } = ('')x(@$fields);
  
   #if ( scalar keys %opt ) { 
   if ( %opt ) { 
      @$row{ keys %opt } = values %opt ;
   }
   
   return $row   
}


sub append {
    my $self = shift;
    my %opt = (
       datatable => '',
       data  => '',
       @_,
    );
    my $data = '';
    $data = $opt{datatable}{data} if $opt{datatable};
    $data = $opt{data} if $opt{data};
    push @{$self->{data}},@$data if $data;
}

sub apply_fmt {
  my $self =  shift;
  my %opt = (
      data     => '', 
      lfirst   => '', # field1,field2,...,fieldn
      ufirst   => '', # field1,field2,...,fieldn
      ucase    => '', # same
      lcase    => '', # same 
      cfmt     => '', # anonymous hash reference with
                      # multiple key values : field_name => field_printf_format 
                      # {field1 => "%d",fiel2 => "%s",... }
      trim     => '', # field1,field2,...,fieldn
      sequence => '', # same
      @_,
        );
   my @sequence = grep !/(data|sequence)/, sort keys %opt;
   @sequence = split /,/,$opt{sequence} if $opt{sequence};
   my $data = $self->{data};
   $data = $opt{data} if $opt{data};
   delete @opt{qw/data sequence/};
   my %map_func = ( lfirst =>  sub { lcfirst( $_[1])        },
                    ufirst =>  sub { ucfirst( $_[1])        },
                    lcase  =>  sub { lc(  $_[1]  )          },
                    ucase  =>  sub { uc(  $_[1]  )          },
                    trim   =>  sub { trim($_[1]  )          },
                    cfmt   =>  sub { sprintf($_[0],$_[1]) },
                  );
   my $pf = $opt{cfmt};
   $opt{cfmt} = join(',',keys %{$pf});
   my @fields;
   foreach my $s ( @sequence ) {
      @fields = split /,/,$opt{$s};
      next if ! @fields;
      foreach my $h ( @$data ) {
        foreach my $f ( @fields ) {
          $h->{$f} =  $map_func{$s}->( $pf->{$f},$h->{$f} ) ; 
        }
      } 
   }
  
}

sub showfields {
  my $self = shift;
  print join ("\n",@{$self->{fields}}),"\n";
}

sub list {
   my $self = shift;
   my %opt = (
     fields => '',
     @_,
   );
   
   return @{$self->dump_array( fields => $opt{fields} )}
   
}


sub swap {
   my $a = shift;
   my $b = shift;

   my $tmp = $a;
   $a=$b;
   $b=$tmp;

   return($a,$b)
	
};
 

1;

__END__

=head1 NAME

DataTable  -  Create and manipulate formatted or unformatted data tables.
             

=head1 SYNOPSYS

use GPS::DataTable;

#create an empty table
my $table = GPS::DataTable->new( );
  
=cut


