package GPS::FTP::Names;
use Exporter;
use Carp;
use GPS::DATES;

@ISA= ('Exporter');

@EXPORT = qw( brdc_name 
              rnx_name
              orbit_file_names
             ) ;

our %ftemplate = ( 
           "jpl_final" => [ "orbits/{cal}.shad",  
                            "orbits/{cal}_nf.eci",
                            "orbits/{cal}tpeo_nf.nml", 
                            "clocks/{cal}_nf.tdpc",
			    "orbits/{cal}_nf.wlpb"
                            #"orbits/{cal}.frame"
                          ],
           "jpl_flinn" => [   "orbits/{cal}.shad",  
                              "orbits/{cal}.eci",
                              "orbits/{cal}tpeo.nml", 
                              "clocks/{cal}.tdpc",
			      "orbits/{cal}.wlpb"
                              #"orbits/{cal}.frame"
                          ],

           "jpl_rapid" => [ "{cal}.shadow_events", 
                            "{cal}.eci",
                            "{cal}.TPNML", 
                            "{cal}.gps_clocks", 
                            "{cal}.yaw_rates" 
                          ],
		   "jpl_xfile" => [ "{frame}/{retro}/{epoch}.{frame}.x",
                            "{cal}.IGS05_nf.qx",
                            "{cal}.IGS00_nf.qx",
                            "{cal}.JPL00_nf.qx",
                            "{cal}.JPL05_nf.qx",
                            #"{epoch}.itrf00.x"
                          ],
		   #"igs_final" => ["igs{wwww}{d}.sp3c","igs{wwww}{d}.sp3",
		   "igs_final" => [ "igs{wwww}{d}.sp3",
	          	            "igs{wwww}{d}.clk",
                            "igs{wwww}{d}.cls",
	            		    "igs{wwww}7.erp",
                            "igs{wwww}7.sum"
                           ],
		   #"igs_rapid" => ["igr{wwww}{d}.sp3c",
		   "igs_rapid" => [ "igr{wwww}{d}.sp3",
			                "igr{wwww}{d}.clk",
                            "igr{wwww}{d}.cls",
				            "igr{wwww}{d}.erp",
                            "igr{wwww}{d}.sum"
                          ],
		   "igs_ultra" => [ #"igu{wwww}{d}_{hh}.sp3c",
                            "igu{wwww}{d}_{hh}.sp3",
                            "igu{wwww}{d}_{hh}.erp",
                            "igu{wwww}{d}_{hh}.sum",
                             #"igu{wwww}{d}_{hh}_cmp.sum"
                          ],
		);

$ftemplate{final_fid}=$ftemplate{jpl_flinn};
$ftemplate{igs_flinn}=$ftemplate{jpl_flinn};
		
my %AC = ( jpl   => 'jpl',
           igs   => 'igs',
          );
my @types = qw/rapid ultra final flinn final_fid/;
my %ORB_TYPE =();
   @ORB_TYPE{@types}=@types;
   $ORB_TYPE{final_fid} = 'flinn';
my %ZIP_EXT = ();
   @ZIP_EXT{qw/gz Z bz2 zip/} = qw/gz Z bz2 zip/;
my %ULTRA_HH = ();
   @ULTRA_HH{qw/00 06 12 18/}= qw/00 06 12 18/;

sub orbit_file_names {
    my %opt = ( ac     => 'jpl',
                type   => 'rapid',
                frame  => 'itrf00', 
                date   => '',
                hh     => '',
                zip    => '',
                @_,
    );
    
    my $z = $opt{zip};  
       $z =~ s/^\.+// if $z;

    if ( $z && ! exists  $ZIP_EXT{$z} ) {
      croak "Invalid extension for compressed file $opt{zip}\n";
    }
    
    my $ac   = lc $opt{ac};
    if (  ! exists $AC{$ac} ) {
          croak "Invalid ac argument.  igs|jpl|flinn != $opt{ac}\n";
    }
    
    my $type = lc $opt{type};
    if ( ! exists $ORB_TYPE{$type} ) {
          croak "Invalid type argument. ".join('|',@types)."!=$opt{type}\n";
    } 
    my $date = $opt{date};
    my ($cal,$hh) = split ' ', get_date($date,'cal,time'); 
    my $yyyy = unpack 'a4',$cal;
    my $yy   = unpack 'x2a2',$yyyy;
    
    if ( $cal !~ /^\d{4}-\d{2}-\d{2}$/ ) {
        croak "Invalid date $opt{date}\n";
    }
    
    $hh = $hh ? unpack('a2',$hh) : '00';
    
    $hh = sprintf "%02d",$opt{hh} if $opt{hh};
    
    if ( $type eq 'ultra' ) {
       if ( ! exists $ULTRA_HH{$hh} ) {
         croak "Invalid hours for ultra rapid product $hh\n";
       }
       if ( $ac ne 'igs' ) {
          croak "Invalid ac argument $opt{ac} for type 'ultra'\n";  
       }
    }
    my ($retro,$odir,$dir,$xdir,$frame); 
    
    if ( $ac eq 'jpl' ) {
      $frame = $opt{frame};
      if ( $yyyy <= 1996 && $type eq 'flinn' ) {
        croak "Invalid type $type for year $yyyy\n";
      }
      $retro = $yyyy <= 1996 && $type eq 'final' ? 'retro' : '';
      $odir=$retro;
      $dir="$odir/$yyyy";
      $dir="$odir/RapidService/orbits/" if $type eq "rapid"; 
      $xdir = 'x-files';
    }

    my $epoch= get_date($cal,'epoch');
    my $gpsw = get_date($cal,'gpsw');
    $gpsw = sprintf("%04d",$gpsw);
    my $gdow = get_date($cal,'gdow'); #gps day of week starts on sundays
     print "$ac $cal $hh, $epoch $gpsw $gdow\n";    
    my $xfiles = [];
    my $orbit_files = [];
    @$orbit_files = @{$ftemplate{"${ac}_$type"}};
    foreach my $f ( @$orbit_files ) {
          $f =~  s/\{cal\}/$cal/g;
          $f =~  s/\{epoch\}/$epoch/g;
          $f =~  s/\{frame\}/$frame/g;
          $f =~  s/\{wwww\}/$gpsw/g;
          $f =~  s/\{d\}/$gdow/g;
          $f =~  s/\{hh\}/$hh/g;
          
          $f = "$dir/$f"  if $ac eq 'jpl';
          $f = "$gpsw/$f" if $ac eq 'igs'; 
          $f .= ".$z"     if $z  && $f !~ /\.frame/;
          # print "$f\n";
    };
    if ( $ac eq "jpl" && $type eq "final" ) {
       @$xfiles = @{$ftemplate{"jpl_xfile"}};
       $epoch = lc $epoch;
       foreach my $x ( @$xfiles ) {
            $x =~ s/\{cal\}/$cal/g;
            $x =~ s/\{epoch\}/$epoch/g;
            $x =~ s/\{retro\}/$retro/g;
            $x =~ s/\{frame\}/$frame/g;
            $x = "$xdir/$x";
            $x .= ".$z"     if $z && $x !~ /\.x/;
            # print "$x\n";
       };
    }
    return ! wantarray ? $orbit_files : ($orbit_files,$xfiles)
}
sub brdc_name {
    my $date = shift || 'today';
    my ($yyyy,$doy) = split ' ', get_date($date,'doy');
    my $yy = unpack 'x2a2',$yyyy;
    return "brdc${doy}0.${yy}n"
}

my %RNX_TYPE = ();
   @RNX_TYPE{qw/obs nav hatanaka o n h d/}= qw/o n d o n d d/;

sub rnx_name {
   my %opt = ( site   => '',
               date   => 'today',
               type   => 'obs',
               seq_n  =>  '0', #daily
               case   => 'l',
               zip    => '',
               @_,
   );
   $opt{case}=unpack 'a1',lc $opt{case};
   
   my $site  = $opt{case} eq 'l' ? lc $opt{site} : uc $opt{site};
   
   if ( ! $site || $site !~ /^\w{4}$/ ) {
      croak "Invalid site name $opt{site}\n";
   }
   
   my $date = $opt{date};
   my ($yyyy,$doy) = split ' ', get_date($date,'doy'); 
   if ( $yyyy !~ /\d{4}/ && $doy !~ /\d{3}/ ) {
        croak "Invalid date $opt{date}\n";
   }
   my $yy = unpack 'x2a2',$yyyy;
   
   my $t = lc $opt{type};

   if ( ! exists $RNX_TYPE{$t} ) {
       croak "Invalid rinex type $opt{type}\n";
   }
   $t = $RNX_TYPE{$t};
   
   my $z = $opt{zip};  $z =~ s/^\.//;

   if ( $z && ! exists  $ZIP_EXT{$z} ) {
      croak "Invalid extension for compressed file $opt{zip}\n";
   }
   
   my $s = lc $opt{seq_n};
   if ( $s !~ /^[0a-x]$/ ) {
      croak "Invalid sequence number $opt{seq_n}. valid: 0, [a-x];\n";
   }
   my $rnx = "$site${doy}$s.${yy}$t";
  
  $rnx .= ".$z" if $z;
  
  return $rnx
}

1;
__DATA__


=head1 NAME

GPS::FTP::Names - Make file names used in GPS processing. 

=head1 SYNOPSYS

use GPS::FTP::Names;

my ($fnames,$xfnames) = orbit_file_names( ac    => 'jpl',
                                          date => 'today',
                                          type  => 'rapid'
                                         );
print "@$fnames @$xfnames\n";

my ($fnames) = orbit_file_names( ac    => 'igs',
                                 date  => 'today',
                                 type  => 'ultra'
                                 hh    => 6
                                );
                                
   ($fnames) = orbit_file_names( ac    => 'igs',
                                 date  => 'today',
                                 type  => 'ultra'
                                 hh    => 6
                               );
print "@$fnames\n";



=head1 DESCRIPTION

GPS::FTP::Names provides functions to make the file names suitable for
downloading files on a specific date.

=head1 METHODS

=over

=item B<orbit_file_names>(%hash)



=over 1

=head2 INPUT

Hash with the  keys: ac, date, type [hh, zip] 

ac => 'jpl'    [default]

ac => 'igs' 

See GPS::DATES for more options.

$date_str = 'yyyy-mm-dd' or 'wwwwd' or 'yyyy,doy' 

date => $date_str 

type => $type  [where $type is one of rapid,final,flinn,final_fid,ultra]

hh   => $ultra_rapid_hh  [ $ultra_rapid_hh is one of 00,06,12,18 ]

hh can be given inside $date as 'yyyy-dd-mm hh:mm:dd'

zip => $zip_ext  [ default = '' , valid: gz,Z,zip,bz2 ]

=head2 OUTPUT

the output are 2 array references :

$orbit_file_names : points to an array of strings with the file names

$xfile_names      : same but with the name of the JPL xfiles.

=back




=item B<brdc_name>($date)

=over 1

=head2 INPUT

$date is a string: 'yyyy-mm-dd' || 'wwwwd' || 'yyyy,doy'

See GPS::DATES for more options.

$date = 'yyyy-mm-dd' or 'wwwwd' or 'yyyy,doy'

=head2 OUTPUT

Returns a string : B<brdc{doy}0.{yy}n>

=back

=head1 VERSION

0.1

=head1 SEE ALSO

GPS::FTP::Orbits, GPS::DATES


=head1 AUTHOR

Victor Marcelo Santillan E<lt>marcelo@geology.cwu.eduE<gt>

=cut











