package GPS::FTP::Orbits;
use Exporter;
use Carp;
use GPS::DATES;
use GPS::FTP::Names;
use GPS::IPNames;
use GPS::IGS;
use GPS::JPL;
use Net::FTP;
use File::Basename;

@ISA= ('Exporter');

@EXPORT = qw( igs_fetch jpl_fetch fetch_orbits ) ;
my %ip = get_ftp_ip();
my %ACENTER = (  "jpl"   => { host => "sideshow.jpl.nasa.gov",
                               dir  => "/pub/gipsy_products"
                              },
                 "igscb" =>   { host => "igscb.jpl.nasa.gov",
                                dir  => "/pub/product"
                              },
#                "igs_cddis" =>   { host => "cddis.gsfc.nasa.gov",
#                                dir  => "/pub/gps/products"
#                              },
                 "igs_sopac" =>   { host =>  "garner.ucsd.edu",
                                dir  =>  "/pub/products"
                              },
              );

sub igs_fetch {
   my %opt = (  zip      => 'Z',
                force    => 0,
                flinn    => 0,
                latest   => 0,
                type     => 'final',
                ac       => 'igs',
                @_,
              );
              
   $opt{fnames} = igs_fnames(%opt);
  
   return fetch_orbits(%opt)

}

sub jpl_fetch {
   my %opt = (  zip      => 'Z',
                force    => 0,
                type     => 'final',
                date     => '',
                ac       => 'jpl',
                @_,
              );
              
   $opt{fnames} = jpl_fnames(%opt);
   # I can't use return value because I am not sure which Xfile I will get
   fetch_orbits(%opt); 
   my @to_download = grep ! /x/ , @{$opt{fnames}};
   my ($x) =  map { glob($_) } qw/*.x *.qx *.x.Z *.qx.Z /;
   #my ($x) = glob "*.x *.qx.Z"; chomp $x;
   push @to_download,$x if $opt{type} eq 'final';
   my $ok = 0;
   foreach my $f ( @to_download ) {
      next if ! $f;
      $ok++ if -e basename($f);
   }
   $ok = $ok == @to_download ;
   return $ok ? $ok : 0;
}

sub fetch_orbits {
   my %opt = (  ftp_passwd => "$ENV{USER}\@geology.cwu.edu",
                force => 0,
                fnames => 0,
                arch   => '',
                ac     => '',
                @_,
              );
   my $fnames = ref $opt{fnames} eq 'ARRAY' ? $opt{fnames} : 0;
   if ( ! $fnames ) {
      croak "Invalid fname => $opt{fname} \n".
            "Use igs_fnames(\%options) or jpl_fnames(\%options)\n";
   }           
   my $ok = 0;
   open ( LOG , "> fetch_orbits.log") or croak "$!\n";
   
   foreach my $ac ( sort grep { /$opt{ac}/ } keys %ACENTER ) {
      #next if $opt{arch} && $opt{arch} ne $ac;
      my $host = $ACENTER{$ac}{host};
      my $dir  = $ACENTER{$ac}{dir};
      print LOG "Connecting to $ac host:$host\n";
      my $ftp = Net::FTP->new ( $ip{$host},
                                Timeout => 60,
                                Debug   => 0,
                                Passive => 1
                               )
                 or print LOG "Can't connect to  $host $@\n";
      next if ! $ftp; 

      $ftp -> login ( "anonymous" , $opt{ftp_passwd} )
              or print LOG  "Couldn't login into $host\n";
      $ftp -> binary ()
              or print LOG  "Couldn't set type to binary in $host\n";
      $ftp -> cwd ( $dir )
                 or print LOG  "Couldn't change to dir $dir\n";
       
      $ok = 0;
      foreach my $f ( @$fnames ) {
          my $local_file = basename $f;
          print LOG "Getting $f : ";
          if ( -e $local_file && ! $opt{force} ) {
             $ok++;
             print LOG " got it\n";
             next;
          }

          $local_file = $ftp -> get ( "$dir/$f" );
          
          if ( defined $local_file ) {
             $ok ++;
             print LOG " got it\n";
          }
          else {
             print LOG "Couldn't get $f \n";
         }
      }
     
      $ftp -> quit()
              or print LOG "Couldn't quit from $host\n";
     
      $ok = $ok == @$fnames ;
      
      last if $ok;
   }
   
   close LOG; 
   
   return $ok
}

1;
