package GPS::FTP::Orbits;

use Exporter;
use Carp;
use File::Basename;
use File::Copy;
use Net::FTP;
use GPS::Defaults;
use GPS::DATES;
use GPS::PRN_GPS;
use Cwd;

@ISA= ('Exporter');

@EXPORT = qw( get_rnx get_brdc get_ba_dates 
              get_fnames fetch_orbit ftp_orbits 
              get_nets get_info 
              get_sources make_get_info ) ;

our %par = get_defaults();
our $username= $ENV{USER};
our $thishost='geology.cwu.edu';
our $password=$username . '@' . $thishost;
#our $ftpsite = 'sideshow.jpl.nasa.gov';
#our %ftpsites = ( "jpl" -> "sideshow.jpl.nasa.gov", 
#		  "igsjpl" -> "igscb.jpl.nasa.gov",
#		);


our %dctype = (  "jpl" => ["jpl"],
                 "igs" => ["igscb","sopac","cddis"]
              );
	       	
our %hosts = ("jpl"   =>   ["sideshow.jpl.nasa.gov","/pub/gipsy_products"], 
              "igscb" =>   ["igscb.jpl.nasa.gov","/pub/product"],
              "cddis" =>   ["cddis.gsfc.nasa.gov","/pub/gps/products"],
	      "sopac" =>   ["garner.ucsd.edu","/pub/products"]
	     );
# Note igs_ultra is not implemented yet
our $nf = '_nf';
#$nf='';
our %ftemplate = ( 
           "jpl_final" => [ "{cal}.shad",  
                            "{cal}$nf.eci",
	                    "{cal}tpeo$nf.nml", 
                            "{cal}$nf.tdpc" 
                          ],
           "jpl_final_fid" => [ "{cal}.shad",  
                                "{cal}.eci",
	                       "{cal}tpeo.nml", 
                                "{cal}.tdpc" 
                              ],

                  "jpl_rapid" => ["{cal}.shadow_events", 
                                  "{cal}.eci",
                                  "{cal}.TPNML", 
                                  "{cal}.gps_clocks", 
                                  "{cal}.yaw_rates" 
                                  ],
		   "jpl_xfile" => ["{cal}.IGS00_nf.qx",
                                   "{cal}.JPL00_nf.qx",
                                   "{epoch}.itrf00.x"
                                  ],
		   #"igs_final" => ["igs{wwww}{d}.sp3c","igs{wwww}{d}.sp3",
		   "igs_final" => ["igs{wwww}{d}.sp3",
	          	           "igs{wwww}{d}.clk",
                                    "igs{wwww}{d}.cls",
				    "igs{wwww}7.erp",
                                    "igs{wwww}7.sum"
                                   ],
		   #"igs_rapid" => ["igr{wwww}{d}.sp3c",
		   "igs_rapid" => ["igr{wwww}{d}.sp3",
			           "igr{wwww}{d}.clk",
                                   "igr{wwww}{d}.cls",
				   "igr{wwww}{d}.erp",
                                   "igr{wwww}{d}.sum"
                                  ],
		   "igs_ultra" => [#"igu{wwww}{d}_{hh}.sp3c",
                                   "igu{wwww}{d}_{hh}.sp3",
				   "igu{wwww}{d}_{hh}.erp",
                                   "igu{wwww}{d}_{hh}.sum",
				   #"igu{wwww}{d}_{hh}_cmp.sum"
                                  ]
		);
		


our @orbit_files = ();
sub get_fnames {
    my $ag = lc shift;
    my $cal = shift;
    my $type= shift;
    my $hh  = shift ||'00';
    our $epoch= cal2epoch($cal);
    our $jd   = cal2jd($cal);
    our $gpsw = jd2gpsw($jd);	
    our $gdow = dow2gdow(jd2dow ($jd)); #gps day of week starts on sundays
                                        #jd2dow starts on monday	
    my $xfiles = [];
    my $orbit_files = [];
    @$orbit_files = @{$ftemplate{"${ag}_$type"}};
    foreach my $f ( @$orbit_files ) {
          $f =~  s/\{cal\}/$cal/g;
          $f =~  s/\{epoch\}/$epoch/g;
          $f =~  s/\{wwww\}/$gpsw/g;
          $f =~  s/\{d\}/$gdow/g;
          $f =~  s/\{hh\}/$hh/g;
          if ( $ag eq 'igs' ) {
             $f = "$gpsw/$f"; 
          } 
    };
    if ( $ag eq "jpl" && $type eq "final" ) {
       $xfiles = $ftemplate{"${ag}_xfile"};
       my $date = lc($epoch);
       foreach ( @$xfiles ) {
            s/\{cal\}/$cal/g;
            s/\{epoch\}/$date/g;
       };
    }
    return($orbit_files,$xfiles)
}
sub fetch_orbit {
        my $date = shift;
	my $agency = shift;
	my $type = shift || "final";
	my $ftp  = shift || 0;
	my $dir = shift  ||'';
	my @missing = ();
	#our @orbit_files = ();
	my $here = cwd(); 
	my $xdir = $par{X_FILES_DIR};
	my @TYPES=();
	my @files=();
	my $xfile='';
	my $getx=1;
	my $ok=0;
	my ($year,$mm,$dd,$hh) = split ' ',get_date($date,'cal,time,sp');
        $cal ="$year-$mm-$dd";	
        my $gpsw = get_date($date,'gpsw');	
        my $gdow = get_date($date,'gdow');	
	#my $orbit_names = qw( Shadow peci tpeo.nml TDPfile );
	
	open ( LOG , "> ${cal}.ftp_orbits.log");
	#if ( $type eq "any" ) {
	#	 push(@TYPES,("final","rapid"));
	#} else { push( @TYPES,$type) };
       
       	my ($orbit_files,$xf)=get_fnames($agency,$cal,$type,$hh);	
        #print "@$orbit_files\n"; 
	if ( $agency eq "jpl" ) {	  
		if ( $type eq "rapid" ) {
		  $dir = $par{JPL_RAPID_ORBITS_DIR} if ! $dir; 
		} else 	{
		  $dir = "$par{JPL_ORBITS_DIR}/$year" if ! $dir; 
		};
        }elsif ($agency eq "igs"){
	        #$dir =  "$par{IGS_ORBITS_DIR}/$gpsw" if ! $dir;
	        $dir =  "$par{IGS_ORBITS_DIR}"; # if ! $dir;
	        mkdir ($dir) if ! -d $dir;	
                my $bd = get_date($cal,'cal',-1); 
                
                my $ad = get_date($cal,'cal', 1);
                my $btype = $type;
                my $atype = $type;
                #print "$bd $ad\n";
                if ( $ad 
                if ( $ad eq get_date('today','cal') ) {
                   $atype = 'ultra';
                   $hh = '00';
                }
                my ($bfiles) = get_fnames($agency,$bd,$btype,$hh); 
                my ($afiles) = get_fnames($agency,$ad,$atype,$hh); 
                push @$orbit_files,@$bfiles;
                push @$orbit_files,@$afiles;
                #print "@$orbit_files\n";
	}
    # print STDERR "orbit dir $dir\n";
	#check local dir for orbits and xfiles
	foreach my $f ( @$orbit_files  ) {
           #print "$ftp $f $dir/$f.Z\n";
           if ( ! -e "$dir/$f.Z" ) { 
             $ftp = 1; 
           } else {
              copy("$dir/$f.Z",$here);
	      push ( @files,"$here/$f.Z" );
	      print LOG "founded $dir/$f.Z\n"; 
           } 
	}
	if ( @$xf ) {
           foreach my $f ( @$xf ) {
              $ftp=0;
              if ( -e "$xdir/$f" ) {
                copy("$xdir/$f",$here); 
		$xfile="$here/$f";
		print LOG "$f founded \n";
		last;
              } 
              else { 
                $ftp=1;
		print LOG "$f not founded \n";
              } 
           }
	 }
     	
         if ( $ftp ) {
            my $ok=0;
            ($ok,$xfile,@files)=ftp_orbits($cal,$agency,$type,$orbit_files,$xf);
            if ( $ok ) {
               print LOG "$type orbits succesfully retrieved \n";
               #copy to orbits dir
               my $gw = '';
               foreach my $f ( @files ) { 
                 $gw= $agency eq 'igs' ? '/'.unpack ('x3a4',basename($f)) : '';
                 `mkdir -p $dir/$gw` if ! -d "$dir/$gw";
                 copy($f,$dir.$gw) or croak "couldn't copy all of $_ :$!\n";
               }
               if ( -e $xfile ) { 
                 #print LOG "copy xfile\n";
                 copy( $xfile, $xdir ) or croak "couldn't copy all of $_ :$!\n";
                 system ( "gzip -df $xdir/$xfile 2> /dev/null" );
               }
           }
           else {  
              print LOG  "FTP did not get all the $agency $type orbits\n ";  
           }
	}
	@files=unpack_files($cal,$agency,$type,$xfile,@files);
        #print "files @files\n";
	push ( @files, $xfile ) if @$xf;
	push ( @$orbit_files, $xfile ) if @$xf;
	unlink(@files) == @files || print LOG  
                                    "couldn't unlink all of:\n @files\n: $!\n";
	$ok =  @$orbit_files != @files ? 0: 1;
	print LOG "ok: $ok\n";
	close(LOG);
	return $ok 
	
}
sub unpack_files {
  my $cal  = shift;
  my $ag   = shift;
  my $type = shift; 
  my $xfile = shift;
  my @orbit_files = @_;
  my @files=();
	#open ( LOG , ">> ${cal}.ftp_orbits.log");
	#print LOG "orbit files:\n";
	#print LOG (map { basename($_)."\n" } @orbit_files) if  @orbit_files;
	
  if ( $ag eq "jpl" ) {
    print LOG "xfile = $xfile\n" ;
    if ( $type eq "rapid" ) {
      foreach my $f ( @orbit_files ) {
         if ( $f =~ /shadow_events/ ) { system ( "zcat $f > Shadow" ) };
         if ( $f =~ /gps_clocks/ ) { system ( "zcat $f > TDPtmp" ) };
         if ( $f =~ /yaw_rates/ ) { system ( "zcat $f > TDPyaw" ) };
         if ( $f =~ /TPNML/ ) { system ( "zcat $f > tpeo.nml" ) };
         if ( $f =~ /eci/ ) { system ( "zcat $f > peci" ) };
      }
      system("
       yaw_rates2tdp TDPyaw |tv2c - >> TDPtmp
       sort -T /tmp -n TDPtmp > TDPfile
       rm -f TDPtmp TDPyaw
     ");
    } 
    else {
     foreach my $f ( @orbit_files ) {
        if ( $f =~ /shad/ ) { system ( "zcat $f > Shadow"   ); }
        if ( $f =~ /tdpc/ ) { system ( "zcat $f > TDPfile"  ); }
        if ( $f =~ /tpeo/ ) { system ( "zcat $f > tpeo.nml" ); }
        if ( $f =~ /eci/ )  { system ( "zcat $f > peci"     ); }
     }
    }
    if ( $xfile ) { 
	if ( $xfile =~ /\.Z$/ ) {
	      system ( "gzip -dfc $xfile > XFILE 2> /dev/null" );
	}
        else { 
           copy($xfile,"XFILE"); 
        }
			#push ( @orbit_files,$xfile );
    }
    @files = @orbit_files;
 }
 elsif ( $ag eq "igs" ) {
    my $clock_file;
    #if ( $type eq "final" ) {
    #`igs_orbits_clks_fetch.pl -b $bdate -e $adate -type $type`;
    #`igs2flinn.pl -b $cal -e $cal -type $type`;
    #print "@$of\n"; 
    my ($bdate,$adate) = get_ba_dates($cal);
    system("gzip -df *.Z 2> /dev/null");
    clk2tdp($cal,glob("ig*.clk"));
    system("
           tv2c.pl TDPfile.$$  > TDPfile 
           tdp_gps_phase_ctr.pl -igs -o tdp_gps_pc
           cat tdp_gps_pc >> TDPfile
           \\rm -f TDPfile.$$
    ");
     @files = (); 
     foreach my $f ( @orbit_files ) {
          $f =~ s/\.Z//;
          push @files,basename($f);
     }
     #print "@files\n";
     $ENV{TPNML} = "tpeo.nml";
     system("tp_nml $bdate $adate 1> tpeo.nml 2> /dev/null");	       
     my $EpochEO = eo_epoch($cal);
     eop($EpochEO);     
     # gps_shadow oi.nio qmfile > Shadow
     # eci2oi -ieci peci -tp tpeo.nml -start $bdate -stop $adate  
     system("cat <<EOT > Shadow
                                      
NUMBER OF GPS SATELLITES IN ECLIPSE: 0
EOT
");
    system("spx2eci *.sp3 2> /dev/null");
    filter_ecis($cal);
    system("
      cat gps*.eci > peci
      \\rm -f gps*.eci ig[rsu]* 2> /dev/null
    ");
    
   } # end ag eq igs
   return @files
}
sub eop {
  my $EpochEO = shift;
  open(my $fh, ">> tpeo.nml") or die "Couldn't open tpeo.nml for writing $!\n";
  my $eo ="\n";
  $eo .= '$earth_orientation'."\n";
  $eo .= qq(
 !        Received from Jim Ray on 10/18/2000
 !

    EpochEO = \"$EpochEO\"
    Extended_UT1_model= .true.
    Extended_XPM_model= .true.
    Extended_YPM_model= .true.
! Ext. UT1 parameter nominals (units = seconds of time)    ->     IERS
  UMD0 =  -7.20e-6,  16.17e-6,    ! U M2 COS ,  U M2 SIN        COS, -SIN
          -0.04e-6,   7.59e-6,    ! U S2 COS ,  U S2 SIN        COS, -SIN
          -1.54e-6,   3.80e-6,    ! U N2 COS ,  U N2 SIN        COS, -SIN
          -0.38e-6,   1.96e-6,    ! U K2 COS ,  U K2 SIN        COS, -SIN
           8.64e-6,  17.71e-6,    ! U K1 COS ,  U K1 SIN        SIN,  COS
         -12.10e-6, -16.05e-6,    ! U O1 COS ,  U O1 SIN       -SIN, -COS
          -2.86e-6,  -5.16e-6,    ! U P1 COS ,  U P1 SIN       -SIN, -COS
          -2.45e-6,  -5.03e-6     ! U Q1 COS ,  U Q1 SIN       -SIN, -COS
! Ext. XPM parameter nominals (units = radians)
  XMD0 = -.1357e-09,     1.6000e-09,      ! X M2 COS ,  X M2 SIN
          .3103e-09,      .7030e-09,      ! X S2 COS ,  X S2 SIN
         -.0630e-09,      .2763e-09,      ! X N2 COS ,  X N2 SIN
          .0824e-09,      .1745e-09,      ! X K2 COS ,  X K2 SIN
         -.7369e-09,      .3782e-09,      ! X K1 COS ,  X K1 SIN
          .6448e-09,     -.2376e-09,      ! X O1 COS ,  X O1 SIN
          .2424e-09,     -.1212e-09,      ! X P1 COS ,  X P1 SIN
          .1261e-09,     -.0291e-09       ! X Q1 COS ,  X Q1 SIN
! Ext. YPM parameter nominals (units = radians)
  YMD0 =  .9502e-09,     -.1794e-09,      ! Y M2 COS ,  Y M2 SIN
          .4218e-09,     -.2860e-09,      ! Y S2 COS ,  Y S2 SIN
          .1600e-09,     -.0533e-09,      ! Y N2 COS ,  Y N2 SIN
          .1067e-09,     -.0873e-09,      ! Y K2 COS ,  Y K2 SIN
         -.3782e-09,     -.7369e-09,      ! Y K1 COS ,  Y K1 SIN
          .2376e-09,      .6448e-09,      ! Y O1 COS ,  Y O1 SIN
          .1212e-09,      .2424e-09,      ! Y P1 COS ,  Y P1 SIN
          .0291e-09,      .1261e-09       ! Y Q1 COS ,  Y Q1 SIN

);
  $eo .= '$end'."\n";


 print $fh $eo;  

}
sub eo_epoch {
  my $cal = shift;
  my $start = get_date($cal,'secj2000',-0.125);
  my $end   = get_date($cal,'secj2000',1.125);
  my $eo = ($start+$end)/2;
  $eo = `gps2utc $eo`; chomp $eo; 
  my $d = sprintf "%4d-%02d-%02d:%02d:%02d:%7.5f",sec2cal($eo );
  
  return get_date($d,'eop')
    
  
}
sub filter_ecis {
    my $cal = shift;
    my $start = get_date($cal,'secj2000',-0.125);
    my $end   = get_date($cal,'secj2000',1.125);
    #my $start   = shift;        # GPS sec past J2000 take all records >=
    #my $end     = shift;        # GPS sec past J2000 take all records <=

    my $tmp_eci = "eci$$";
    unlink $tmp_eci if(-e $tmp_eci);

    #my $st_utc = `gps2utc $start`; chomp $st_utc;
    #my $end_utc = `gps2utc $end`; chomp $end_utc;
    my $st_utc =  gps2utc( $start);
    my $end_utc = gps2utc($end);
    
    foreach my $g ( glob "gps*.eci" ){
        open( T, ">$tmp_eci");
        open( G, "<$g" ) or die "Unable to open $g";
        while(<G>){
            my @l = split(' ');
            # fix GPS32 for now
            next if $l[0] == 32 || $l[0] eq '00';
            next if $l[0] eq '00';
            my $t = cal2sec(@l[1..6]);
            print T $_ if( $t >= $st_utc && $t <= $end_utc );
        }
        close T;
        close G;
        rename $tmp_eci, $g or die "Unable to rename $tmp_eci to $g ";
    }
}

sub clk2tdp {
    my $cal = shift;
    my @clk_files = @_;
    my $start = get_date($cal,'secj2000',-0.125);
    my $end   = get_date($cal,'secj2000',1.125);
    my $sl = 299792.458;
    my $pg = GPS::PRN_GPS->new();
    open(my $th , "> TDPfile.$$") or croak "Couldn't open TDPfile $!\n";    
    foreach my $clk ( @clk_files ) { 
      $clk =~ s/\.Z//;
      open(my $ch , "< $clk") or croak "Couldn't open clock file $clk $!\n";
      while ( <$ch> ) {
        next if ! /^AS G(\d\d)/ ;
        my $prn = $1;
        $prn =~ s/^0//;
        my @l = split ' ',$_;
        my $clk_bias = $l[9]*$sl;
        my $sig      = $l[10]*$sl;
        my $secd = cal2sec(@l[2..7]);
        next unless( $secd >= $start && $secd <= $end );
        my $gps = $pg->gps( prn =>$prn , secj2000 => $secd);
        my $name = sprintf "GPS%02d",$gps;
        printf $th  "%15.4f %17.10E  %14.9f    %10.3E SAT BIAS%s\n",
                    $secd,0,$clk_bias,$sig,$name;
     }
     
     close($ch);
   }
   
}
sub get_ba_dates{
    my $cal = shift;
    my $jd = cal2jd($cal);
    my ($by,$bm,$bd) = jd2cal($jd - 10);
    my ($ay,$am,$ad) = jd2cal($jd + 10);
    my $bmm = m2month($bm); 
    my $amm = m2month($am); 
    #my $b = sprintf("%02d-%3s-%4d",$bd,$bmm,$by);
    #my $a = sprintf("%02d-%3s-%4d",$ad,$amm,$ay);
    my $b = sprintf("%02d-%3s-%4d",$bd,$bmm,$by);
    my $a = sprintf("%02d-%3s-%4d",$ad,$amm,$ay);
    return ( $b,$a)
    #return ( jd2cal($jd-10) , jd2cal($jd +10 ) )
}
sub ftp_orbits {
	my $cal = shift;     #yyyy-mm-dd 
	my $agency = shift;  # jpl or igs
	my $type = shift;    # rapid or final
	my $orb=shift;       #
	my $xf = shift;
	
    	my @files=();
	my $retro='';
	my $epoch=cal2epoch($cal);	
        my $jd   = cal2jd($cal);
        my $gpsw = jd2gpsw($jd);	
	my $ok = 1;
	my $here=cwd();
	my $getx=1;
	my $xfile='';
        my ($odir,$dir,$xdir);
          	
        my ($y,$m,$d) = split(/-/,$cal);
	#open ( LOG , ">> ${cal}.ftp_orbits.log");
	#select LOG;

	my $data_centers = $dctype{$agency};
    foreach my $dc ( @$data_centers ) {

      my ($host,$hdir) = @{$hosts{$dc}}; 
     	
	  if ( $agency eq "jpl" ) { 
	      $retro="retro" if $y <= 1996;
          $odir="$hdir/$retro";
          $dir="$odir/$y/orbits";
	      $dir="$odir/RapidService/orbits/" if $type eq "rapid";
	       $xdir ="$hdir/x-files/";
        }
        elsif ( $agency eq "igs" ) {
             $dir = $hdir;
             #$dir = "$hdir/$gpsw";
        }	
          $ok=1; 
	  my $ftp = Net::FTP->new( "$host",
					  Timeout => 60,
					  Debug => 0,
					  Passive => 1 )
		or  print LOG ( "Can't connect to host $host : $@\n" );
          next if ! $ftp;
	  $ftp -> login ( "anonymous", $password ) 
		   or print LOG ( "Couldn't login into $host\n");
	  $ftp -> binary ()
	 	   or print LOG  ( "Couldn't set type to binary in $host\n");
	  $ftp -> cwd ( "$dir" )
		   or print LOG ( "Couldn't change to dir $dir\n");
	
	  foreach my $of ( @$orb ) {
            next if -e "$of.Z";
            if ( $of =~ /\_nf\.tdpc$/ ) {      
                 $ftp -> get ( "$odir/$y/clocks/$of.Z")
	                   or print LOG "Couldn't get $of.Z\n";
            }
            else {	       	
               #print "getting $dir/$of.Z from $host\n";
               $ftp -> get ( "$dir/$of.Z" )
			   or print LOG "Couldn't get $of.Z\n";
			   #or print  "Couldn't get $dir/$of.Z\n";
	    }
	  }
	  #get xfile	
	  if ( @$xf ) {
	   # I could list and grep the name,but it is too slow, 
           # it is faster to add a new name
	   foreach my $f ( @xf ) {
	      $f = "itrf00/$retro/$f" if $f =~ /\.itrf00\.x/;   
		  $ftp -> get ( "$xdir/$f.Z" ) 
			or print LOG "Couldn't get $xf\n";		
		  if ( -e basename($xf) ) {
		     $xfile=basename($xf);
			 print LOG "Got $xfile\n";
			 last;
		  } 	
	   }
	 }
    
	 $ftp -> quit() 
		or print LOG "Couldn't quit from $host\n";
	
	foreach my $f ( @$orb ) {
            $f = basename $f;
            my $ff = "$here/$f.Z";
	    #print "$f \n" if -e $f;
	    if ( ! -e $ff ) { 
               $ok = 0; 
            } 
	    else {
	       push( @files,$ff ); 
	       print LOG "founded $f\n"; 
	   }
	}
	
    last if $ok;
    
  } 
      #close(LOG);
      print LOG "$ok $xfile @files\n";
      #print  "$ok $xfile |@files|\n";
      return( $ok,$xfile,@files);
};



sub get_brdc {
	my $yyyy = shift;
	my $doy = shift;
	my (undef,$yy) = unpack("a2a2", $yyyy);
	my $brdc = "brdc${doy}0.${yy}n";
	my $host = "cddisa.gsfc.nasa.gov";
	my $dir = "/pub/gps/gpsdata/brdc/$yyyy";
	my $ok;
	print "brdc $brdc";
	if ( ! -e "$par{BRDC_DIR}/$yyyy/$brdc.Z" ) {
		my $ftp = Net::FTP->new( $host,
					  Timeout => 60,
					  Debug => 0,
					  Passive => 1 ) || print "Can't connect to host $host : $@\n";
		  $ftp -> login ( "anonymous", $password ) || print "Couldn't login into $host\n";
	 	  $ftp -> binary () || print "Couldn't set type to binary in $host\n";
		  $ftp -> cwd ( $dir ) || print "Couldn't change to dir $dir\n";
		  $ok = $ftp -> get ( "$brdc.Z" ) ;
		  $ftp -> quit() || print "Couldn't quit from $host\n";
		  copy( $brdc.Z ,"$par{BRDC_DIR}/$yyyy") if  -e "$brdc.Z" ;
			
	} else {
		copy("$par{BRDC_DIR}/$yyyy/$brdc.Z",".") || carp "couldn't copy $_ :$!\n";
	};
	if ( -e "$brdc.Z" ) {
		system( "gzip -df $brdc.Z" );
		return (1);
	} else { return (0) };
	 
}

sub get_rnx {
	my $site = shift;
	my $yyyy= shift;
	my $doy = shift;
	my $gipsy = shift || 1 ;
	my $FTP = 1 ;
	my $mm =0;
	my $dd = 0;

	my $yyyymmdd='';
	my @sites=();
	my @missing_sites=();
	my @networks=();
	my @sources=();
	my %ftp_info=();
	my $result=0;

	my $logfile = '';

	($y,$mm,$dd) = doy2cal($yyyy,$doy);
  
	$yyyymmdd=sprintf ("%d-%02d-%02d",$yyyy,$mm,$dd);	

	(undef,$yy) = unpack("a2a2", $yyyy);

	$doy=sprintf("%003d",$doy);

	$logfile = "$yyyymmdd.$doy.get_rnx.log";

	unlink ( $logfile );
	
	open ( LOGFILE , ">$logfile" ) || carp "unable to open $logfile for write $!";

	select(LOGFILE); # all  print statements go to logfile

	#if ( ! $gipsy ) { $gipsy = 1 };

	if ( ! $FTP )  { $FTP = 1 };

	#create array of site names
	@sites = split(/,/,join(',',$site));
#	push(@sites,$site);
	print ( "sites to downlad @sites\n" );#lc( @sites);

	#print "${yyyy} $yy $mm $dd $yyyy $doy $site $FTP $gipsy \n";

	foreach my $stn ( @sites ) { # loop  for sites

		@networks=get_nets($stn); #get the network[s] in which the site is listed (should be only one, but not always) 

		print " site $stn in network[s]: @networks\n";
        
		@sources = get_sources( @networks ); #get sources, i.e. places where to get the rinex file

		print ( "site $stn available from @sources\n" ); 

		%ftp_info = make_get_info($stn,$yyyy,$doy,@sources); # make specific information for downloading/copying
		my $ok;
		my $rinex='';
		foreach my $src ( @sources ) { # loop over all possible sources
			
			my $host='';
			my $dir='';
			($host,$dir,$rinex) = @{$ftp_info{$src}};
			unlink ( $rinex ); # delete rinex file, just in case
			#		print "$src\n";
			print " site info for $stn: @{$ftp_info{ $src }}\n";

			if ( $src =~ /^LOCAL/ ) { # see if is available locally
				my $target = $dir.'/'.$rinex;
				print "copying $target to local dir \n";
				$result = copy( $target, $rinex );
				if ( ! $result ) { print "Couldn't copy $target : $!\n" };
			}elsif ( $FTP ) { # try to download file with ftp if it is required
				print "downloading $rinex from $host; dir: $dir\n" ;
				my $ftp = Net::FTP->new( "$host",
					  Timeout => 60,
					  Debug => 0,
					  Passive => 1 ) || print "Can't connect to host $host ( $src ): $@\n";
					  $ftp -> login ( "anonymous", $password ) || print "Couldn't login into $host\n";
	 			      $ftp -> binary () || print "Couldn't set type to binary in $host\n";
					  $ftp -> cwd ( $dir ) || print "Couldn't change to dir $dir\n";
		  	  	$ok = $ftp -> get ( $rinex ) ;
					  $ftp -> quit() || print "Couldn't quit from $host\n";
			};
		
			#};
		
		if ( $ok ) { $result = 1; last } # leave the loop if I've got the rinex
	    else { print "Couldn't get $rinex\n"};
	 }
	 
	 if ( $result && -e $rinex ) { #if I've Got rinex file
		my ($rname,$rext,$cext) = split /\./, $rinex ; 		#get parts of filename
		my $unzip = "gzip -dc";
		my $rnx = $rname.".".$rext;
		system ( "$unzip $rinex 1>$rnx 2>$logfile" );   	#uncompress rinex
		#print "$rname $rext $cext\n";
		if ( $rext =~ /\d\dd/ ) {  				#if it is Hatanaka format 
			system ( "(crz2rnx $rnx 2>&1) 1>$logfile" );	# convert to standard rinex
		};
		
		if (  ! -e "$rname.${yy}o" )  { 			#if rinex doesn't exist
			 print "$rname.${yy}o not founded, fail to convert $rinex: $!";
			 $result=0
		} else { $rnx =~ s/d$/o/g; };				# change extension from YYd to YYo if it is founded;
		
		if ( $gipsy && $ok) { 					#change name to gipsy convention if it is required	

				my $gnx = `nametrans $rnx 2>$logfile` ;
				chomp $gnx;				#get rid off end of line
				print ("changing name: $rnx to $gnx\n");
                symlink $rnx,$gnx or 
                  print ( "couldn't symlink name $rnx to  $gnx: $!\n" ); 
				#move( $rnx, $gnx) || print ( "couldn't change name $rnx to $gnx: $!\n" );
		};
				
	 } 
	 if ( ! $result ) { push( @missing_sites, $stn ) } ; # not ok after all -> add site to the missing list

	my $out = @missing_sites;

	print ( "ok $result $out missing sites: @missing_sites\n" );

	return $result; # number of files
	
	};

}

#========================END===============================================================================#

sub get_nets {
my $site = shift;
my @networks=();

open ( NETSITE,"<", "$par{GPS_HOME_DIR}/etc/net_sites") or croak "couldn't open net_sites\n";

while ( <NETSITE> ) {
	chomp;   		#no newline
	s/#.*//; 		#no comments
	next if /^\s*$/;        #skip blank lines
	my @stn = split;
	if ( grep {$site eq $_} @stn ) {
		push (@networks,shift @stn);  
	};
};
return(@networks); 	
}

sub get_sources {

my @nets = @_;

my @sources = ();

open ( SRCNET, "<", "$par{GPS_HOME_DIR}/etc/source_net") or croak "couldn't open source_net\n";

while ( <SRCNET> ) {
	chomp;   		#no newline character
	s/#.*//; 		#no comments
	next if /^\s*$/;        #skip blank lines
	my @src = split;
	foreach my $net (@nets) {
	  	if ( grep { $net eq $_ } @src ) {
			push (@sources,shift @src);
		}
	}
}
return(@sources); 

}

sub make_get_info {
my $site    = shift;
my $yyyy    = shift;
my $doy     = shift;
my @sources = @_;
my $SITE = uc($site);
my $yy =0;

my %ftp_info = ();

(undef,$yy) = unpack("a2a2", $yyyy);


open ( FTPSRC, "<", "$par{GPS_HOME_DIR}/etc/ftp_address_source" ) or croak "couldn't open ftp_addresses\n";

	while ( <FTPSRC> ) {

		chomp;   		#no newline

		s/#.*//;		#no comments
	
		next if /^\s*$/;	#skip blank lines
	
		#replace values
		s/\$\{SITE\}/$SITE/g;
		s/\$\{site\}/$site/g;	 
		s/\$\{yyyy\}/$yyyy/g;
		s/\$\{yy\}/$yy/g;
		s/\$\{doy\}/$doy/g;
		my @fields = split;
	
		foreach my $source (@sources) {
			next unless $source eq $fields[0];
			
			my $src = shift @fields;
			#$host = shift @fields;
			#$dir = shift @fields;
			#$rnx = shift @fields;
			
			push ( @{$ftp_info{$src}},@fields[0..2]);
			#push ( @{$ftpsrc{$src}},$dir);
			#push ( @{$ftpsrc{$src}},$rnx);

		}

	};
#return @{$ftp_info{$source}};
return %ftp_info;

}



1;
