package GPS::GMTMap;
use File::Basename;
use Exporter;

#Routines to make basic maps with GMT, and
#routines to manipulate ascii data.
#The ascii data format is GEO::Distance compatible.
#Routines were ported in a hurry from cshell-scripts
#and need to be perl-ized.
#
#To do:
#change/add arguments to init_base_map
#add more projections options
#etc...
#
#Marcelo Santillan
#April 2005
#Geodesy Lab & PANGA
#Central Washington University

@ISA= ('Exporter');

@EXPORT = qw( media_pt media_xy 
              get_hv get_ar get_map_center
	      get_hv_scale get_scale_bar_dim
	      get_xy_offsets make_region
	      get_hv_ticks get_map_res
	      lonlatcorner get_llminmax
	      set_dhdv set_graph_scale_length 
	      load_data array2data data2array 
	      print_data filter_data cat_data
	      sort_data fix_lon add_field 
	      find_baricenter find_center find_limits 
	      select_gwindow_data substract_data ps2jpg
	      init_base_map plot_names plot_symbols 
	      $PROJ  $MEDIA  $XPTS2CM  $YPTS2CM 
 	      $ORIENT $HDIM $VDIM $H $V $HTITLE
              $XPAGE $YPAGE
	      $TITLE  $SUBTITLE $MAP_FNAME
              $AR $RIVERS
	      $SCL_LDIM $SCL_WDIM $SCALE_BAR_DIV
              $GRAPH_SCALE_LENGTH $MAXRES
	      $LON  $LAT $DL $DH $DL $DV $DLAT $DLON
              $LONMIN $LONMAX $LATMIN $LATMAX
	      %paper_media  %rgb %boundaries
	      %rivers %orient %symbols
            );

#Define default parameters

our $PROJ = 'm';
our $MEDIA = "letter";
our $UNIT = "cm";
our $H = 21.6; #/2.54;  #inch
our $V = 27.9; #/2.54;  #inch
our $XPTS2CM = 612/$H;
our $YPTS2CM = 792/$V;
our $XPAGE;
our $YPAGE;
our $ORIENT  = "P"; #P=Portrait L=Landscape
our $HDIM  = 60;
our $VDIM  = 40;
our $HTITLE = $V*0.2*$VDIM/100;
our $TITLE = 'MAP TITLE';
our $SUBTITLE = 'MAP SUBTITLE';
our $MAP_FNAME = 'map';
our $AR = $H/$V;
our $RIVERS = 'a';  # all rivers and canals 
our $SCL_LDIM = 50;
our $SCL_WDIM = 5;
our $SCALE_BAR_DIV=10;
our $GRAPH_SCALE_LENGTH = 50;  #km
our $P_HDIM = 80;
our $P_VDIM = 70;
our $L_HDIM = 80;
our $L_VDIM = 70;


our $MAXRES = 0.5; #minutes = 30 sec; Maxres of gtopo30

#Seattle,WA
our $LON = -122.30;
our $LAT = 47.45;
#############################################################
# Coverage area as the rectangle 
# 2*DH x 2*DV centered in LAT / LON
#
# DL = 3 - 4.3 degrees covers NMSZ and GPS sites
# DL = 21      degrees covers the trackers
# DL = 1.5     for Enola
#############################################################
our $DL = 4.3;
our $DH = $DL;
our $DV = $DH*$AR;
our $DLAT=2*$DV;
our $DLON=2*$DH;
our ($LONMIN,$LONMAX,$LATMIN,$LATMAX);



our %orient = ( "P","-P","L"," ");
our %unit = ("inch","i","cm","c","meter","m");

our %symbols = ("star","a",
	        "bar","b",
		"circle","c",
		"diamond","d",
		"ellipse_dir","e",
		"ellipse_azi","E",
		"hexagon","h",
                "inv_triang","i",
                "point","p",
                "square","s",
                "triangle","t",
                "vector","v",
                "pie_wedge","w",
                "cross","x",
	       );
# Paper Media List width height in  points
#      Media  widthxheight
# 
our %paper_media = (
	"A0", "2380x3368",
	"A1", "1684x2380",
	"A2", "1190x1684",
	"A3", "842x1190",
	"A4", "595x842",
	"A5", "421x595",
	"A6", "297x421",
	"A7", "210x297",
	"A8", "148x210",
	"A9", "105x148",
	"A10", "74x105",
	"B0", "2836x4008",
	"B1", "2004x2836",
	"B2", "1418x2004",
	"B3", "1002x1418",
	"B4", "709x1002",
	"B5", "501x709",
	"archA", "648x864",
	"archB", "864x1296",
	"archC", "1296x1728",
	"archD", "1728x2592",
	"archE", "2592x3456",
	"flsa", "612x936",
	"halfletter", "396x612",
	"note", "540x720",
	"letter", "612x792",
	"legal", "612x1008",
	"11x17", "792x1224",
	"ledger", "1224x792"
);

our %boundaries = ( "national",1,
	            "states",2,
		    "marine",3,
		    "all",'a'
                   );

our %rivers = ( "permanent",1,
	        "additional_major",2,
		"add",  3,
		"minors",4,
		"int_major",5,
		"int_add",6,
		"int_minor",7,
		"canals_major",8,
		"canals_minor",9,
		"irrig_canals",10,
		"all_r_c",'a',
		"all_p", 'r',
		"all_i", 'i',
		"all_c", 'c'
	      );
		   
our %rgb = ("YELLOW","255/255/50",
   "DKYELLOW","214/216/0",
   "ORANGE","255/192/50",
   "BROWN","160/64/32",
   "LTBROWN","216/166/75",
   "PINK","255/225/255",
   "LTRED","250/225/225",
   "RED","250/0/0",
   "DKRED","196/50/50",
   "PURPLE","255/50/255",
   "SKY","100/173/236",
   "LTBLUE","192/192/250",
   "VLTBLUE","225/250/250",
   "BLUE","0/0/255",
   "DKBLUE","4/24/178",
   "VDKBLUE","9/46/115",
   "CYAN","50/255/255",
   "GREEN","0/255/0",
   "DKGREEN","37/138/32",
   "FORESTGREEN","34/139/34",
   "OLIVEDRAB3","154/205/50",
   "SPRINGGREEN3","0/205/102",
   "WHITE","255/255/255",
   "VLTGRAY","210/210/210",
   "LTGRAY","192/192/192",
   "GRAY","128/128/128",
   "DKGRAY","88/88/88",
   "VDKGRAY","25/25/25",
   "BLACK","0/0/0",
   "SEAFILL0","192/192/250",
   "SEAFILL1","0/0/250",
   "SEAFILL2","0/0/240",
   "SEAFILL3","32/156/183",
   "SEAFILL","100/173/236",
   "LANDFILL","0/205/102",
   "LANDFILL2","0/205/122",
   "RIVERFILL","32/156/183"
);

sub swap {
   my $a = shift;
   my $b = shift;

   my $tmp = $a;
   $a=$b;
   $b=$tmp;

   return($a,$b)
	
};
# Scale factor Points -> Cm to convert media size to cm
sub media_pt {
    my $media = shift;
    my ( $h,$v ) = split(/x/ ,$paper_media{$media});    
    return ($h,$v);	
};
sub media_xy {
    
    my $media = shift;
    my $or = shift;
    my ($h,$v);
    
    ( $h,$v ) = split(/x/ ,$paper_media{$media}) ;    
    
    
    my $x = $h/$XPTS2CM;
    my $y = $v/$YPTS2CM;
   
   ($x,$y) = swap ( $x,$y) if $or eq "L"; 
   
   #print "media_xy ",$x," ",$y,"\n";
    return ($x,$y)	
};

sub set_dhdv {
   my $dl = shift;
   $DH = $dl;
   $DV = $dl * get_ar();
};
sub get_hv {
    my $media = shift;
    my $or = uc(shift);
    my $xp = shift;
    my $yp = shift;;  

    #my ($xp,$yp) = media_xy($media,$or);  
    my ($hd,$vd);
    if ( $or eq "P" ) {
        $hd=$P_HDIM;
	$vd=$P_VDIM;
    } else {
        $hd=$L_HDIM;
	$vd=$L_VDIM; 
    };
    
    #print "get_hv $or ",$xp*$hd/100," ",$yp* $vd/100,"\n";
    return ($xp * $hd/100,$yp*$vd/100)
    
}
sub get_ar {
   ($H,$V) = media_xy($MEDIA,$orient{$ORIENT});	
   return ($H/$V);
}
sub get_map_center {
   my $fcenter = shift;
   my $center = shift;
   my ($lat,$lon);
   my (@fields,@center);
   if ( -e $fcenter ) {
      open(FILE,"< $fcenter");
      while ( <FILE>){
         s/\#.*//;
	 next if /^\s*$/;
        (undef,$lon,$lat)=split(' ',$_) if /^$center/;	    	 
      };
   };
   return ($lon,$lat)
}

sub lonlatcorner {
     my $px = shift;
     my $py = shift;
     my $lonmin = shift;
     my $lonmax = shift;
     my $latmin = shift;
     my $latmax = shift;
     my $corner = uc(shift);

     my $dlat = $latmax - $latmin;
     my $dlon = $lonmax - $lonmin;
     
     my ($lon,$lat);
     
     if ( $corner =~ /(L|R)(T|B)/ ) {
	 if ( $corner eq 'LT' ) {    
           $lon = $lonmin + $dlon*$px/100;
	   $lat = $latmax - $dlat*$py/100
   	 };    
	 if ( $corner eq 'LB' ) {    
           $lon = $lonmin + $dlon*$px/100;
	   $lat = $latmin + $dlat*$py/100
   	 };    
	 if ( $corner eq 'RT' ) {    
           $lon = $lonmax + $dlon*$px/100;
	   $lat = $latmax - $dlat*$py/100
   	 };    
	 if ( $corner eq 'RB' ) {    
           $lon = $lonmax + $dlon*$px/100;
	   $lat = $latmin + $dlat*$py/100
   	 };    
     }else {
        die "CORNER: $corner is bad formed [[L|R][T|B]]\n";
     };
     
     return ($lon,$lat)	
}

sub get_hv_scale {
      my $h = shift;
      my $v = shift;
      my $dlon = shift;
      my $dlat = shift;
      my $or = uc(shift);   
      my $AR=1;
      my $ar = abs($dlon/$dlat);
   
      
      my $hscale = $h/$dlon;
      my $vscale = $v/$dlat; #$hscale; # for now
      
      #print "hv_scale hv $h $v\n";
      #print "hv_scale $hscale $vscale\n";
      #print "ar_d $AR $ar \n";
      if ( $ar < 0.9) { 
	      #print "swaping ar\n";
	      ($hscale,$vscale)=swap($hscale,$vscale) 
      };
 
      return ($hscale,$vscale) 
}
sub set_graph_scale_length {
    my $dl = shift;
    my  $l = abs($dl)*111.1*0.20;
    my @ls = (1/1000,1/100,1/10,0.5,1,2,10,25,50,100,250,300,500,1000,2000,5000);
    my $ll ;
    while ( $ls[0] < $l ) {
         $ll = shift @ls;
      }; 
    $GRAPH_SCALE_LENGTH = $ll;
    return $ll
};
sub get_scale_bar_dim {
    #scale bar dimension 50% of map height
    my $ldim = shift;
    #scale bar width 5% of map height
    my $wdim = shift ;
    # scale bar divisions by default 10
    my $divs = shift ;
    my $v    = shift;

    my $length = $v*$ldim/100;
    my $width  = $v*$wdim/100;

    return ($length,$width,$divs)
    
}

sub get_xy_offsets {
     my $xp = shift;
     my $yp = shift;
     my $h  = shift;
     my $v  = shift;     
     #my ($xp,$yp) = media_xy($MEDIA,$ORIENT);  
     #my ($h,$v) = get_hv ($ORIENT,$MEDIA);
     #my $scl_bar = shift || 0;
     my ($l,$w,$d); 
     ($l,$w,$d) = get_scale_bar_dim($SCL_LDIM,$SCL_WDIM,10,$v) if $scl_bar; 
     my $ar = $h/$v;
     my $xoff = ($xp - $h - $w)/2;
     my $yoff = ($yp -$v )/2;
     #($xoff,$yoff)=swap($xoff,$yoff) if $ORIENT eq "L";
     #print "xy_offsets $xoff $yoff $xp $h  $yp $v\n";
     return ($xoff,$yoff) 
};

sub get_llminmax {
    my $lon= shift;
    my $lat= shift;
    my $dh = shift;
    my $dv = shift;
    
    
    my $lonmin = $lon - $dh;
    my $lonmax = $lon + $dh;
    my $latmin = $lat - $dv;
    my $latmax = $lat + $dv;
   
    return($lonmin,$lonmax,$latmin,$latmax)
}

sub make_region{
    my $lon = shift;
    my $lat= shift;	
    my $dh = shift;
    my $dv = shift;
    
    
    my $lonmin = $lon - $dh;
    my $lonmax = $lon + $dh;
    my $latmin = $lat - $dv;
    my $latmax = $lat + $dv;

    my $gwindow = "$lonmin/$lonmax/$latmin/$latmax";
    
    return $gwindow
}

sub get_hv_ticks {
    my $lonmin = shift;
    my $lonmax = shift;
    my $latmin = shift;
    my $latmax = shift;
    my $dlat = $latmax - $latmin;
    my $dlon = $lonmax - $lonmin;
    my ($vticks,$hticks); 
    my $mxt = 5;
    
    
    if ( $dlat/$mxt >= 1 ) {
      $vticks = sprintf ("%d",$dlat/$mxt);
    } else {
      $vticks = sprintf ("%2.1f",0.5);
    };  
    if ( $dlon/$mxt >= 1 ) {
      $hticks = sprintf ("%d",$dlon/$mxt);
    } else {
      $hticks = sprintf ("%2.1f",0.5);
    };  
    
    return ($hticks,$vticks)
};

sub get_map_res {
    my $dlon = shift;
    my $xpage = shift;

    my $res = $dlon*60/($XPTS2CM*$xpage);
    
    if ( $res <= $MAXRES ) {
       $res = "0.5m";
    } else {
       $res = sprintf("%2.0fm",$res);
    }; 

    return $res

}
sub fix_lon {
    my $data = shift;
    foreach my $f ( @$data ) {
       $f->{lon} = $f->{lon} + 360 if $f->{lon} < 0;
    };
}
#
# synopsis
# $data = get_data({"file" => "data.txt","fields" => ["n","lon","lat","site"]});
# 
# Example:
# pbo_all.txt fields
# Name Longitude Latitude Region Type Group_ Location
# 
# $data_args = {"file"   => "pbo_all.txt",
#               "fields" => [ "name","lon","lat","region","type","group"]};
# $data = get_data($data_args);
#               
sub load_data {

  my $arg = shift; #reference to a hash
  
  my $f = $arg ->{file} || die "You need to supply: {file => filename}\n";

  my $keys = $arg->{fields} || die "You need to supply fields => [\"key1\",\"key2\"...] \n";
 
  
  my $keys_out = $arg->{output} || $arg->{fields};
  
  my @data;
  open(FILE,"< $f ") || die "Couldn't open  $f $!\n";
  while ( <FILE> ) {
      chomp;
      s/#.*//;
      s/^\s+//;
      next if /^\s*$/;
      my @fields = split(' ',join(' ',$_));
      my $h = {};
      foreach my $k ( @$keys ) {
	  my $field = shift @fields;    
	  $$h{$k}=$field if (grep /$k/,@$keys_out);
      }; 
      push @data ,$h ;   	      
   };
  close(FILE);
  return \@data
 
}
# synopsis
# $data = array2data({"fields" => ["n","lon","lat","site"]},@array);
sub array2data {
    my $arg = shift;
    my @array= @_;
    my $keys = $arg->{fields} || die "You need to supply fields => [\"key1\",\"key2\"...] \n";
  
    my $keys_out = $arg->{output} || $arg->{fields};
    
    my @data; 
    foreach my $l ( @array ) {
       my @fields = split(' ',join(' ',$l));
       my $h = {};
       foreach my $k ( @$keys ) {
	  my $field = shift @fields;    
	  $$h{$k}=$field if (grep /$k/,@$keys_out);
       }; 
      push @data,$h;
    }
   
   return \@data    

}

#Synopsis
#@array = data2array($data,{"output" => ["lon","lat","name"],"field_sep"=>" "});
sub data2array {
    my $data = shift;
    my $opt = shift || {};

    my $keys = $opt->{output} || ["lon","lat"];
    my $fsep  = $opt->{field_sep} || "";
    my $eol  = $opt->{field_eol} || "";
    my @array;

    foreach my $loc ( @$data ) {
       my $l='';
       foreach my $k ( @$keys ) {
           $l.="$loc->{$k}$fsep";
       };
       $l.=$eol;
       push @array,$l;
    };
    return @array
}

sub sort_data { # my sort
   my $data = shift; 	
   my $field = shift;
   my $rev = shift || 0;
   my $out = [];
   # check if field is alphanumeric
   if ( (grep { $_->{$field} =~/\p{Letter}/ } @$data) ) { 
      @$out =  sort { $a->{$field} cmp $b->{$field} } @$data if ! $rev;
      @$out =  sort { $b->{$field} cmp $a->{$field} } @$data if $rev;
        
   } else {  #I assume is a number 
       @$out =  sort { $a->{$field} <=> $b->{$field} } @$data if ! $rev;  
       @$out =  sort { $b->{$field} <=> $a->{$field} } @$data if $rev;  
   }
   @$data = @$out;
   return $data
}

#Based on sort algorithm from Geo::Distance (seems slow )
sub sort_data2 {
   my $data = shift; 	
   my $field = shift;
     my $tmp;
     my ($i,$j);
     for(my $i=@$data-1; $i>=0; $i--){
        for(my $j=$i-1; $j>=0; $j--){
            if($data->[$i]->{$field} < $data->[$j]->{$field}){
                  $tmp = $data->[$i];
                  $data->[$i] = $data->[$j];
                  $data->[$j] = $tmp;
	    }
       }
     }
}


#$filtered = filter_data($data,{ "filter" => ["region","PNW"] });
sub filter_data {
   my $data = shift;
   my $filter = shift;
   my $neg  = shift || 0;
   
   #my $keys = [keys %{$$data[0]}];
  
   #get the field and a regular expression to filter it 
   my ($key,$re_val) = @{$filter->{field}};
   
   my @data_out; 
   foreach my $d ( @$data ) {
	   #if ($d->{$key} eq "$val" ) {     
	if ( ! $neg ) {     
	  push @data_out, $d if  $d->{$key} =~ /$re_val/ ; 
	} else {
	  push @data_out, $d if  $d->{$key} !~ /$re_val/ ; 
	}
   };
   
   return \@data_out   
}

sub substract_data {
   my $data = shift;
   my $data2 = shift;
   my $key = shift;
   
   my @rm; 
   my $data_out=[];
   
   #find common elements
   foreach my $d ( @$data ) {
      foreach my $d2 ( @$data2 ) {
         if ( $d->{$key} eq $d2->{$key} ) {
	     push @rm, $d->{$key};  	 
	}; 
      };
   };
   
   # save uncommon
   foreach my $d ( @$data ) {
      push @$data_out,$d if ! (grep /$d->{$key}/,@rm) ; 	 
   };
   
   return $data_out   
}



sub add_field {
   my $data = shift;
   my $field = shift;
   my $force = shift || 0;
   my ( $key ,$val ) = @$field;
   
   foreach my $d ( @$data ) {
       $d->{$key} = $val if ! exists $d->{$key} || $force;
   }
	
}
sub find_limits {
    my $data = shift;
    my ($lonmin,$lonmax,$latmin,$latmax)=0 x 4;
    if ( @$data > 1 ) {  
      #find North - South limits
      sort_data($data,"lat");
      ($latmin,$latmax) = ($$data[0]->{lat},$$data[@$data-1]->{lat});

      # find West - East limits
      sort_data($data,"lon");
      ($lonmin,$lonmax) = ($$data[0]->{lon},$$data[@$data-1]->{lon});
   } elsif ( @$data == 1 ) {
      ($latmin,$latmax) = ($$data[0]->{lat},$$data[@$data-1]->{lat});
      ($lonmin,$lonmax) = ($$data[0]->{lon},$$data[@$data-1]->{lon});
   }
   return [$lonmin,$lonmax,$latmin,$latmax]
}
sub find_center {
  my $data = shift;
  my ($clon,$clat);
  my $limits = find_limits($data);
  my ($lonmin,$lonmax,$latmin,$latmax) = @$limits;
  
  #print "$clon/$n  $clat/$n  $name\n";
  return [{"lon"=>($lonmin+$lonmax)/2,"lat"=>($latmin+$latmax)/2}];
}
sub find_baricenter {
  my $data = shift;
  $n = @$data; #number of elements
  my ($clon,$clat);
  foreach my $f ( @$data ) {
      $clon += $f->{lon};
      $clat += $f->{lat};
  };
  #print "$clon/$n  $clat/$n  $name\n";
  return [{"lon"=>$clon/$n,"lat"=>$clat/$n}];
}

sub cat_data {
    my $data = shift;
    my $data2 = shift;
    push @$data,@$data2;
}

# return data within a geographic window
sub  select_gwindow_data {
     my $data = shift;
     my $gwindow = shift;  #limits
     my $gwdata=[];  
     my ($lonmin,$lonmax,$latmin,$latmax) = @$gwindow;
     foreach my $d ( @$data ) {
	push @$gwdata, $d if ( $d->{lon} >= $lonmin && $d->{lon} <= $lonmax && 
	                       $d->{lat} >= $latmin && $d->{lat} <= $latmax     ) ;
     }
	
     return $gwdata
}

sub print_data {
  my $data = shift;
  my $opt = shift;
  my $fh = shift || \*STDOUT;
  my $keys = $opt ->{fields}|| [keys %{$$data[0]}];
  foreach my $h ( @$data ) {
    foreach my $k ( @$keys  ) { 	  
        print $fh " $h->{$k}";
    }
    print $fh "\n";    
  };
}

sub init_base_map {
   my $media  = shift;
   my $or     = uc(shift);
   my $gwindow = shift;
   my $proj  =  shift;
   my $name =  shift;
   my $title =  shift || $TITLE;
   my $land_color = uc(shift) || "LANDFILL"; 
   my $sea_color = uc(shift) || "SEAFILL"; 
   my $river_color = uc(shift) || "RIVERFILL"; 
   my $cmd;  
   
   $land_color = $rgb{$land_color};
   $sea_color = $rgb{$sea_color};
   $river_color = $rgb{$river_color};
   $MEDIA = $media;
   $MAP_FNAME = $name;    
   $ORIENT = $orient{$or}; 
   my $datares = 'i';  
   my ($lonmin,$lonmax,$latmin,$latmax) = split(/\//,$gwindow);
   my $dlon = $lonmax - $lonmin;
   my $dlat = $latmax - $latmin;

   
   my ($xp,$yp) = media_xy($media,$or);  
   
   my ($h,$v) = get_hv($media,$or,$xp,$yp);
   
   my ($hscl,$vscl) = get_hv_scale($h,$v,$dlon,$dlat);  
  
   my ($xoff,$yoff) = get_xy_offsets($xp,$yp,$h,$v); 
   
   my ($gslon,$gslat) = lonlatcorner(50,12,$lonmin,$lonmax,$latmin,$latmax,"LB");
   
   my  $gslength = set_graph_scale_length($dlon); 
  
   my ($hticks,$vticks) = get_hv_ticks($lonmin,$lonmax,$latmin,$latmax); 
   my $cmd_common;
   
   $cmd_common  = " -J${proj}${hscl}c";
   #$cmd_common  = " -J${proj}${h}c";
   $cmd_common .= " -R$gwindow";
   $cmd = $cmd_common;
   #$cmd .= "  -Lf$gslon/$gslat/$gslat/${gslength}k";
   $cmd .= " -K"; #continue
   $cmd .= " -G$land_color";
   $cmd .= " -Ba$hticks/a${vticks}:.\"${title}:\"";
   $cmd .= " -S$sea_color";
   $cmd .= " -N$boundaries{all}";
   $cmd .= " -I$rivers{permanent}";
   $cmd .= " -D$datares";
   $cmd .= " -W1/$rgb{BLACK}";
   $cmd .= " -I1/1p/$rgb{RIVERFILL}";
   $cmd .= " $ORIENT";
   $cmd .= " -X${xoff}c";
   $cmd .= " -Y${yoff}c";
  
   #print "pscoast $cmd\n"; 
   system("gmtset PAPER_MEDIA $media'+'
	   gmtset DOTS_PR_INCH 600
	   pscoast $cmd > $name.ps
	 ");   
  return $cmd_common 
}

sub plot_names {
    my $args_common= shift;
    my $name      = shift;
    my $font_size = shift;
    my $data = shift;
    my @names;
    foreach my $d ( @$data ) {
         push @names,sprintf("%f %f %d 0 1 CT %s\n",$d->{lon},$d->{lat},$font_size,$d->{name});
    }
     my $args = $args_common;
     $args .= " -K";
     $args .= " -O";
     $args .= " -G$rgb{BLACK}";
     $args .= " -P";
     $args .= " -D0/-0.1";
     #print "pstext $args\n";
     open(PS ,"| pstext $args  >>  $name.ps");
     print PS @names;
     close PS;
}
sub plot_symbols {
    my $args_common= shift;
    my $name      = shift;
    my $symb = lc(shift) || "triangle" ;
    my $size = shift ;
    my $c = uc(shift) || "RED";
    my $data = shift;
    my @lonlat;
    $symb = $symbols{$symb}; 
    my $color = $rgb{$c};
  
    foreach my $d ( @$data ) {
        push @lonlat,sprintf("%f %f\n",$d->{lon},$d->{lat});
    }
     my $args = $args_common;
     $args .= " -K";
     $args .= " -O";
     $args .= " -S${symb}${size}";
     $args .= " -G$color";
     $args .= " -W2/$rgb{BLACK}";
     $args .= " -M";
     $args .= " -L";
     #print "psxy $args\n";
     open(PS ,"| psxy $args  >>  $name.ps");
     print PS @lonlat;
     close PS;
}

sub ps2jpg {
    my $psfile = shift;
    my $media = shift;
    my $rotate = shift || "";
   
    $psfile =~ s/\.ps$//;
     

    $rotate = "-rotate 90" if $rotate;
    
    system("convert -page $media $psfile.ps -quality 100 -trim $rotate $psfile.jpg");

}

1;


