package GPS::GetOrbits;
use Exporter;
use Carp;
use File::Basename;
use File::Copy;
use File::Path;
use Net::FTP;
use GPS::DATES;
use GPS::Defaults;
use GPS::IPNames;
use GPS::FTP::Orbits;
use GPS::IGS;
use GPS::JPL;
use Cwd;

@ISA= ('Exporter');

@EXPORT = qw( get_goa_orbits get_orbits get_brdc ) ;

our $username=$ENV{USER};
our $thishost='geology.cwu.edu';
our $password=$username . '@' . $thishost;
our %ip = get_ftp_ip();
our %par = get_defaults( env => \%ENV );

my %ARCHIVE = ( jpl       => { rapid => $par{JPL_RAPID_ORBITS_DIR},
                               final => $par{JPL_ORBITS_DIR},
                               x     => $par{X_FILES_DIR},
                              },
                igs       => { rapid => $par{IGS_ORBITS_DIR},
                               final => $par{IGS_ORBITS_DIR},
                               ultra => $par{IGS_ORBITS_DIR},
                               flinn_rapid => "$par{IGS_ORBITS_DIR}/flinn_rapid",
                               flinn_final => "$par{IGS_ORBITS_DIR}/flinn_final",
                             },
              );
sub get_goa_orbits {

my $cache_dir = "$par{CACHE_DIR}/orbits";
my $cache_age= 0.5; #2; #days
my $cached = 0;

my @of = qw/peci  Shadow  TDPfile  tpeo.nml XFILE wlpb/;
my $xdir = $par{X_FILES_DIR}; 
   $ac = lc $ac;
   if ( $ac !~ /(jpl|igs)/ ) { die "Unknown agency $ac (use igs|jpl )\n"; };
   if ( $today ) {
     my @tmp = split ( /;/,$now );
	    $cal = $tmp[0];
   };

   $cal = get_date($date ,'cal',$delay);

   $latest =  $latest ? 'yes' : '_';

   $type=lc($type);
   my @answer; 
  my $ok=0;

  if ( ! $odir ) {
    $odir = "$par{ORBITS_DIR_HOME}";
  }

  if ( $clear ) {
    $cache_age = 0 if $clear;
  }
  clear_orbit_cache($ac,$cache_age);

  $cached = check_local_orbits($cal,$ac,$type);
 
  $ok = 0;
  #$ftp = 1 if ! check_local_orbits($cal,$ac,$type);
  #if ( $gnet && ! $cached ) {
  if ( ! $cached ) {
     my $co_dir= "$cache_dir/$ac/$cal";
     system ("
              mkdir -p $co_dir 2> /dev/null
              touch $co_dir/$type.log 2> /dev/null                
            ");
     my %opt = ( date    => $cal,
                 ac      => $ac,
                 type    => $type,
                 ftp     => $ftp,
               );
     $opt{flinn} = 1 if $ac eq 'igs';
     $ok = get_orbits(%opt);
     if ( $ok ) {
       foreach my $f ( @of ) {
         copy $f , $co_dir if -e $f;  
       }
     }
     $cached = 0;
  }
  if ( $cached ) {
     my $result=0;
     foreach my $f ( @of ) { 
	  #print STDERR "copying $f\n";
      my $of = "$cache_dir/$ac/$cal/$f";
 	  copy($of,$here) or die "couldn't copy $of to $here $!\n";
      $result ++ if -e basename($of);
    }
    $ok = @of == $result; 
    #print STDERR @answer;
 } 


  #print STDERR "$ok success\n";  	
  print $ok;

}


sub get_orbits {
    my %opt = (
      unpack_files => 1,
      frame   => 'itrf05',
      date    => '',
      hh      => '00',
      ac      => '',
      orb_int => '111',
      type    => 'final',
      flinn   => '',
      ftp     => 0,
      dir     => '',
      zip     => 'Z',
      out_dir => cwd,
      @_,
    );
    my $here = $opt{out_dir};
    my $date = $opt{date} or croak "get_orbits: NO INPUT DATE! \n";
	open ( LOG , "> get_orbits.log") or die "$!\n";
       
    my $cal = get_date($date,'cal');
    my $yyyy = unpack 'a4',$cal;
    my $gpsw = get_date($cal,'gpsw');
    $gpsw = sprintf("%04d",$gpsw);
    my $ac = $opt{ac} ;
    my $type = $opt{type};
    my $odir = $ARCHIVE{$ac}{$type}; 
    my $xdir = '';
    my $frame=$opt{frame};
    my $xfile = '';
    if ( ! -d $odir ) { `mkdir -p $odir`; }       
   
    my @orbit_files=() ;
    my @xf=();

    if ( $ac eq 'igs' ) {
      @orbit_files = igs_fnames(%opt);    
    }
    elsif ( $ac eq 'jpl' ) {
      $xdir = "$ARCHIVE{$ac}{x}/$frame";
      @orbit_files = grep ! /x/, jpl_fnames(%opt);    
      @xf = grep /x/, jpl_fnames(%opt) ;
    }
    # print STDERR "orbit dir $dir\n";
	#check local dir for orbits and xfiles
    foreach my $ff ( @orbit_files  ) {
        my $f = basename $ff;
        if ( $ac eq 'igs' ) {
           ($gpsw,$gdow) =  unpack 'x3a4a1', $f;
           $gdow = $gdow > 6  ? 6 : $gdow ;
           $yyyy         = unpack 'a4', get_date("$gpsw$gdow",'cal');
        }                           
 
        if ( $ac eq 'jpl') {
           $yyyy         = unpack 'a4', $f ;
        }
 
        $dir = $ac eq 'jpl' && $type eq 'final'       ?  "$odir/$yyyy" :
               $ac eq 'jpl' && $type eq 'rapid'       ?   $odir        :
               $ac eq 'igs' && $type eq 'final'       ?  "$odir/$gpsw" :
               $ac eq 'igs' && $type eq 'rapid'       ?  "$odir/$gpsw" :
               $ac eq 'igs' && $type eq 'ultra'       ?  "$odir/$gpsw" :
               $ac eq 'igs' && $type eq 'flinn_final' ?  "$odir/$yyyy" :
               $ac eq 'igs' && $type eq 'flinn_rapid' ?  "$odir/$yyyy" :
                                                          ''           ;                                                  
        if ( ! -d $dir ) { mkpath $dir; } #`mkdir -p $dir`; }       
        
        if ( $opt{ftp} || ! -e "$dir/$f" || -z "$dir/$f" ) { 
	      $ftp = 1;
	    } 
        else {
	      copy("$dir/$f",$here);
	      push ( @files,"$here/$f" );
	      print LOG "founded $dir/$f\n"; 
        }
     }

     if ( @xf ) {
       foreach my $ff ( @xf ) {
          my $f = basename $ff;
	      if ( $opt{ftp} || ! -e "$xdir/$f" ) {
	        $ftp=1;
	        print LOG "$xdir/$f not founded \n";
          }
          elsif ( -e "$xdir/$f" ) { 
              $ftp=0;
		      copy("$xdir/$f",$here); 
		      $xfile="$here/$f";
		      print LOG "founded $xdir/$f\n";
		      last;
	      } 
       }
    }
     	
    my $ok=0;
	if ( $ftp ) {
	   $ok = $ac eq 'igs' ? igs_fetch(%opt) : jpl_fetch(%opt);
	   if ( $ok ) {
		   print LOG "$type orbits succesfully retrieved \n";
		   #copy to orbits dir
          foreach my $ff ( @orbit_files  ) {
             my $f = basename $ff;
             if ( $ac eq 'igs' ) {
               print "Filename in GetOrbits.pl $f\n";
               ($gpsw,$gdow) =  unpack 'x3a4a1', $f;
               $gdow = $gdow > 6  ? 6 : $gdow ;
               $yyyy         = unpack 'a4', get_date("$gpsw$gdow",'cal');
             }                           
             if ( $ac eq 'jpl') {
                 $yyyy         = unpack 'a4', $f ;
             }
             $dir = $ac eq 'igs' && $type eq 'final'       ?  "$odir/$gpsw" :
                    $ac eq 'igs' && $type eq 'rapid'       ?  "$odir/$gpsw" :
                    $ac eq 'igs' && $type eq 'ultra'       ?  "$odir/$gpsw" :
                    $ac eq 'igs' && $type eq 'flinn_final' ?  "$odir/$yyyy" :
                    $ac eq 'igs' && $type eq 'flinn_rapid' ?  "$odir/$yyyy" :
                    $ac eq 'jpl' && $type eq 'final'       ?  "$odir/$yyyy" :
                    $ac eq 'jpl' && $type eq 'rapid'       ?   $odir        :
                                                               ''           ;         
             if (  -e $f ) { 
	           copy( "$here/$f" , $dir);
	           print LOG "founded $dir/$f\n"; 
	         }
	      }
	       
          if ( @xf ) {
              foreach my $ff ( @xf ) {
                 my $f = basename $ff;
				 $xfile="$here/$f";
			     if ( -e "$here/$f" ) {
                    `gzip -d $xfile`;
                    $xfile =~ s/\.Z$//;
                    copy($xfile,$xdir); 
				    print LOG "$xfile founded \n";
				    last;
			     } 
                 else { 
				     print LOG "$f not founded \n";
			     } 
                 
		      }
              
          } # @xf
         
	  } # ok 
      else {  
           print LOG  "FTP did not get all the $agency $type orbits\n ";  
      }
       
	} #ftp
    else {
      $ok=1;
    }

    if ( $opt{unpack_files} ) { 
      if ( $ac eq 'jpl' ) {
         jpl_unpack( %opt );
       }
       elsif ( $ac eq 'igs' ) {
          igs2jpl(%opt,flinn_names => '') if $opt{unpack_files};
       }
       $ok=0;
       if ( -s "$here/TDPfile"   &&
            -s "$here/Shadow"    &&
            -s "$here/peci"      &&
            -s "$here/tpeo.nml"      ) { $ok = 1; }
    }
    print LOG "ok: $ok\n";
	close(LOG);
	return $ok 
}

sub clear_orbit_cache {
  my $ac  = shift;
  my $age = shift || 7; #days
  opendir(my $dh,"$cache_dir/$ac"); 		
  my @dirs = grep /^\d{4}-\d{2}-\d{2}$/,readdir($dh);
  foreach my $d ( @dirs ) {
     if ( -M "$cache_dir/$ac/$d" > $age ) {
        #print "erasing $d\n";
        system("rm","-rf","$cache_dir/$ac/$d");
     }
  }

}

sub check_local_orbits {
	my $cal = shift;
	my $ac  = shift;
	my $type = shift;
	my $result=0;
    my $local_orbits_dir = "$cache_dir/$ac/$cal";
       #$local_orbits_dir = "$outdir/$ac/$cal";
	if ( -e $local_orbits_dir ) {
		opendir(my $dh,$local_orbits_dir);
		my @files = grep !/log$/,grep ! /\.+/,readdir($dh);
		#check if we have the same type 
		if ( -e "$local_orbits_dir/$type.log" ){ ;
		   #check if we have all the files 	
		   foreach my $f ( @of ) {
	    		$result++ if  -e "$local_orbits_dir/$f"; 
		   }
		}
	}
    return $result == @of
}	

sub get_brdc {
    my %opt = (
       date => '',
       year => '',
       doy  => '',
       @_,
    );
	my $yyyy = $opt{year};
	my $yy = unpack("x2a2", $yyyy);
	my $doy =  $opt{doy};
    if ( $opt{date} ) {
      ($yyyy,$doy) = split ' ', get_date( $opt{date},'doy');
    }
	my $brdc = 'brdc'.$doy.'0.'.$yy.'n';
    
    my %info = ( cddis => { host => "cddis.gsfc.nasa.gov",
                            dir  => "/pub/gps/data/daily/$yyyy/brdc",
                            brdc => "$brdc.Z"
                           },
                 cors =>  { host => "cors.ngs.noaa.gov",
                            dir  => "/cors/rinex/$yyyy/$doy",
                            brdc => "$brdc.gz"
                           },
                  panga => { host => "ftp.panga.cwu.edu",
                             dir  => "/pub/data/$yyyy/$doy/nav",
                             brdc => "$brdc.Z"
                            }
                 );
	#my $host = "cddis.gsfc.nasa.gov";
	#my $dir = "/pub/gps/data/daily/$yyyy/brdc";
	#my $host = "ftp.panga.cwu.edu";
	#my $dir = "/pub/data/$yyyy/$doy/nav";
	#my $host = "cors.ngs.noaa.gov";
	#my $dir = "/cors/rinex/$yyyy/$doy";
	#my $ok;
    if ( -e $brdc ) { return $brdc }
    if ( ! -d "$par{BRDC_DIR}/$yyyy" ) { mkpath "$par{BRDC_DIR}/$yyyy"; }
	if ( ! -e "$par{BRDC_DIR}/$yyyy/$brdc.Z" ) {
      foreach my $src ( sort keys %info ) {
        my $ok = 0;
        my $host = $info{$src}{host};
        my $brdcz = $info{$src}{brdc};
        my $dir   = $info{$src}{dir};
        #print "$src $host $dir $brdcz\n";
		my $ftp = Net::FTP->new( $host , #$ip{$host},
					  Timeout => 60,
					  Debug => 0,
					  Passive => 1 ) 
                  or warn "Can't connect to host $host : $@\n";
                  
		  $ftp -> login ( "anonymous", $password ) 
                  or warn "Couldn't login into $host\n";
                  
	 	  $ftp -> binary () or warn "Couldn't set type to binary in $host\n";
		  $ftp -> cwd ( $dir ) or warn "Couldn't change to dir $dir\n";
		  $ok = $ftp -> get ( $brdcz );
          if ( -e $brdcz ) {
		  `gzip -df $brdcz; compress $brdc`; 
		  # This is for substituting gzip for compress
		  #`gzip -df $brdcz; compress -f -S .Z $brdc`; 
               copy( "$brdc.Z" ,"$par{BRDC_DIR}/$yyyy"); 
               $ok = 1;
          }
		  $ftp -> quit() or warn "Couldn't quit from $host\n";
          last if $ok;
      }
	}
    else {
		copy("$par{BRDC_DIR}/$yyyy/$brdc.Z",".") 
          or carp "couldn't copy $_ :$!\n";
	}
	if ( -e "$brdc.Z" ) {
		`gzip -df $brdc.Z `;
		return  $brdc ;
	} else { return 0  };
	 
}


1;
