package GPS::GetRinex;
use Exporter;
use GPS::DATES;
use GPS::Defaults;
use GPS::IPNames;
use Carp;
use File::Basename;
use Cwd;
use File::Copy;
use Net::FTP;

@ISA = ( 'Exporter' );

@EXPORT = qw/ get_rnx
              rnx_name
            /;
our %par = get_defaults( );
our $gps_home = $ENV{GPS_HOME_DIR};
our $username=$par{GPS_USER};
our $password = "$username\@geology.cwu.edu";
our %sr = ( '01' => 'i',
            '05' => 'v',
            '15' => 'y',
            '30' => '0',
);

=head1 SYNOPSIS

Get a RINEX file

=head1 FUNCTIONS

=cut

sub get_rnx {
   my %opt = ( 
        rnx        => '',
        save       => 0,
        date       => '',
        year       => '',
	    doy	       => '',
        sampling   => 30,
	    ftp	       => 1 ,
	    gipsy_name => '',
	    site	   => '',
        unpack_rnx => 1 ,
        local_only => 0,
        @_,
   );   
   my ($y,$yy,$doy,$cal,$site,@missing_sites,$save);
   my @missing_sites=();
   $save = $opt{save};
   $site = lc $opt{site} if $opt{site};
   if ( $opt{rnx} =~ /(\w{4})(\d{3}).\.(\d{2})/ ) {
      $site = lc $1;
      $doy = $2;
      $y = $3 < 50 ? $3 + 2000 : $3 + 1900;
   }
   elsif ( $opt{date} ) {
      ($yyyy,$doy) = split ' ',get_date($opt{date},'doy');
   }
   elsif ( $opt{year} && $opt{doy} ) {
      $yyyy   = $opt{year};
      $doy = $opt{doy};
   }
   else {
      croak "No date or year,doy \n";
   }
   if ( ! $site || $site !~  /[A-Za-z0-9_]{4}/ ){ 
         croak "Not a valid site name: $site\n";
   }  
   $yy = unpack('x2a2',$yyyy);
   my $sampling = sprintf "%02d", $opt{sampling} > 30 ? 30 : $opt{sampling};
   die "Invalid sampling rate $sampling\n" if ! $sampling || $sampling eq '00'; 
   die "Invalid sampling rate $sampling\n" if  $sampling != 30 && $opt{gipsy_name} ; 
   my ($rname,$rext,$cext);
   my $unzip = "gzip -dc";
   my @sites = split /,/, $site;
   my @sources = get_sources();
   my %ftp_info = ();
   init_log( "$yyyy.$doy.get_rnx.log" );
   my %ip = get_ftp_ip(); 
   foreach my $stn ( @sites ) {
      %ftp_info = get_ftp_info($stn,$yyyy,$doy,$sampling);   
      my $ok = 0;
      my $rinex = '';
      foreach my $src ( @sources ) {
         next if $opt{local_only} && $src !~ /^LOCAL/;
         my $host = '';
         my $dir = '';
         ($host,$dir,$rinex) = @{$ftp_info{$src}};
         $host = $ip{$host};
	 if (-e $rinex) {print_log("File $rinex already exists, removing\n"); unlink($rinex);}
         print_log( "site info for $stn: @{$ftp_info{ $src }}\n" );
         
         if ( $src =~ /^LOCAL/ ) {
	     $ok = get_rnx_local($dir,$target,$rinex);
         }
         elsif ( $opt{ftp} ) {
	     $ok = get_rnx_remote($host,$password,$dir,$rinex);
		}
	 if ($ok) {print_log("Successfully downloaded $rinex\n");}
         if ( $ok && $save ) {
            mkdir "$par{RNX_DIR}/$yyyy/$doy" if ! -d "$par{RNX_DIR}/$yyyy/$doy";
            if ( $rinex ne "$site${doy}0.${yy}d.Z" ) {
                $rinex=repack_rnx($rinex);
            }
            if ( ! -e "$par{RNX_DIR}/$yyyy/$doy/$rinex" ) {
               copy $rinex,"$par{RNX_DIR}/$yyyy/$doy"; 
            }
          }
          last if $ok; # leave the loop if I've got the rinex

      } # loop over sources
      
      if ( $ok && -e $rinex && $opt{unpack_rnx} ) {
        #get parts of filename
		($rname,$rext,$cext) = split /\./, $rinex ; 		
		 $unzip = $cext eq 'bz2' ? 'bzip2 -dc' : 'gzip -dc';
		 my $rnx = "$rname.$rext";
         `$unzip $rinex 1> $rnx 2> /dev/null`; 
         `crz2rnx $rnx` if $rext =~/\d{2}d/; 
          
         if ( ! -e "$rname.${yy}o" ) {
          print_log( "$rname.${yy}o not founded, fail to convert $rinex: $!\n" );
          $ok = 0;
         } 
         else {
           $rnx =~ s/d$/o/;
         }
        
         if ( $opt{gipsy_name} && $ok ) {
			my $gnx = `nametrans $rnx 2> /dev/null` ;
		    chomp $gnx;				#get rid off end of line
			print LOGFILE ("changing name: $rnx to $gnx\n");
			move( $rnx, $gnx) 
            or print_log ( "couldn't change name $rnx to $gnx: $!\n" );
         }
        
      }
     # not ok after all -> add site to the missing list
	 if ( ! $ok ) { push( @missing_sites, $stn ) } ;
      
   }# loop over sites
   my $out = @missing_sites;
   print_log ( "$out missing sites: @missing_sites\n");
   return ! $out
}

=head2

Get a RINEX file from a local directory

=cut

sub get_rnx_local
{
    my($dir,$target,$rinex) = @_;

    my $target = "$dir/$rinex";
    if ( -e $target ) {
	print_log ("copying $target to local dir \n"); 
	$ok = copy($target,$rinex);
	print_log("get_rnx couldn't copy $target: $!\n") if ! $ok;
    }

    return $ok;
}

=head2

Get a RINEX file from a remote host

=cut

sub get_rnx_remote 
{
    my($host,$password,$dir,$filename) = @_;

    print_log("Connecting to $host as user: anonymous, password: $password\n");
    my $ftp = Net::FTP->new( $host,
			     Timeout => 60, # or 15
			     Debug => 0,
			     Passive => 1 ) 
	or print_log ( "Can't connect to host $host ( $src ): $@\n" );
    if (! defined $ftp) { print_log("Error connecting to host $host\n"); return undef;}
    $ftp-> login ( "anonymous", $password ) 
	or print_log ( "Couldn't login into $host\n");
    $ftp-> binary () 
	or print_log ( "Couldn't set type to binary in $host\n");
    $ftp-> cwd ( $dir ) 
	or print_log ( "Couldn't change to dir $dir\n");
#my @rnx = grep { /^$stn$doy[a-z0]\.$yy(d|o)\.(bz2|gz|Z|zip)$/i }   $ftp->ls();
#($rinex) = @rnx > 1 ? grep { /d.(bz2|gz|Z|zip)/ } @rnx : @rnx;
    @rnx = $ftp->ls();
    ($rinex) = grep {$_ eq $filename} @rnx; 
    if ($rinex) {
        print_log( "Attempting to download $filename from $host; dir: $dir\n" );
	$ok = $ftp-> get ( $rinex ) 
	    or print_log ( "Couldn't get RINEX file: $rinex\n");
    } else {
	print_log("RINEX file $filename not found\n");
    }
    $ftp-> quit() 
	or print_log ( "Couldn't quit from $host\n");

    return $ok;
}

sub repack_rnx {
   my $rinex = shift;
   my ($rname,$rext,$cext) = split /\./, $rinex ; 		#get parts of filename
      $rname =~ s/.$/0/;
   my $rnx = "$rname.$rext";
   my $unzip = $cext eq 'bz2' ? 'bzip2 -dc': 'gzip -dc';
   print_log("Uncompressing with $unzip $rinex 1> $rnx\n");
   system("$unzip $rinex 1> $rnx"); 
   if ( $rext =~ /\d\d[oO]$/ ) {
      $rext =~ s/o$/d/i;
      system("rnx2crz $rnx");
   }
   else {
     print_log("Compressing with compress $rnx\n");
     `compress $rnx`;
     # This is the fix for using gunzip
     #`compress -S ".Z" $rnx`;
   }
   return "$rname.$rext.Z" 
}

sub get_sources {
  my @src = ();
  my @f;
  my $ftp_addresses =  "$gps_home/etc/ftp_address_source";
  open ( my $fh, "<", $ftp_addresses ) 
      or croak "couldn't open $ftp_addresses \n";
  while ( <$fh> ) {
	chomp;   		#no newline character
	s/#.*//; 		#no comments
	next if /^\s*$/;        #skip blank lines
        @f = split;  	
	push @src, shift @f;
  }
  return @src 
}

sub get_ftp_info {
  my $site    = shift;
  my $yyyy    = shift;
  my $doy     = shift;
  my $sampling = sprintf "%02d", shift;
  my $session = $sr{$sampling};
  my @sources = @_;
  my $SITE = uc $site;
  my $yy =0;

  my %ftp_info = ();

  $yy = unpack('x2a2',$yyyy);
 
  my $ftp_addresses =  "$gps_home/etc/ftp_address_source";
  
  open ( my $fh, "<", $ftp_addresses ) 
        or croak "couldn't open $ftp_addresses\n";

	while ( <$fh> ) {

		chomp;   		#no newline

		s/#.*//;		#no comments
	
		next if /^\s*$/;	#skip blank lines
	
		#replace values
		s/\$\{SITE\}/$SITE/g;
		s/\$\{site\}/$site/g;	 
		s/\$\{yyyy\}/$yyyy/g;
		s/\$\{yy\}/$yy/g;
		s/\$\{doy\}/$doy/g;
		s/\$\{sampling\}/$sampling/g;
		s/\$\{SR\}/$session/g;
		my @fields = split;
		my $src = shift @fields;
			
        push ( @{$ftp_info{$src}},@fields[0..2]);

	}
    
    return %ftp_info;

}

sub rnx_name {
    my %opt = (
      site => '',
      date => '',
      year => '',
      doy  => '',
      type => 'rinex_obs',
      comp_ext => '',
      lower => 1,
      @_,
    );
    my ($y,$doy);
    if ( $opt{site} !~  /[A-Za-z0-9]{4}/ ){ 
       croak "Not a valid site name: $opt{site}\n";
    }  
    my $site = $opt{site}; 
    
    $site = lc $site if $opt{lower};
    
    #if ( $opt{date} ) {
    #   ($y,$doy) = split ' ',scalar get_date($opt{date},'doy');
    #}
    #els
    if ( $opt{year} && $opt{doy} ) {
      $y   = $opt{year};
      $doy = $opt{doy};
    }
    else {
       croak "No date or year,doy \n"; 
    }
    
    $y   = unpack('x2a2',$y);
    
    my %rnx_type = ( rinex_obs => 'o',
                     hatanaka => 'd',
                     rinex_nav => 'n',
                   );
    
    return  sprintf("%s%03d0.%02d%s",$site,$doy,$y,
                                     $rnx_type{$opt{type}},$opt{comp_ext})
}


sub init_log {
   my $logfile = shift;
   open( our $log_fh , '>', $logfile ) or croak "can't open $logfile $@\n";
}
sub print_log  {
   my @lines = @_;
#   print  @_; 
   my $timestamp = localtime;
   print {$log_fh} "$timestamp| @_"; 
}


