package GPS::IGS;
use Exporter;
use Carp;
use GPS::DATES qw( get_date cal2sec sec2cal gps2utc jd2cal m2month );
use GPS::PRN_GPS;
use GPS::FTP::Names;
use File::Basename;
use File::Copy;
use Cwd;

@ISA= ('Exporter');

@EXPORT = qw( igs_fnames igs2jpl ) ;

sub igs_fnames {
   my %opt = ( date     => '',
               sdate    => '',
               edate    => '',
               gpsw     => '',
               type     => 'final',
               hh       => '00',
               flinn    => 0, 
               goa      => 0, 
               @_,
   );
   
   my $sdate  = $opt{sdate} ? $opt{sdate}   :
                $opt{date}  ? $opt{date}    :
                $opt{gpsw}  ? "$opt{gpsw}0" :
                              ''            ;  
   croak "No valid start date sdate=$sdate\n" if ! $sdate;
   
   #$sdate = get_date($sdate,'jd');
   
   my $edate  =  $opt{edate} ? $opt{edate}   : 
                 $opt{gpsw}  ? "$opt{gpsw}6" :
                                $sdate       ;
   croak "No valid end date edate=$edate\n" if ! $edate;
   
   #$edate = get_date($edate,'jd');
   

   my $bd   =  $sdate ;
   my $ad   =  $edate ;

   my $today = get_date('today','jd');
   my $gpsw  = get_date($today,'gpsw');
   
   
   #my $latest = 
   
   my $type = $opt{type};
   my $atype = $type;
   if ( $opt{flinn} ) {
     ($bd,$ad,$atype)=ba_flinn( sdate => $sdate,
                                edate => $edate,
                                type  => $type,
                               );
   }
   else {
     $bd = get_date($sdate,'jd');
     $ad = get_date($edate,'jd');
   }
   
   my $fnames = [];
   my $d = $bd;
   #print "b $bd a $ad\n";
   while ( $d <= $ad ) {
      $type = $atype if $d == $ad;
      #print "$type d: $d $ad ".get_date($d,'cal')." \n";
      my $f = [];
      $f = orbit_file_names(   ac   => 'igs',
                               type => $type,
                               date => $d,
                               zip  => $opt{zip},
                             );
      push @$fnames,@$f;
      $d++;
   }
   
   #print map { "$_\n" } @$fnames; 

   return wantarray ? @$fnames : $fnames   

}
sub igs2jpl  {
    my %opt = ( date   => '',
                flinn  => '',
                goa    => '',
                type   => 'final',
                latest => 0,
                eci    => 'peci',
                tdp    => 'TDPfile',
                shad   => 'Shadow',
                tpnml  => 'tpeo.nml',
                flinn_names =>  'yes',
                flinn_comp  =>  'yes',
                @_,
    );  
    $opt{date} = get_date($opt{date},'cal');
    my $date = $opt{date};
    croak "No date or invalid date $opt{date}\n" if ! $date;
    
    my $here = cwd;
    my @fnames = map { "$here/".basename($_,'.Z') } igs_fnames(%opt) ;
    my @fnames_z  =  map { "$_.Z" }  @fnames; 
    
    my ($bdate,$adate) = get_ba_dates($date);
   
    `gzip -df @fnames_z 2> /dev/null`; 
    
    my @sp3    = grep /\.sp3$/ ,@fnames; 
  
    my $eci   = "$here/$opt{eci}";
    my $tpnml = "$here/$opt{tpnml}";
    my $tdp   = "$here/$opt{tdp}";
    my $shad  = "$here/$opt{shad}";

    $ENV{TPNML} = $tpnml;
    system("tp_nml $bdate $adate 1> $tpnml 2> /dev/null");
 
    my $EpochEO = eo_epoch($date);
    eop($EpochEO);
   
    my $start =  get_date($date,'secj2000');
    my $end   =  get_date($date,'secj2000',1);
    
    if ( $opt{flinn} ) { 
      #$start -= 10800  ;  # minus 3 hours from the date at 0h 0min 0sec
      #$end   += 10800 ;  # 1 day  plus 3 hours
    }

   spx2eci( $start, $end, \@sp3, $eci );
   #spx2eci( $date, $end, \@sp3, $eci );

    #my @clk    = grep /\.clk$/,@fnames;
    my @clk    = glob( "*.clk");

    clk2tdp($start,$end,\@clk,$tdp);

    dummy_shadow($shad);
    
    system("
       tv2c.pl $tdp.$$  > $tdp
       tdp_gps_phase_ctr.pl -igs -o tdp_gps_pc
       cat tdp_gps_pc >> $tdp
      \\rm -f $tdp.$$ @fnames
   ");
    
    if ( $opt{flinn_names} ) {
       my $cal = get_date($date,'cal');
       move $eci, "$here/$cal.eci";
       move $shad, "$here/$cal.shad";
       move $tdp , "$here/$cal.tdpc";
       move $tpnml, "$here/${cal}tpeo.nml";
       my @fflin = ( "$here/$cal.eci",
                     "$here/$cal.shad",
                     "$here/$cal.tdpc",
                     "$here/${cal}tpeo.nml"
                   );
     `compress -f @fflinn` if $opt{flinn_comp};
    }
     
}

sub spx2eci {
    my $start = shift ; # get_date($cal,'secj2000',-0.125);
    my $end   = shift || $start; # get_date($cal,'secj2000',1.125);
    my $sp3   = shift;
    my $eci   = shift || 'peci';
    my $here   = cwd;   

    my $tmp_dir = "$here/spx2eci_tmp.$$";

    mkdir $tmp_dir or die "Coulnd't make $tmp_dir $!\n";

    chdir $tmp_dir or die "$!\n";

    `spx2eci @$sp3 2> /dev/null`;

    filter_eci_files($start,$end); 

    `cat gps*.eci > $eci 2> /dev/null`;

    `rm -rf $tmp_dir`;

     chdir $here ;
}

sub ba_flinn {
   my %opt = (
       sdate   => '',
       edate   => '',
       type    => '',
       latest  => 0,
       gwdelay => 2,
       @_,
   );
   my $cal   = get_date('today','cal');
   my $yesterday = get_date($cal,'jd',-1) ;
   my $gpsw  = get_date($cal,'gpsw') - $opt{gwdelay} ;
   my $bd    = get_date($opt{sdate},'jd');
   my $ad    = get_date($opt{edate},'jd');
   my $gad   = get_date($opt{edate},'gpsw');
   my $atype  = $opt{type};
   $bd--; 
   $ad++; 
   $opt{latest} =  $atype eq 'final' &&  $gad <   $gpsw      ?  0 : 
                   $atype eq 'final' &&  $gad ==  $gpsw      ?  1 : 
                   $atype eq 'final' &&  $gad >   $gpsw      ? -1 : 
                   $atype eq 'rapid' &&  $ad  <   $yesterday ?  0 : 
                   $atype eq 'rapid' &&  $ad  ==  $yesterday ?  1 : 
                                                               -1 ;
                   
   if ( $opt{latest} < 0 ) {
       $ad--;
       #carp "No all products available for igs to flinn conversion\n";
   }
   if ( $opt{latest} >= 0 ) {
        $atype = 'rapid' if $atype eq 'final' ;
   }
   
   return ($bd,$ad,$atype)
   
}

sub dummy_shadow{
  my $f = shift;              # name of shadow file
  open(F, ">$f") or die "Can open $f";
  print F  "                                      \n"; # enough blanks for compress
  print F "NUMBER OF GPS SATELLITES IN ECLIPSE: 0\n";
  close F;
}

sub eo_epoch {
  my $cal = shift;
  my $start = get_date($cal,'secj2000',-0.125);
  my $end   = get_date($cal,'secj2000',1.125);
  my $eo = ($start+$end)/2;
  $eo = `gps2utc $eo`; chomp $eo; 
  my $d = sprintf "%4d-%02d-%02d:%02d:%02d:%7.5f",sec2cal($eo );
  return get_date($d,'eop')
}

sub eop {
  my $EpochEO = shift;
  open(my $fh, ">> tpeo.nml") or die "Couldn't open tpeo.nml for writing $!\n";
  my $eo ="\n";
  $eo .= '$earth_orientation'."\n";
  $eo .= qq(
 !        Received from Jim Ray on 10/18/2000
 !

    EpochEO = \"$EpochEO\"
    Extended_UT1_model= .true.
    Extended_XPM_model= .true.
    Extended_YPM_model= .true.
! Ext. UT1 parameter nominals (units = seconds of time)    ->     IERS
  UMD0 =  -7.20e-6,  16.17e-6,    ! U M2 COS ,  U M2 SIN        COS, -SIN
          -0.04e-6,   7.59e-6,    ! U S2 COS ,  U S2 SIN        COS, -SIN
          -1.54e-6,   3.80e-6,    ! U N2 COS ,  U N2 SIN        COS, -SIN
          -0.38e-6,   1.96e-6,    ! U K2 COS ,  U K2 SIN        COS, -SIN
           8.64e-6,  17.71e-6,    ! U K1 COS ,  U K1 SIN        SIN,  COS
         -12.10e-6, -16.05e-6,    ! U O1 COS ,  U O1 SIN       -SIN, -COS
          -2.86e-6,  -5.16e-6,    ! U P1 COS ,  U P1 SIN       -SIN, -COS
          -2.45e-6,  -5.03e-6     ! U Q1 COS ,  U Q1 SIN       -SIN, -COS
! Ext. XPM parameter nominals (units = radians)
  XMD0 = -.1357e-09,     1.6000e-09,      ! X M2 COS ,  X M2 SIN
          .3103e-09,      .7030e-09,      ! X S2 COS ,  X S2 SIN
         -.0630e-09,      .2763e-09,      ! X N2 COS ,  X N2 SIN
          .0824e-09,      .1745e-09,      ! X K2 COS ,  X K2 SIN
         -.7369e-09,      .3782e-09,      ! X K1 COS ,  X K1 SIN
          .6448e-09,     -.2376e-09,      ! X O1 COS ,  X O1 SIN
          .2424e-09,     -.1212e-09,      ! X P1 COS ,  X P1 SIN
          .1261e-09,     -.0291e-09       ! X Q1 COS ,  X Q1 SIN
! Ext. YPM parameter nominals (units = radians)
  YMD0 =  .9502e-09,     -.1794e-09,      ! Y M2 COS ,  Y M2 SIN
          .4218e-09,     -.2860e-09,      ! Y S2 COS ,  Y S2 SIN
          .1600e-09,     -.0533e-09,      ! Y N2 COS ,  Y N2 SIN
          .1067e-09,     -.0873e-09,      ! Y K2 COS ,  Y K2 SIN
         -.3782e-09,     -.7369e-09,      ! Y K1 COS ,  Y K1 SIN
          .2376e-09,      .6448e-09,      ! Y O1 COS ,  Y O1 SIN
          .1212e-09,      .2424e-09,      ! Y P1 COS ,  Y P1 SIN
          .0291e-09,      .1261e-09       ! Y Q1 COS ,  Y Q1 SIN

);
  $eo .= '$end'."\n";


 print $fh $eo;  

}



sub filter_eci_files {
    #my $cal = get_date( shift, 'cal');
    #my $start =  get_date($cal,'secj2000',-0.125);
    #my $end   =  get_date($cal,'secj2000',1.125);
    my $start = shift ; # get_date($cal,'secj2000',-0.125);
    my $end   = shift || $start; # get_date($cal,'secj2000',1.125);
    #my $start   = shift;        # GPS sec past J2000 take all records >=
    #my $end     = shift;        # GPS sec past J2000 take all records <=

    my $tmp_eci = "eci$$";
    unlink $tmp_eci if(-e $tmp_eci);

    #my $st_utc = `gps2utc $start`; chomp $st_utc;
    #my $end_utc = `gps2utc $end`; chomp $end_utc;
    my $st_utc =  gps2utc( $start);
    my $end_utc = gps2utc($end);
    
    foreach my $g ( glob "gps*.eci" ){
        open( T, ">$tmp_eci");
        open( G, "<$g" ) or die "Unable to open $g";
        while(<G>){
            my @l = split(' ');
            # fix GPS32 for now
            #next if $l[0] == 32 || $l[0] eq '00';
            next if $l[0] eq '00';
            my $t = cal2sec(@l[1..6]);
            print T $_ if( $t >= $st_utc && $t <= $end_utc );
        }
        close T;
        close G;
        rename $tmp_eci, $g or die "Unable to rename $tmp_eci to $g ";
    }
}
sub clk2tdp_jpl {
    my $start = shift; 
    my $end   = shift; 
    my $clk_files = shift;
    my $tdp  = shift || 'TDPfile';
    my $sl = 299792.458;
    my $pg = GPS::PRN_GPS->new();
    open(my $th , "> $tdp.$$") or croak "Couldn't open $tdp $!\n";    
    foreach my $clk ( @$clk_files ) { 
      $clk =~ s/\.Z//;
      open(my $ch , "< $clk") or croak "Couldn't open clock file $clk $!\n";
      while ( <$ch> ) {
        next if ! /^AS G(\d\d)/ ;
        my $prn = $1;
        $prn =~ s/^0//;
        my @l = split ' ',$_;
        my $clk_bias = $l[9]*$sl;
        my $sig      = $l[10]*$sl;
        my $secd = cal2sec(@l[2..7]);
        next unless( $secd >= $start && $secd <= $end );
        my $gps = $pg->gps( prn =>$prn , secj2000 => $secd);
        my $name = sprintf "GPS%02d",$gps;
        printf $th  "%15.4f %17.10E  %14.9f    %10.3E SAT BIAS%s\n",
                    $secd,0,$clk_bias,$sig,$name;
     }
     
     close($ch);
   }
   
}

sub clk2tdp {
    my $start = shift; 
    my $end   = shift; 
    my $clk_files = shift;
    my $tdp  = shift || 'TDPfile';
    my $sl = 299792.458;
    my $pg = GPS::PRN_GPS->new();
    my %TDP = ();
    open(my $th , "> $tdp.$$") or croak "Couldn't open $tdp $!\n";    
    foreach my $clk ( @$clk_files ) { 
      $clk =~ s/\.Z//;
      open(my $ch , "< $clk") or croak "Couldn't open clock file $clk $!\n";
      while ( <$ch> ) {
        next if ! /^AS G(\d\d)/ ;
        my $prn = $1;
        $prn =~ s/^0//;
        my @l = split ' ',$_;
        my $clk_bias = $l[9]*$sl;
        my $sig      = $l[10]*$sl;
        my $secd = cal2sec(@l[2..7]);
        #my $secd = `cal2sec @l[2..7] 2> /dev/null`;
        next unless( $secd >= $start && $secd <= $end );
        my $gps = $pg->gps( prn =>$prn , secj2000 => $secd);
        next if $gps eq '00';
        my $name = sprintf "GPS%02d",$gps;
        push @{$TDP{$name}}, sprintf( "%15.4f %17.10E  %14.9f    %10.3E SAT BIAS%s\n", $secd,0,$clk_bias,$sig,$name);
     }
     close($ch);
   }
   my %seen;
   foreach my $s ( sort keys %TDP ) {
        print $th ( grep { ! $seen{$_}++ } sort @{$TDP{$s}} );
   } 
}
sub get_ba_dates{
    my $jd = get_date(shift,'jd');
    my ($by,$bm,$bd) = jd2cal($jd - 10);
    my ($ay,$am,$ad) = jd2cal($jd + 10);
    my $bmm = m2month($bm); 
    my $amm = m2month($am); 
    #my $b = sprintf("%02d-%3s-%4d",$bd,$bmm,$by);
    #my $a = sprintf("%02d-%3s-%4d",$ad,$amm,$ay);
    my $b = sprintf("%02d-%3s-%4d",$bd,$bmm,$by);
    my $a = sprintf("%02d-%3s-%4d",$ad,$amm,$ay);
    return ( $b,$a)
    #return ( jd2cal($jd-10) , jd2cal($jd +10 ) )
}

1;
