package GPS::IGS::Sum;
use Carp;
use File::Basename;
use GOA::Util;
use GPS::PRN_GPS;
use GPS::DATES;

my $pg = GPS::PRN_GPS->new();
sub new { 
    my $class = shift;
    my $self = { file => '',
                 sats => {},
                 date => '',
               };
    $self->{file} = shift if @_;
    bless $self,$class;       
    $self->load();
    return $self
}

sub file {
  my $self = shift;
  if ( @_ ) {
    $self->{file} = shift;
  }
  return $self->{file}
}

sub sats {
  my $self = shift;
  my $date = @_ && $_[0] ? get_date(shift,'gwd') : '';
  my (@sats,$h);
  foreach my $d ( sort keys %{$self->{sats}} ) {
     next if $date && $date ne $d;
     foreach my $h ( @{$self->{sats}{$d}} ) {
       push @sats, $h;
     }
  }
  return wantarray ? @sats : \@sats
}

sub load {
   my $self = shift;
   my $file = $self->file(@_);
   my ($prn,$gps,$date,$week,$d,$acs);
   my $ftype = unpack ('a3',basename($file));
   my $tn = $ftype eq 'igr' ? 2 :
            $ftype eq 'igs' ? 3 : 
            $ftype eq 'ig1' ? 3 : croak "Invalid file $file\n";
   open(my $fh,'<',$file) or croak "Couldn't open $file $!\n";
   my $o=0;
   my $flags='';
   while(<$fh> ) {
     if ( /Table $tn\.(\d+)\.(\d) / ) {
       $week = $1;
       $d=$2;
       $date = "$1$2";;
     }
     next if ! $date;
     next if ! $date && ! /^ PRN /;
     next if /^ PRN /;
     if ( /^-+/ ) {
       $o= (1 xor $o);
       $date = '' if $o == 0;
       next;
     }
     if ( /(.*?)\|(.*?)\|/ ) {
       $prn = trim($1);
       $acs = trim($2);
       $flags='';
       while( $prn =~ /(X|E|M)/xmsg ) {
         my $f = $1; 
         $flags.=$f;
         $prn =~ s/$f//g;
       }
       my $rms = (split (' ',$acs))[-1];
       $gps = $pg->gps( prn => $prn, date => $date  );
       
       push @{$self->{sats}{$date}},{ prn => $prn, 
                                      gps => $gps , 
                                      flags => $flags,
                                      rms => $rms,
                                     };
       #print "$prn $gps $rms $date $flags \n"; #$acs\n";
     }
   }
}

1;
