package GPS::JPL;

use Exporter;
use Carp;
use GPS::DATES;
use GPS::FTP::Names;
use File::Copy;
use File::Basename;
use Cwd;

@ISA= ('Exporter');

@EXPORT = qw( jpl_fnames jpl_unpack ) ;

sub jpl_fnames {
    my %opt = (  date     => '',
                 sdate    => '',
                 edate    => '',
                 type     => 'final',
                 gpsw     => '',
                 frame    => 'itrf00',
                 @_, 
       );

     my $sdate  = $opt{sdate} ? $opt{sdate}   :
                  $opt{date}  ? $opt{date}    :
                  $opt{gpsw}  ? "$opt{gpsw}0" :
                                ''            ;
    croak "No valid start date sdate=$sdate\n" if ! $sdate;
  
    my $edate  =  $opt{edate} ? $opt{edate}   :
                  $opt{gpsw}  ? "$opt{gpsw}6" :
                                $sdate        ;
    
    croak "No valid end date edate=$edate\n" if ! $edate;

    my $type = $opt{type}; 
    my $frame = $opt{frame};
    my $d   =  get_date($sdate,'jd');
    my $ed  =  get_date($edate,'jd');


    my $fnames = [];
    while ( $d <= $ed ) {
        my $f = [];
        my $x = [];
        ($f,$x) = orbit_file_names( ac   => 'jpl',
                                    type => $type,
                                    date => $d,
                                    frame => $frame,
                                    zip  => $opt{zip},                                                                  );
        push @$fnames,@$f;
        push @$fnames,@$x;
        $d++;
    }    
    
    return wantarray ? @$fnames : $fnames
    
}

sub jpl_unpack {
   my %opt = ( date   => '',
               type   => 'final',
               eci    => 'peci',
               tdp    => 'TDPfile',
               shad   => 'Shadow',
               tpnml  => 'tpeo.nml',
               yaw    => 'TDPyaw',
               xfile  => 'XFILE',
	       wlpb   => 'wlpb',
               @_,
             );

    my $date = get_date($opt{date},'cal');
    croak "No date or invalid date $opt{date}\n" if ! $date;
     
    my $type = $opt{type};
    my $frame = $opt{frame};

    my $here = cwd;
    my @fnames = map { "$here/".basename($_) }
                 jpl_fnames( date  => $date, 
                             type  => $type,
                             frame => $frame,
                           );
    my @fnames_z  =  map { "$_.Z" } grep { ! /\.x/ } @fnames;
    
    my ($x) =  map { glob($_) } qw/*.x *.qx *.x.Z *.qx.Z / ;   
    
    push @fnames_z, $x;
    
    `gzip -df @fnames_z 2> /dev/null `;

    my $eci   = "$here/$opt{eci}";
    my $tpnml = "$here/$opt{tpnml}";
    my $tdp   = "$here/$opt{tdp}";
    my $shad  = "$here/$opt{shad}";
    my $yaw   = "$here/$opt{yaw}";
    my $xf    = "$here/$opt{xfile}";
    my $wlpb  = "$here/$opt{wlpb}";
     
    if ( $type eq 'rapid' ) {
       foreach my $f ( @fnames ) {
           move $f,$shad       if $f =~ /shadow_events/;
           move $f,"$tdp.tmp"  if $f =~ /gps_clocks/;
           move $f,$yaw        if $f =~ /yaw_rates/;
           move $f,$tpnml      if $f =~ /TPNML/;
           move $f,$eci        if $f =~ /eci/;
       }
       system("
               yaw_rates2tdp $yaw |tv2c - >> $tdp.tmp
               sort -T /tmp -n $tdp.tmp > $tdp
               rm -f $tdp.tmp $yaw
      ");
    }
    else {
       foreach my $f ( @fnames ) {
           move $f,$shad   if $f =~ /shad/;
           move $f,$tdp    if $f =~ /tdpc/;
           move $f,$tpnml  if $f =~ /tpeo.*\.nml/;
           move $f,$eci    if $f =~ /eci/;
           move $f,$xf     if $f =~ /x/;
	   move $f,$wlpb   if $f =~ /wlpb/;
       }
    }
    `eci_filter $date $eci`; 
}



1;





