package GPS::PCV;
use Carp;
use Cwd;
use File::Basename;

our @FIELDS = qw/l1_offset l2_offset l1_deg l2_deg 
                old_code goa_name vendor
                description comment 
                zen1 zen2 dzen dazi
               /;
our %valid_field;

@valid_field{@FIELDS}=(1)x@FIELDS;

sub new {
  my $class = shift;
  my $self =  {};

  bless $self,$class;

  return $self->load(@_)
}

sub load {
  my $self = shift;
  my $pcv_file =  shift || '';
  my %fmt = ( igs => "a15a20a25",
              ngs => "a20a40x1a4x2a3x2a2x1a2x1a2"
             );
  my %fields = ( igs => [ 'vendor','igs_code','description' ],
                 ngs => [ qw(igs_code description data_src ntests yy mm dd )] 
             );
  my @pcvs = split(/\,/,$pcv_file);
  my $type;
		  
 foreach my $file ( @pcvs ) {	 
    $type = "ngs" if $file =~ /ant_info/;
    $type = "igs" if $file =~ /igs_.*pcv$/;
    open(my $fh,'<', $file) or croak "couldn't open file: $file for reading \n";
    #skip header
    my $i=0;
    while ( <$fh> ) {
      $i++ if /\| Elevation/;	 
      last if $i > 1;  
    }
    my $old_name='';
    while ( <$fh> ) {
       s/^\s+//;   
       s/\s+$//;   
       next unless length;
       my $p={ vendor =>   '' , 
               goa_name => '' ,
               zen1     => 0.0,
               zen2     => 90.0,
               dzen     => 5,
               dazi     => 0.0,
             };
       
       @$p{ @{$fields{$type}} }=unpack($fmt{$type},$_);
       my $a = uc $p->{igs_code};
       $p->{igs_code} = $a eq  'NONE'.' 'x12 .'    ' ? ' 'x16 .'NONE' :
                        $a eq  'NONE'.' 'x12 .'NONE' ? ' 'x16 .'NONE' : 
                                                       $a;
       $p->{igs_code} =~ s/    $/NONE/;
       if ( exists $self->{$p->{igs_code}} ) { 
	   for ( 1..6 ) { readline($fh) };
	   next;
       }
       if ( $p->{description} =~ /OLD NAME/ ) {
           $old_name = $p->{igs_code};
	   for ( 1..6 ) { readline($fh) };
	   next;
       }
       
#       $p->{vendor} = $self->get_vendor( igs_code => $p->{igs_code},
#                                         vendor   => $p->{vendor},
#				         file     => $file );
       
       $p->{old_code}= $old_name if $old_name;
       $old_name='';
        
       $p->{l1_offset}= [ split(' ',readline($fh)) ];
       my $l = readline($fh);
       $l.=readline($fh);
       $p->{l1_deg}= [ split(' ',$l) ]; 
       $p->{l2_offset}= [ split(' ',readline($fh)) ];
       my $l = readline($fh);
       $l.=readline($fh);
       $p->{l2_deg}=[ split(' ',$l) ]; 
       $p->{comment} = pack("A16",basename($file));
#       $p->{goa_name} = $self->goa_name( igs_code => $p->{igs_code},
#                                         vendor   => $p->{vendor} );
       $self->{ $p->{igs_code} } = $p ;
       #print "$p->{igs_code} | $p->{old_code}\n";       
    }
 }
   return $self 

}
sub list {
   my $self = shift;
   foreach my $a ( sort keys %$self ) {
        $a = $a eq  'NONE'.' 'x12 .'    ' ? ' 'x16 .'NONE' :
             $a eq  'NONE'.' 'x12 .'NONE' ? ' 'x16 .'NONE' : 
                                            $a;
        push @list, $a;
   }
   return wantarray ? @list : \@list;
}

sub set  {
   my $self = shift;
   my %opt = (
      igs_code => '',
      field    => '',
      value    => '',
      @_,      
   );
   return $self->get(%opt)
}

sub get  {
   my $self = shift;
   my %opt = (
      igs_code => '',
      field    => '',
      @_,      
   );
   my $igs_code = uc $opt{igs_code};
   if ( ! $igs_code ) {
     carp "No igs antenna code !\nuse igs_code => ant_code\n";
     return ''
   }
   if ( length $igs_code != 20 ) {
     carp "IGS code |$igs_code| doesn't have 20 characters\n";
     return ''
   }
   if ( ! exists $self->{$igs_code} ) {
     carp "igs_code |$igs_code| not found\n";
     return ''
   }

   my $field = lc $opt{field};
   if ( ! $valid_field{$field} ) {
     carp "Field |$field| is not a valid field\n".
           "use field => field_name\n".
           "Valid Fields:@FIELDS\n" ;
     return ''
   }

   if ( exists $opt{value} ) {
      $self->{ $igs_code }{ $field } = $opt{value} if $opt{value};
   }

   return $self->{ $igs_code }{ $field }
   
}
sub vals {
  my $self = shift;
  my %opt = ( antenna   => '',
              freq_code => '',
              @_,
            );
  
   my $fc =  unpack 'a2', lc $opt{freq_code};
   if ( $fc !~ /^(l1|l2)$/ ) {
      carp "Invalid frequency code |$opt{freq_code}|\n".
           "use L1 || L2 \n";
      return ''
   } 
   return $self->get( igs_code => $opt{antenna},
                      field    => "${fc}_deg",
                    )
}
sub trim {
 my $s = shift;
    $s =~ s/^\s+//;
    $s =~ s/\s+$//;
 return $s
}


1;

