package GPS::PRN_GPS;
use GPS::DATES qw( get_date sec2cal);
use Carp;
sub new {
   my $class = shift;
   my %opt   = @_;
   my $self  = {};
   bless $self,$class;
   return $self->init( %opt )
}
sub init {
  my $self = shift;
  my %opt  = (
     prn_gps       =>  -e "$ENV{GOA_ETC}/PRN_GPS" ? "$ENV{GOA_ETC}/PRN_GPS" : "$ENV{GOA_VAR}/etc/PRN_GPS",
     @_,
  );
  $self->{prn_gps} = $opt{prn_gps};
  open(my $fh , '<', $self->{prn_gps} ) 
        or croak "Couldn't open $self->{prn_gps} $!\n";
  my $pg = [];
  my @fields = qw/launch deactiv gps prn block /;
  my @v;
  while ( <$fh> ) {
     next if $. == 1;
     @v = split ' ', $_;
     next if $v[4] eq 'GENRC';
     my $h = {};
     $v[1] = '3000-01-01' if  $v[1] eq '0000';
     $v[0] =~ s/-//g; 
     $v[1] =~ s/-//g; 
     @$h{@fields}=@v[0..4];
     push @$pg,$h;
  }
  $self->{pg_table} = $pg;
  return $self
}

sub get_entry {
   my $self = shift;
   my %opt = (
     prn        => '',
     gps       => '',
     secj2000  =>  0,
     date       => '',
     @_,
   );
   my ($prn,$gps)=(0,0);
   
   if ( $opt{prn} ) {
     $opt{prn} =~ s/^PRN//i; $opt{prn} =~ s/^0+//;
     $prn = sprintf("%02d",$opt{prn});
   }
   if ( $opt{gps} ) {
     $opt{gps} =~ s/^(G|GPS)//i;  $opt{gps} =~ s/^0+//;
     $gps = sprintf("%02d",$opt{gps});
   }
   if ( ! $gps && ! $prn )  {
     croak "You must supply prn/gps number\n";
   }
   my $time = $opt{secj2000} ? sec2cal($opt{secj2000},'cal')        :
              $opt{date}     ? get_date($opt{date},'cal') :
                               '' ;
   my $pg={ prn => '00' , gps => '00' , date => $time };
   $time =~ s/-//g;
   if ( ! $time ) {
     croak "You must supply a time: date => mydate || secj2000 => ssss.ss\n";
   }
   foreach my $h ( sort { $a->{gps} <=> $b->{gps} } @{ $self->{pg_table} } ) {
      #last if $h->{gps} > 61;
      last if $h->{block} !~ /^I*/;
      next if $prn && $h->{prn} != $prn;
      next if $gps && $h->{gps} != $gps;
      next if $time < $h->{launch} || $time > $h->{deactiv} ;
      $pg = $h;
      last;
   }
    
   return $pg 
}

sub block {
   my $self = shift;
   my %opt = (
     prn       => '',
     gps       => '',
     secj2000  => '',
     date      => 'today',
     @_,
   );
   
   croak "No PRN/GPS number supplied !\n" if ! $opt{prn} && ! $opt{gps}; 
   my $pg = $self->get_entry( %opt );

   return $pg->{block}
   
}
sub gps {
   my $self = shift;
   my %opt = (
     prn       => '',
     secj2000  => '',
     date      => 'today',
     @_,
   );
   croak "No PRN number supplied !\n" if ! $opt{prn}; 
   my $pg = $self->get_entry( %opt );
   if ( $pg->{gps} eq '00' ) {
     carp "No translation for PRN $opt{prn} for $pg->{date}\n";
   }

   return $pg->{gps}
   
}
sub prn {
   my $self = shift;
   my %opt = (
     gps       => '',
     secj2000  => '',
     date      => 'today',
     @_,
   );
   
   croak "No GPS number supplied !\n" if ! $opt{gps}; 
   my $pg = $self->get_entry( %opt );
   if ( $pg->{prn} eq '00' ) {
     carp "No translation for GPS $opt{gps} for $pg->{date}\n";
   }
   return $pg->{prn}
   
}

1;
