package GPS::SINEX;
use Fortran::F90Format;
use Carp;

our $AUTOLOAD;
my %bname = (  file_reference             => 'FILE/REFERENCE',
               file_reference_description => 'FILE/REFERENCE',
               file_reference_output      => 'FILE/REFERENCE',
               file_reference_contact     => 'FILE/REFERENCE',
               file_reference_hardware    => 'FILE/REFERENCE',
               file_reference_software    => 'FILE/REFERENCE',
               file_reference_input       => 'FILE/REFERENCE',
               file_comment   => 'FILE/COMMENT',
               input_history  => 'INPUT/HISTORY',
               input_files    => 'INPUT/FILES',
               input_acknowledgements => 'INPUT/ACKNOWLEDGMENTS',
               nutation_data   => 'NUTATION/DATA',
               precession_data =>  'PRECESSION/DATA',
               source_id       =>  'SOURCE/ID',
               site_id         =>  'SITE/ID',
               site_data       =>  'SITE/DATA',
               site_receiver   =>  'SITE/RECEIVER',
               site_antenna    =>  'SITE/ANTENNA',
               site_gps_phase_center =>  'SITE/GPS_PHASE_CENTER',
               site_eccentricity     =>  'SITE/ECCENTRICITY',
               bias_epochs           =>   'BIAS/EPOCHS',
               solution_discontinuity =>  'SOLUTION/DISCONTINUITY',
               solution_epochs       =>   'SOLUTION/EPOCHS',
               solution_statistics   =>   'SOLUTION/STATISTICS',
               solution_estimate     =>   'SOLUTION/ESTIMATE',
               solution_apriori      =>   'SOLUTION/APRIORI',
               solution_matrix_estimate_l_corr => 'SOLUTION/MATRIX_ESTIMATE L CORR',
               solution_matrix_estimate_l_cova => 'SOLUTION/MATRIX_ESTIMATE L COVA',
               solution_matrix_estimate_l_info => 'SOLUTION/MATRIX_ESTIMATE L INFO',
               solution_matrix_estimate_u_corr => 'SOLUTION/MATRIX_ESTIMATE U CORR',
               solution_matrix_estimate_u_cova => 'SOLUTION/MATRIX_ESTIMATE U COVA',
               solution_matrix_estimate_u_info => 'SOLUTION/MATRIX_ESTIMATE U INFO',
               solution_matrix_apriori_l_corr => 'SOLUTION/MATRIX_APRIORI L CORR',
               solution_matrix_apriori_l_cova => 'SOLUTION/MATRIX_APRIORI L COVA',
               solution_matrix_apriori_l_info => 'SOLUTION/MATRIX_APRIORI L INFO',
               solution_matrix_apriori_u_corr => 'SOLUTION/MATRIX_APRIORI U CORR',
               solution_matrix_apriori_u_cova => 'SOLUTION/MATRIX_APRIORI U COVA',
               solution_matrix_apriori_u_info => 'SOLUTION/MATRIX_APRIORI U INFO',
               solution_normal_equation_vector => 'SOLUTION/NORMAL_EQUATION_VECTOR',  
               solution_normal_equation_matrix_l => 'SOLUTION/NORMAL_EQUATION_MATRIX L',
               solution_normal_equation_matrix_u => 'SOLUTION/NORMAL_EQUATION_MATRIX U'
			 );
my %conf_name = reverse %bname;

my @default_blocks = qw( 
                         file_comment
						 site_id
						 site_receiver
						 site_antenna
						 site_gps_phase_center
						 site_eccentricity
                       );

my @Blocks=@default_blocks;

sub new {
  my $class = shift;
  my %opt   = ( file => '', 
               @_
			   );
  my $self  = { };
  
  bless $self,$class;
  
  $self->config;
  
  foreach my $k ( keys %{$self} ) {
     $self->{$k}{fmt} = Fortran::F90Format->new( fmt => $self->{$k}{fmt_str} );  
  }
  
  if ( $opt{file} ) {
     open( my $fh, '<', $opt{file} ) or croak "Couldn't open $opt{file} $!\n";
     $self->{fh} = $fh;
   }
   else {
     $self->{fh} = \*STDOUT;
   }
   return $self
}

sub _AUTOLOAD {
  my $self = shift;
  #my $type = ref $self or croak "$self is not an object\n";
  #my $type = ref $self || $self;
  my $block = $AUTOLOAD;
  $block =~ s/.*:://;   # strip fully-qualified portion
  if ( ! exists $self->{$block} ) {
    croak "Can't access $block field in class type \n";
  }
  my @fields = @{$self->{$block}{fields}};
  if ( @_ == 1 ) {
    my $line = shift; chomp $line;
    $line = sprintf("% -80s",$line);
    @{$self->{$block}}{@fields} = $self->{$block}{fmt}->read($line);
  }
  elsif ( ! ref($_[0]) && @_ ) {
    my %input = @_;
    foreach my $f ( @fields ) {
      delete $input{$f} if ! exists $self->{$block}{$f};
    }
    if ( %input ) {
     return @{$self->{$block}}{keys %input} = values %input;
    }
    else { carp "No valid fields found for $block \n" }
  }
  else {
     my @values = @{$self->{$block}}{@fields};
     return $self->{$block}{fmt}->write(@values)
  }
   
}

sub bname {
  my $self = shift;
  croak "Empty block name as input\n" if ! @_;
  my $block = $_[0] =~ /[A-Z]/ && exists $conf_name{$_[0]} ? $conf_name{$_[0]} : shift;
  croak "$block is not a valid SINEX block name\n" if ! exists $self->{$block};
  return $block  
}

sub fmt {
  my $self = shift;
  
  my $block = $self->bname(@_);

  return $self->{$block}{fmt}
}

sub fields {
  my $self = shift;
  my $block = $self->bname(@_);
  my @f = @{$self->{$block}{fields}};
  return wantarray ? @f : \@f  
}

sub AUTOLOAD {
  my $self = shift;
  #my $type = ref $self or croak "$self is not an object\n";
  #my $type = ref $self || $self;
  my $block = $AUTOLOAD;
  $block =~ s/.*:://;   # strip fully-qualified portion
  if ( ! exists $self->{$block} ) {
    croak "Can't access $block field in class type \n";
  }
  my @fields = $self->fields($block);
  my @values = ();
  if ( @_ == 1 ) {
    my $line = shift; chomp $line;
    $line = sprintf("% -80s",$line);
    @{$self->{$block}}{@fields} = $self->fmt($block)->read($line);
  }
  elsif ( @_ ) {
    my %input = @_;
    foreach my $f ( @fields ) {
      delete $input{$f} if ! exists $self->{$block}{$f};
    }
    if ( %input ) {
      @{$self->{$block}}{keys %input} = values %input;
    }
    else { carp "No valid fields found for $block \n" }
  }
  
  @values = @{$self->{$block}}{@fields};
  
  return wantarray ? @values : $self->fmt($block)->write(@values)
   
}

sub print {
   my $self = shift;
   my @blocks = @_;
    @blocks = ! @_ ?  @default_blocks : (@default_blocks,@_);
    my $fh = $self->{fh};
    my (@fields,@values);
    
	print $fh $self->header;
	foreach my $b ( @blocks ) {
	  next if ! exists $self->{$b};
	  print $fh "+$bname{$b}\n";
	    @fields = @{$self->{$b}{fields}};
	    @values = @{$self->{$b}}{@fields};
        print $fh $self->{$b}{fmt}->write(@values);      
	  print $fh "-$bname{$b}\n";
    }
    print $fh $self->footer;
}
sub print_config {
   my $self = shift;
   my $fh = \*DATA;
   seek($fh,0,0);
   my $flag = 0;
   while(<$fh>) {
     if ( /__DATA__/ ) {
	   $flag++;
	   next;
	 }
     last if /\[END\]/i;
     print $_ if $flag > 1;
   }
}
sub config {
 my $self = shift;
 my $fh = \*DATA;
 my ($var,$flag);
 $flag=0;
 while ( <$fh> ) {
   chomp;                  # no newline
   s/#.*//;                # no comments
   s/^\s+//;               # no leading white
   s/\s+$//;               # no trailing white
   next unless length;     # anything left?
   #$_ = lc $_;
   last if /\[END\]/i;
   if ( /^\[(.*)\]/ ) {
      $var =  $1;
      $self->{$var}={};
	  push @Blocks,$var;
      next;
   }
   ($hvar, $hvalue) = split(/\s*=\s*/, $_, 2);
   $hvalue =~ s/^('|")//;
   $hvalue =~ s/('|")$//;
   push @{$self->{$var}{fields}},$hvar if $hvar ne 'fmt_str';
   $self->{$var}{$hvar}=$hvalue;
   #print " $var $hvar = $self->{$var}{$hvar} \n";
 }
 return $self
}

1;

__DATA__
[comment]
  fmt_str = 1H*,A79
  comment = 
[header] # mandatory
  fmt_str =  A1,A1,A3,1X,F4.2,1X,A3,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A3,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A1,1X,I5.5,1X,A1,6(1X,A1)
  c1                  =  %
  c2                  =  =
  doc_type            =  SNX
  version             =  2.01
  file_agency_code    =  CWU
  ct_yy               =  0
  ct_doy              =  0
  ct_sec              =  0
  agency_code         =  CWU
  st_yy               =  0  
  st_doy              =  0  
  st_sec              =  0
  et_yy               =  0
  et_doy              =  0
  et_sec              =  0
  obs_code            =  P
  num_estimates       =  0   
  constr_code         =  1
  sol_content_1       =  S  #S, O, E, T, C, A
  sol_content_2       =     
  sol_content_3       =  
  sol_content_4       =  
  sol_content_5       =  
  sol_content_6       =  

[footer]  # mandatory
  fmt_str       =  A7
  end_sinex     = %ENDSNX

[file_reference] # Mandatory
  fmt_str = 1X,A18,1X,A60
  
  info_type = DESCRIPTION
#  info_type = OUTPUT
#  info_type = CONTACT
#  info_type = SOFTWARE
#  info_type = HARDWARE
#  info_type = INPUT

  info      =   

[file_comment] # optional
  fmt_str = 1X,A79
  comment =  

# time fmt: 1X,I2.2,1H:,I3.3,1H:,I5.5

[input_history] # recommended 
  fmt_str = 1X,A1,A3,1X,F4.2,1X,A3,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A1,1X,I5.5,1X,A1,5(1X,A1)
  file_code      =  =    # '=' for output solution file
                         # '+' for file contributing to sinex file
  document_type  =  SNX
  format_version =  2.01
  ct_yy          =  0  
  ct_doy         =  0  
  ct_sec         =  0
  agency_code    =  CWU
  st_yy          =  0  
  st_doy         =  0  
  st_sec         =  0
  et_yy          =  0
  et_doy         =  0
  et_sec         =  0
  obs_tech       =  P
  num_estimates  =  0
  constr_code    =  0
  sol_cont_stations = S #S  all station parameters: coord, vel.,biases,geocenter 
  sol_cont_orbits   =   #O  Orbits
  sol_cont_eop      =   #E  Earth Orientation Parameter
  sol_cont_tropo    =   #T  troposphere
  sol_cont_crf      =   #C  celestial ref. frame

[input_files] # optional
  fmt_str = 1X,A3,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A29,1X,A32
  agency_code      =  CWU
  ct_yy          =  0  
  ct_doy         =  0  
  ct_sec         =  0
  file_name        =              # file containing the solution (29 chars.)
  file_description =              # general description of the file (32 chars.)

[input_acknowledgements]  # optional
  fmt_str = 1X,A3,1X,A75
  agency_code = CWU
  agency_description = Geodesy Lab and PANGA Facility,Central Washington University

[nutation_data] # Mandatory for VLBI
  fmt_str = 1X,A8,1X,A70
  nutation_code =   # code nutation for reference
                    # IAU1980
                    # IERS1996
					# IAU2000a
					# IAU2000b
					
  comments      =   # general description of the nutation code used
  

[precession_data] # Mandatory for VLBI
   fmt_str  =  1X,A8,1X,A70
   precession_code =  # Code for precession reference
                      # IAU1976
					  # IERS1996

   comments        =  # General description of the nutation model used

[source_id] # Mandatory for VLBI
  fmt_str = 1X,A4,1X,A8,1X,A16,1X,A68
  source_code =   # call sign for a source
  iers_des    =   # IERS designation of the radio source
  icrf_des    =   # ICRF designation of the radio source
  comments    =   # comments or other names of the radio source

[site_id] # Mandatory
  fmt_str = 1X,A4,1X,A2,1X,A9,1X,A1,1X,A22,1X,I3,1X,I2,1X,F4.1,1X,F7.1
  site_code       =      # call sign for a site
  point_code      =  A   # physical monument used at a site
  monument_id     =      # 9 char. DOMES/DOMEX for the ITRF
  obs_code        =  P   # Observation technique
  sta_description =      # site desc., town and/or country
  lon_deg         = 
  lon_min         = 
  lon_sec         = 
  lat_deg         = 
  lat_min         = 
  lat_sec         = 
  height          = 

[site_data] # optional
  fmt_str = 1X,A4,1X,A2,1X,A4,1X,A4,1X,A2,1X,A4,1X,A1,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A3,1X,I2.2,1H:,I3.3,1H:,I5.5
  site_code_solved  =    # solved station site code
  point_code_solved = A  # 
  sol_id            = 
  site_code_input   =    # site code from input SINEX
  point_code_input  = A
  sol_id_input      = 
  obs_code_input    = P
  st_yy          =  0  
  st_doy         =  0  
  st_sec         =  0
  et_yy          =  0
  et_doy         =  0
  et_sec         =  0
  agency_code    =  CWU
  ct_yy          =  0  
  ct_doy         =  0  
  ct_sec         =  0

[site_receiver] # Mandatory for GPS
  fmt_str = 1X,A4,1X,A2,1X,A4,1X,A1,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A20,1X,A5,1X,A11
  site_code  = 
  point_code =  A
  sol_id     =  ----
  obs_code   =  P
  st_yy      =  0  
  st_doy     =  0  
  st_sec     =  0
  et_yy      =  0
  et_doy     =  0
  et_sec     =  0
  rcvr_type  = 
  rcvr_sn    = -----
  rcvr_fw    = -----------


[site_antenna] # Mandatory for GPS
  fmt_str = 1X,A4,1X,A2,1X,A4,1X,A1,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A20,1X,A5
  site_code = 
  point_code = A
  sol_id     = ----
  obs_code   = P 
  st_yy      = 0  
  st_doy     = 0  
  st_sec     = 0
  et_yy      = 0
  et_doy     = 0
  et_sec     = 0
  ant_type   = 
  ant_sn     = -----

[site_gps_phase_center] # Mandatory for GPS
  fmt_str =  1X,A20,1X,A5,1X,F6.4,1X,F6.4,1X,F6.4,1X,F6.4,1X,F6.4,1X,F6.4,1X,A10 
  ant_type = 
  ant_sn   = -----
  l1_u          =   # Up(+)    offset from the ARP to the L1 phase center
  l1_n          =   # North(+) offset from the ARP to the L1 phase center
  l1_e          =   # East(+)  offset from the ARP to the L1 phase center
  l2_u          =   # Up(+)    offset from the ARP to the L2 phase center
  l2_n          =   # North(+) offset from the ARP to the L2 phase center
  l2_e          =   # East(+)  offset from the ARP to the L2 phase center
  ant_cal_model = 

[site_eccentricity]
  fmt_str = 1X,A4,1X,A2,1X,A4,1X,A1,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A3,1X,F8.4,1X,F8.4,1X,F8.4
  site_code = 
  point_code = A
  sol_id     = ----
  obs_code   = P 
  st_yy      =  0  
  st_doy     =  0  
  st_sec     =  0
  et_yy      =  0
  et_doy     =  0
  et_sec     =  0
  ecc_ref    = UNE # local ref. Up,North,East
                   # XYZ cartesian ref. system
  u_x        = 
  n_y        = 
  e_z        = 

[bias_epochs] # Mandatory if bias parameters are included
  fmt_str = 1X,A4,1X,A2,1X,A4,1X,A1,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5
  site_code  = 
  point_code =   # satellite ID estimated biases: L1,L2 for LAGEOS-1 and -2 
  sol_id     =   # sequential number of the bias
  bias_type  =   # R  range bias
                 # T  time  bias
				 # S  scale bias
				 # Z  troposphere bias at zenith
  st_yy      =  0  
  st_doy     =  0  
  st_sec     =  0
  et_yy      =  0
  et_doy     =  0
  et_sec     =  0
  wmt_yy     =  0
  wmt_doy    =  0
  wmt_sec    =  0

[solution_discontinuity]
  fmt_str    =  1X,A4,1X,A2,1X,A4,1X,A1,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A1,1X,A1,1X,A
  site_code  = 
  point_code =  A 
  sol_id     = 
  obs_code   =  P 
  st_yy      =  0  
  st_doy     =  0  
  st_sec     =  0
  et_yy      =  0
  et_doy     =  0
  et_sec     =  0
  models     =    # ( P,V,E,A,S,...) 
                  # Position /Velocity (P/V)
                  # Periodic signal, annual (A), and semi-annual (S)
                  # Exponential Decay (E)
  axis       =    # (X,Y,Z,E,N,U,-)
                  # "-" means all axis (3D).
  comments   =
  
[solution_epochs] # Mandatory fmt_str = 1X,A4,1X,A2,1X,A4,1X,A1,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,I2.2,1H:,I3.3,1H:,I5.5
  site_code = 
  point_code = 
  sol_id     = 
  obs_code   = 
  st_yy      =  0  
  st_doy     =  0  
  st_sec     =  0
  et_yy      =  0
  et_doy     =  0
  et_sec     =  0
  mt_yy      =  0
  mt_doy     =  0
  mt_sec     =  0

[solution_statistics] # Recommended if available
  fmt_str = 1X,A30,1X,F22.15
  info_type =  # NUMBER OF OBSERVATIONS   
               # NUMBER OF UNKNOWNS     
			   # SAMPLING INTERVAL (SECONDS)
			   # SQUARE SUM OF RESIDUALS (VTPV)
			   # PHASE MEASUREMENTS SIGMA
			   # CODE MEASUREMENTS SIGMA
			   # NUMBER OF DEGREES OF FREEDOM
			   # VARIANCE FACTOR
			   # WEIGHTED SQUARE SUM OF O-C   
			   
   info     =  # relevant info of previous field

[solution_estimate] # Mandatory
  fmt_str = 1X,I5,1X,A6,1X,A4,1X,A2,1X,A4,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A4,1X,A1,1X,E21.15,1X,E11.6
  par_index =    # estimated parameter index
  par_type  = 
  site_code = 
  point_code = 
  sol_id     = 
  st_yy          =  0  
  st_doy         =  0  
  st_sec         =  0
  par_units  =     # m     (metres)
                   # m/y   (metres per year)
				   # m/s2  (metres per second**2)
				   # ppb   (parts per billion)
				   # ms    (milliseconds)
				   # msd2  (milliseconds per day**2)
				   # mas   (milli-arc-seconds)
				   # ma/d  (milli-arc-seconds/day)
				   # rad   (radians)
				   # rd/y  (radians per year)
				   # rd/d  (radians per day)
 
  constr_code  = 
  par_estimate = 
  par_stdv     = 


[solution_apriori] # Mandatory
   fmt_str = 1X,I5,1X,A6,1X,A4,1X,A2,1X,A4,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A4,1X,A1,1X,E21.15,1X,E11.6 
   par_index = 
   par_type  =  # TX,TY,TZ Translation on X,Y,Z direction            (m)
                # RX,RY,RZ Rotation around X,Y,Z axis                (mas)
				# SC scale                                           (ppb)
				# TXR,TYR,TZR Rate of translation in X,Y,Z direction (m/y)
				# RXR,RYR,RZR Rate of rotation around X,Y,Z axis     (mas/y)
				# SCR Rate for scale                                 (ppb/y)

   site_code    = 
   point_code   = 
   sol_id       = 
   st_yy        =  0  
   st_doy       =  0  
   st_sec       =  0
   par_units    =    
   constr_code  = 
   par_estimate = 
   par_stdv     = 

[solution_matrix_estimate_l_corr]
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =

[solution_matrix_estimate_l_cova] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =

[solution_matrix_estimate_l_info] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =

[solution_matrix_estimate_u_corr] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =



[solution_matrix_estimate_u_cova] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =


[solution_matrix_estimate_u_info] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =

[solution_matrix_apriori_l_corr]
  fmt_str = 1X,I5,1X,I5,1X,E21.16,1X,E21.16,1X,E21.16 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =

[solution_matrix_apriori_l_cova] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.16,1X,E21.16,1X,E21.16 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =

[solution_matrix_apriori_l_info] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.16,1X,E21.16,1X,E21.16 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =

[solution_matrix_apriori_u_corr] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.16,1X,E21.16,1X,E21.16 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =



[solution_matrix_apriori_u_cova] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.16,1X,E21.16,1X,E21.16 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =


[solution_matrix_apriori_u_info] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.16,1X,E21.16,1X,E21.16 
  mat_est_row =
  mat_est_col =
  mat_est_1   =
  mat_est_2   =
  mat_est_3   =




[solution_normal_equation_vector] # Mandatory
  fmt_str = 1X,I5,1X,A6,1X,A4,1X,A2,1X,A4,1X,I2.2,1H:,I3.3,1H:,I5.5,1X,A4,1X,A1,1X,E21.15
  par_index =
  par_type  =
  site_code =
  point_code =
  sol_id     = 
  st_yy          =  0  
  st_doy         =  0  
  st_sec         =  0
  par_units  =
  constr_code = 
  rhs_ne      = 

[solution_normal_equation_matrix_l] # Mandatory
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14
  neq_row = 
  neq_col = 
  mat_element_1 = 
  mat_element_2 = 
  mat_element_3 = 

[solution_normal_equation_matrix_u]
  fmt_str = 1X,I5,1X,I5,1X,E21.14,1X,E21.14,1X,E21.14
  neq_row = 
  neq_col = 
  mat_element_1 = 
  mat_element_2 = 
  mat_element_3 = 

[END] #end of config information. Man page follows


=head1 NAME

B<GPS::SINEX>

=head2 Reads/Writes formatted SINEX (Solution INdependent EXchange Format) block lines

=head1 SYNOPSYS

use GPS::SINEX;

my $snx = GPS::SINEX->new();

#set version field in header block

$snx->header( version => 2.01 );

#set info_type and info in file_reference block

$snx->file_reference( info_type => 'DESCRIPTION', info => 'Made at CWU' );

#to get the formatted content just call the method with no arguments

print $snx->header;

print $snx->footer;

#read a line from a existing SINEX file

open(my $fh,'<',$mysnx) or die "$!\n";

my $hline = <$fh>;

#set the whole header with the new string

$snx->header($hline);


=head1 DESCRIPTION

B<GPS::SINEX> reads/writes formatted SINEX lines using AUTOLOAD to automatically
generate the appropriate method for each block. The I<print_config> function
allows to print the field names for each block to be able to set the values for
each of them. To print the I<config> information from the command line use:

perl -MGPS::SINEX -e '$snx = GPS::SINEX->new(); $snx->print_config;'

=head1 METHODS

=over

=item B<print_config>

$snx->print_config;

print I<config> information to STDOUT

=head1 VERSION

0.1

=head1 AUTHOR

Victor Marcelo Santillan E<lt>marcelo@geology.cwu.edu<gt>

=head1 COPYRIGHT

Copyright (c) 2008 Central Washington University. 


=cut








