package GPS::SiteLog;


sub new {
   my $class = shift;
   my $self = { file => '',
                sections => {}
              };
   
   bless $self,$class;
   
   if ( $self->file(@_) ) {
     $self->parse;
   }
   else {
     return $self
   }
}

sub file {
  my $self = shift;
  if ( @_ ) {
     my $f = shift;
     if ( -e $f ) {
        $self->{file} = $f;
     }
     else {
       die "GPS::SiteLog: file $f doesn't exists\n";
     }
  }
  return $self->{file}
}


my $rgx_header =qr{([A-Z0-9_]{4})\s+Site Information Form};
my $rgx_multiline = qr{(Distance/activity|Additional Information|Notes|Agency|Mailing Address|Marker Description)};
my $rgx_no_colon =  qr{(Approximate Position \(ITRF\)|Differential Components from GNSS Marker to the tied monument \(ITRS\)|Primary Contact|Secondary Contact|Hardcopy on File|Antenna Graphics with
Dimensions|If Update:)};
my $rgx_is_section = qr{^(\d{1,2})\.\s+(.+)$}; 
my $rgx_is_sub_section = qr{^(\d{1,2}(\.\d{1,2})*)\s+([A-Z].+)\s+:\s+(.*$)};
my $rgx_is_sub_section_x = qr{^\d{1,2}\.(\d{1,2}\.)*x\s+[A-Z]};
my $rgx_antgraphs = qr{Antenna Graphics with Dimensions|insert text graphic from file antenna\.gra};
my $rgx_is_sub_section_field = qr{^([A-Za-z4].+?)\s*?:\s*(.*)$};
sub parse {
   my $self = shift;
   my $fname = $self->file;
   open(my $fh,'<',$fname) or die "Couldn't open $fname $!\n";
   my $sections = $self->{sections};
   my $line=''; 
   my $antgraphs=0;
   my $ss_x = 0;
   my ($n,$sname,$ssname,$ssval,$multiline,$header,%seen,$i);
   while ( <$fh> ) {
      chomp;
      $line=$_;
      s/^\s+//;
      s/\s+$//;
      if ( /$rgx_antgraphs/ ) {
         $antgraphs=1;
      }
      if ( $antgraphs ) {
         push @{$sections->{antgraph}},$line;
         #print "$line\n";
         next;
      }
      next unless length;
      if ( /$rgx_header/ ) {
          $sections->{header}{site} = $1;
          $header = 1;
      }
      if ( /$rgx_is_section/ ) {
        %seen=();
        $header=0;
        $multiline=0;
        $no_colon=0;
        $ss_x = 0;
        $n = $1;
        $sn = undef;
        $sname=$2;
        #print "section:|$n|$sname|\n";
        $sections->{$n}{section_name}=$sname;
      } 
      if ( /$rgx_is_sub_section/ ) {
         %seen=();
         $multiline=0;
         $no_colon=0;
         $sn =  $1 ;
         $sn = $n if ! $sn;
         $ssname = $3;
         $ssval = $4;
         $ss_x = 0;            
         #print "sub section:|$sn|$ssname|$ssval|\n";
         $sections->{$sn}{sub_section_name}=$ssname;
         $sections->{$sn}{$ssname}=$ssval;
         next;
      }
      if ( /$rgx_is_sub_section_x/ ) {
        $multiline=0;
        $no_colon=0;
        $ss_x =1;
      }
      if ( /$rgx_no_colon/ ) {
         $multiline=0;
         next;
      }
      next if $header;
      if ( ! $ss_x ) {
         $sn = $n if ! $sn;
         $multiline=0;
        if ( /$rgx_is_sub_section_field/ ) {
          $ssname=$1;
          $ssval = $2 || '';
          if ( /$rgx_multiline/ ) {
             $multiline=1;
             push @{$sections->{$sn}{$ssname}},$ssval;
          }
          else {
            #print "sub section field:|$sn|$ssname|$ssval|\n";
            if ( exists $sections->{$sn}{$ssname} ) {
                $i = sprintf(" %02d",++$seen{$ssname}+1 );
                $sections->{$sn}{$ssname.$i} = $ssval; 
            }
            else {
              $sections->{$sn}{$ssname}=$ssval;
            }
          }
        } 
        elsif ( $multiline ) {
           ($ssval = $_) =~ s/^:\s+//;
           #print "sub section field:|$sn|$ssname|$ssval|\n";
           push @{$sections->{$sn}{$ssname}},$ssval;

        }
      }
      else {
        next;
      }
      
   }
   return $self
}

1;
