package GPS::TimeSeries;
use GPS::DATES;
use XML::Simple;
use GOA::Util;
use Scalar::Util qw(looks_like_number);
use POSIX qw(ceil floor);
my $JAN_01_1970=-946728000000; 
my %Par =(
            YZERO => 'y_zero',
            SLOPE => 'vel',
            'ANNUAL AMPLITUDE' => 'a_amp',
            'SEMI-ANNUAL AMPLITUDE' => 'sa_amp',
            'ANNUAL PHASE'     => 'a_phase',
            'SEMI-ANNUAL PHASE'     => 'sa_phase',
            'AMPANNUAL SIN'         => 'aas',
            'AMPSEMI-ANNUAL SIN'    => 'ass',
            'AMPANNUAL COS'         => 'aac',
            'AMPSEMI-ANNUAL COS'    => 'asc',
            'OFFSET'                => 'jmp',
            'OFFSET-EVENT'          => 'evt',
);
my %Dtype=( CLEANED_DATA => 'data', LDETRENDED => 'ldet', RESIDUALS => 'resd');
our $AUTOLOAD;
our %ok_field;
my @info_fields = qw( npar chi2_dof 
                      site comp dtype 
                      max_sig max_res
                      n_tot n 
                      ti tf tspan t0 t_unit dt
                      rms std_dev mean y_unit
                      data 
			        );

my @param_fields = qw( y_zero    y_zero_sig 
                       vel       vel_sig
                       a_amp     a_amp_sig 
                       sa_amp    sa_amp_sig 
                       a_phase   a_phase_sig
                       sa_phase  sa_phase_sig
                       aas       aas_sig
                       aac       aac_sig
                       ass       ass_sig
                       asc       asc_sig
                     );
                
for ( @fields      ) { $ok_field{$_}++; } 
for ( @info_fields ) { $ok_field{$_}++; } 

sub AUTOLOAD {
  my $self = shift;
  my $attr = $AUTOLOAD;
  $attr =~ s/.*:://;   # strip fully-qualified portion
  return unless $attr =~ /[^A-Z]/; #skip destroy and all caps methods
  die "Invalid attribute method: ->$attr( )\n" if ! $ok_field{$attr};
  
  if ( exists $self->{$attr} ) {
    $self->{$attr} = shift if @_;
    return $self->{$attr};  
  }
  
  foreach my $h ( @{$self->{parameters}} ) {
    if (  $h->{name} eq $attr ) {
       ( $h->{value} ,$h->{sig} )= @_ if @_;
       return wantarray ? ($h->{value},$h->{sig}) : $h->{value} ;  
    }
  }
  
}


sub t {
  my $self = shift;
  return  map { $_->[0] } @{$self->data}
}
sub x {
  my $self = shift;
  return  map { $_->[1] } @{$self->data}
}
sub sig {
  my $self = shift;
  return  map { $_->[2] } @{$self->data}
}

sub parameters {
  my $self = shift;
  return wantarray ? @{$self->{parameters}} : $self->{parameters}
}

sub data {
  my $self = shift;
  return wantarray ? @{$self->{data}} : $self->{data};
}

sub new {
   my $what = shift;
   my $class= ref($what) || $what;
   my $self={ header => 0 };
   @$self{@info_fields}=(0)x@info_fields;
   bless $self,$class;
   $self->file(shift);
   $self->header(@_);
   return $self->load()
}
sub header {
  my $self = shift;
  if ( @_ ) {
    my $h = shift;
    $self->{header} = 1 if $h;
  }
  return $self->{header}
}

sub file {
 my $self = shift;
 if ( @_ ) {
   my $f = shift;
   die "file $f doesn't exists\n" if ! -e $f;
   $self->{file} = $f;
 }
 return $self->{file}
}

sub load {
  my $self = shift;
  if ( @_ ) {
    $self->{header}=1;
  }
  my $file = $self->file;
  open(my $fh,'<',$file) or die "Couldn't open $file $!\n";
  my $i=0;
  while( <$fh> ) {
     chomp;
     #s/\*+/0/g; 
     if ( /N=\s+(\d+)\s+PARAMETERS/ ) {   
         $self->{npar} = $1;
     }
     elsif ( /CHI.2.DOF=\s*(.*)$/ ) {
         $self->{chi2_dof} = $1+0;
     }
     elsif ( /OUTPUT:\s*([A-Z_]+)/ ) {
         $self->{dtype} = $Dtype{$1};
     }
     elsif ( /INPUT FILE:.*(....)\.(lat|lon|rad|n|e|u|x|y|z)/ ) {
         $self->{site} = uc $1;
         $self->{comp} = lc $2;
     }
     elsif ( /TOTAL OBSERVATIONS MTOT=\s+(\d+)/ ) {
        $self->{n_tot} = $1;
     }
     elsif ( /OBSERVATIONS: USED M=\s+(\d+)\s/ ) {
        $self->{n} = $1;
     }
	 elsif ( /MAXSIG=(.*)/ ) {
	    $self->{max_sig} = trim($1);
	 }
	 elsif ( /MAXRES=(.*)/ ) {
	    $self->{max_res} = trim($1);
	 }
     elsif ( /TIME INTERVAL=\s(.*?)\s-\s(.*?)\s=\s(.*)/ ) {
        $self->{tf}=$1;
        $self->{ti}=$2;
        $self->{tspan}=$3;
        #print map { "$_ $self->{$_}\n" } qw( ti tf tspan );
     }
     elsif ( /RMS=(.*?)SDEV=(.*?)MEAN=(.*?)$/ ) {
        $self->{rms}=trim($1);
        $self->{std_dev}=trim($2);
        $self->{mean}=trim($3);
     }
     elsif (
     /P\s*(\d+)\s=\s+([\-\d\.]+)\s\+\-\s([\d\.\*\s]+)\s+([\d\.\*]+)\s([A-Z\- ]+[A-Z]*)([\d\.\s]+)/ ) {
        #print "|$_|\n";
        my $p  = $1;
        my $pn = $5;
        my $par=$Par{trim($5)};
        my $val   = $2+0;
        my $sig   = $3+0; 
        my $err   = $4;
        my $epoch = trim($6)|| 0;
        #print "|$pn|$par|$p|$val|$sig|$err|$epoch|\n";
        #   $self->{"${par}$epoch"}= $val =~ /\*+/ ? 0 : $val+0;
        #   $self->{"${par}_sig$epoch"}= $sig =~ /\*+/ ? 0 : $sig+0;
		my $hp = { name => $par , 
		           value=> $val ,
				   sig => $sig,
				   epoch => $epoch
				 };
		push @{$self->{parameters}},$hp;
     }
     
     s/\s+$//;
     s/^\s+//;
     s/#.*$//;
     
     next unless length;
     last if $self->{header};
     if ( $i == 0 ) {
       ($self->{t0}) = split(' ');
     } 
     @{$self->{data}[$i++]} = split ' ',$_;

  }
  return $self
}
sub stats {
  my $self = shift;
  my $n = @{$self->{data}};
  my ($mean,$stdv,$rms);
  my $sum = 0;
  foreach ( $self->data ) {
    $sum += $_->[1];
  }
  $mean=$sum/$n;
  $sum = 0;
  foreach my $x ( $self->data ) {
     $sum += ($x->[1] - $mean)*($x->[1]-$mean);
  }
  $stdv = sqrt($var/($n-1));
  $rms=sqrt($sum*($n-1)/$n+$mean*$mean);

  return($mean,$stdv,$rms)
}
sub autocorr {
  my $self = shift;
  my $n = @{$self->{data}};
  my $order = shift || $n;
  my @R=();
  my ($sum);
  my $x = $self->{data};
  for( my $i=0; $i < $order+1 ; $i++) {
    $sum = 0;
    for ( my $j = 0; $j < $n - $i ; $j++) {
           $sum += $x->[$j] * $x->[$j+$i];
    }
    $R[$i] = $sum;
  } 
  return wantarray ? @R : \@R;  
}

sub events {
   my $self = shift;
   return $self->jumps('evt');
}
sub jumps {
   my $self = shift;
   my $type = shift || 'jmp';
   my @jumps =();
   foreach my $k ( grep { /^${type}_sig/ } keys %$self ) {
       my $t = $k ; $t =~ s/${type}_sig//;
       push @jumps,{ epoch =>$t, 
                     val => $self->{"$type$t"}, 
                     sig =>$self->{"${type}_sig$t"} 
                   };
   }
   return wantarray ? @jumps : \@jumps
}

sub histogram {
  my $self = shift;
  my %h = ();
  my $x=0;
  my $n=$self->{n};
  for ( @{$self->{data}} ) {
     $x=sprintf("%.1f",abs($_->[1]));
     $h{$x}++;
  }
  #my @h = map { [$_, $h{$_}*100/$n] } sort { $a <=> $b } keys %h;
  my @h = map { [$_, $h{$_}] } sort { $a <=> $b } keys %h;
  return wantarray ? @h : \@h; 
}

sub min_max {
   my $self = shift;
   my $data = $self->{data};
   
   my ( $tmin,$tmax) = ( sort { $a <=> $b } map { $_->[0] } @$data)[0,-1];
   my ( $xmin,$xmax) = ( sort { $a <=> $b } map { $_->[1] } @$data)[0,-1];
   my ( $sxmin,$sxmax) = ( sort { $a <=> $b } map { $_->[2] } @$data)[0,-1];

  return ($tmin,$tmax,$xmin,$xmax,$sxmin,$sxmax)   
}

sub hist_error {
   my $ts = shift;
   my $N=0;
   my $n=$ts->{n};
   my $ns = 0.7*$n;
   my ($xmin,$xmax)=($ts->min_max)[2,3];
   my $dx = $xmax - $xmin;
   my $sigma = $dx/2*0.7;
   my ($v,$nb);
   foreach my $h ( $ts->histogram ) {
     $N+=$h->[1];
     last if ($N > $ns); # && ($h->[0] > $sigma);
     ($v,$nb)=@$h;
     #print "$v $nb\n";
   }
   
   my $tspan = $ts->tspan;
   my $tv = $v/$tspan;
   return $tv
}

sub print {
  my $self = shift;
  foreach ( $self->data ) {
    print "@{$_}\n";
  }

}

sub xml {
  my $self = shift;
  my $xml = qq(<?xml version="1.0" encoding="UTF-8"?>
<TimeSeries>
);
 my $h = {};
 $h->{info}={};
 $h->{parameters}={};
 $h->{data} = $self->{data};
 my @info =  qw( chi2_dof comp dtype mean 
              n n_tot npar rms 
			  site std_dev
			  ti tf tspan ymax ymin 
			);
 my ($tmin,$tmax,$xmin,$xmax,$sxmin,$sxmax) = $self->min_max;
 @{$h->{info}}{@info} = @{$self}{@info}; 
 $h->{info}{ymin}=$xmin;
 $h->{info}{ymax}=$xmax;
 
 $xml .= XMLout($h->{info},RootName => 'info',AttrIndent => 1);

 $xml .= XMLout($self->{parameters},RootName => 'parameters',AttrIndent => 0);

 $xml =~ s/<anon /<parameter /g;
 $xml .= "<points>\n";
 foreach my $a ( @{$h->{data}} ) {
    $xml .= sprintf('<point epoch="%s" value="%s" sigma="%s" />',@$a)."\n";
 }
 $xml .= "</points>\n";
 $xml .= "</TimeSeries>\n";
print $xml;

}

sub json_time {
  my $t = ceil(get_date(shift,'secj2000'));
  return $t*1000-$JAN_01_1970;
}
sub json {
  my $self = shift;
  my @ifields=();
  my @param=();
  my ($name,$val,$sig,$ep)=('')x4;
  $self->t_unit("milliseconds");
  my ($t,$t0,$ti,$tf);
  $t0 = $self->data->[0][0];
  $self->t0( json_time($self->t0) );
  $self->ti( json_time($self->ti) );
  $self->tf( json_time($self->tf) );
  $self->tspan($self->ti -$self-tf);
  foreach my $f ( @info_fields ) {
      next if $f eq 'data';
      #print "info |$f| $self->{$f}\n";
      $val = $self->{$f};
      $val = looks_like_number($val) ? $val : "\"$val\"";
      push @ifields ,"\"$f\":$val";
  }

  my(@jumps,@events);
  my $json = '{';
  $json .= '"info":{'. join(',',@ifields) . '}';
  $json .= ',';
  foreach my $h ( $self->parameters ) {
      $val=$h->{value} ;
      $sig=$h->{sig};
      $ep = $h->{epoch}; 
      $ep = json_time($h->{epoch}) if $ep != 0.0;
      $name =$h->{name}; 
      if ( $name eq 'jmp' ) {
         push @jumps,"{\"val\":$val,\"sig\":$sig,\"epoch\":$ep}";
         next;
      }
      if ( $name eq 'evt' ) {
         push @events,"{\"val\":$val,\"sig\":$sig,\"epoch\":$ep}";
         next;
      }
      push @param ,"\"$name\":{\"val\":$val,\"sig\":$sig}";
  }
  if ( @events ) {
     push @param,"\"offset_evt\":[".join(',',@events)."]"; 
  }
  if ( @jumps ) {
     push @param,"\"offset\":[".join(',',@jumps)."]"; 
  }
  $json .= '"param":{'. join(',',@param) . '}';
  $json .= ',';
  $json .= '"data":[';
  my $flag=0;
  my ($sig,$sigp,$val,$valp,$sign,$ival,$isig);
  my $tp = $self->t0; 
  my $dt = $self->dt + 68000;
  foreach my $p ( $self->data ) {
     if ( $flag ) {
        $json .= ',';
     }
     else {
       $flag++;
     }
     #$json .= "{x:$p->[0],y:$p->[1],ysig:$p->[2]}";
     #$json .= sprintf("[%.5g,%.4g,%.2g]",($p->[0]-$tp)/$dt,$p->[1],$p->[2]);
     next if $p->[2] == 0.0;
     #$sig = ceil(log($p->[2])/$base);
     #$val = ceil(log(abs($p->[1]))/$base);
     $t = json_time($p->[0]);
     if ( ($t-$tp ) > $dt ) {
       #print "$p->[0] $t $tp ".($t-$tp)." ". ($t-$tp)/$dt."\n";
       $json .= sprintf("[%d,null],",ceil(($t-$tp)/$dt));
     }
     $json .= sprintf("[%.2f,%.1f]",$p->[1],$p->[2] );
                                    
     $tp = $t;
  }
  $json .=']';

  $json .='}';
  return $json;
  print $json,"\n";
  exit;
}

1;
